@@ -301,7 +301,7 @@
     * @param mixed $options
     * @return array, see evaluate for return information. ['result'] is whatever the application returns, or -2 on failure to find application
     */
-    function exec($application, $options)
+    function exec($application, $options = array())
     {
       if(is_array($options)) $options = join(',', $options);
       return $this->evaluate("EXEC $application $options");
@@ -870,7 +870,7 @@
      $proceed = false;
      if($escape_digits != '' && $buffer != '')
      {
-       if(!strpos(chr(255) . $escape_digits, $buffer{strlen($buffer)-1}))
+       if(!strpos(chr(255) . $escape_digits, $buffer[strlen($buffer)-1]))
          $proceed = true;
      }
      if($buffer == '' || $proceed)
@@ -880,7 +880,7 @@
          $buffer .= chr($res['result']);
        return $res;
      }
-     return array('code'=>AGIRES_OK, 'result'=>ord($buffer{strlen($buffer)-1}));
+     return array('code'=>AGIRES_OK, 'result'=>ord($buffer[strlen($buffer)-1]));
    }
 
    /**
@@ -899,7 +899,7 @@
      $proceed = false;
      if($escape_digits != '' && $buffer != '')
      {
-       if(!strpos(chr(255) . $escape_digits, $buffer{strlen($buffer)-1}))
+       if(!strpos(chr(255) . $escape_digits, $buffer[strlen($buffer)-1]))
          $proceed = true;
      }
      if($buffer == '' || $proceed)
@@ -909,7 +909,7 @@
          $buffer .= chr($res['result']);
        return $res;
      }
-     return array('code'=>AGIRES_OK, 'result'=>ord($buffer{strlen($buffer)-1}));
+     return array('code'=>AGIRES_OK, 'result'=>ord($buffer[strlen($buffer)-1]));
    }
 
    /**
@@ -928,7 +928,7 @@
      $proceed = false;
      if($escape_digits != '' && $buffer != '')
      {
-       if(!strpos(chr(255) . $escape_digits, $buffer{strlen($buffer)-1}))
+       if(!strpos(chr(255) . $escape_digits, $buffer[strlen($buffer)-1]))
          $proceed = true;
      }
      if($buffer == '' || $proceed)
@@ -938,7 +938,7 @@
          $buffer .= chr($res['result']);
        return $res;
      }
-     return array('code'=>AGIRES_OK, 'result'=>ord($buffer{strlen($buffer)-1}));
+     return array('code'=>AGIRES_OK, 'result'=>ord($buffer[strlen($buffer)-1]));
    }
 
    /**
@@ -957,7 +957,7 @@
      $proceed = false;
      if($escape_digits != '' && $buffer != '')
      {
-       if(!strpos(chr(255) . $escape_digits, $buffer{strlen($buffer)-1}))
+       if(!strpos(chr(255) . $escape_digits, $buffer[strlen($buffer)-1]))
          $proceed = true;
      }
      if($buffer == '' || $proceed)
@@ -967,7 +967,7 @@
          $buffer .= chr($res['result']);
        return $res;
      }
-     return array('code'=>AGIRES_OK, 'result'=>ord($buffer{strlen($buffer)-1}));
+     return array('code'=>AGIRES_OK, 'result'=>ord($buffer[strlen($buffer)-1]));
    }
 
    /**
@@ -989,7 +989,7 @@
      $proceed = false;
      if($escape_digits != '' && $buffer != '')
      {
-       if(!strpos(chr(255) . $escape_digits, $buffer{strlen($buffer)-1}))
+       if(!strpos(chr(255) . $escape_digits, $buffer[strlen($buffer)-1]))
          $proceed = true;
      }
      if($buffer == '' || $proceed)
@@ -999,7 +999,7 @@
          $buffer .= chr($res['result']);
        return $res;
      }
-     return array('code'=>AGIRES_OK, 'result'=>ord($buffer{strlen($buffer)-1}), 'endpos'=>0);
+     return array('code'=>AGIRES_OK, 'result'=>ord($buffer[strlen($buffer)-1]), 'endpos'=>0);
    }
 
    /**
@@ -1018,7 +1018,7 @@
      $proceed = false;
      if($escape_digits != '' && $buffer != '')
      {
-       if(!strpos(chr(255) . $escape_digits, $buffer{strlen($buffer)-1}))
+       if(!strpos(chr(255) . $escape_digits, $buffer[strlen($buffer)-1]))
          $proceed = true;
      }
      if($buffer == '' || $proceed)
@@ -1028,7 +1028,7 @@
          $buffer .= chr($res['result']);
        return $res;
      }
-     return array('code'=>AGIRES_OK, 'result'=>ord($buffer{strlen($buffer)-1}), 'endpos'=>0);
+     return array('code'=>AGIRES_OK, 'result'=>ord($buffer[strlen($buffer)-1]), 'endpos'=>0);
    }
 
    /**
@@ -1047,7 +1047,7 @@
      $proceed = false;
      if($escape_digits != '' && $buffer != '')
      {
-       if(!strpos(chr(255) . $escape_digits, $buffer{strlen($buffer)-1}))
+       if(!strpos(chr(255) . $escape_digits, $buffer[strlen($buffer)-1]))
          $proceed = true;
      }
      if($buffer == '' || $proceed)
@@ -1057,7 +1057,7 @@
          $buffer .= chr($res['result']);
        return $res;
      }
-     return array('code'=>AGIRES_OK, 'result'=>ord($buffer{strlen($buffer)-1}), 'endpos'=>0);
+     return array('code'=>AGIRES_OK, 'result'=>ord($buffer[strlen($buffer)-1]), 'endpos'=>0);
    }
 
    /**
@@ -1075,7 +1075,7 @@
      $proceed = false;
      if($escape_digits != '' && $buffer != '')
      {
-       if(!strpos(chr(255) . $escape_digits, $buffer{strlen($buffer)-1}))
+       if(!strpos(chr(255) . $escape_digits, $buffer[strlen($buffer)-1]))
          $proceed = true;
      }
      if($buffer == '' || $proceed)
@@ -1085,7 +1085,7 @@
          $buffer .= chr($res['result']);
        return $res;
      }
-     return array('code'=>AGIRES_OK, 'result'=>ord($buffer{strlen($buffer)-1}));
+     return array('code'=>AGIRES_OK, 'result'=>ord($buffer[strlen($buffer)-1]));
    }
 
    /**
@@ -1173,7 +1173,7 @@
       {
         foreach($choices as $prompt)
         {
-          if($prompt{0} == '*')
+          if($prompt[0] == '*')
             $ret = $this->text2wav(substr($prompt, 1), $keys);
           else
             $ret = $this->stream_file($prompt, $keys);
@@ -1236,9 +1236,9 @@
       $ret = array('name'=>'', 'protocol'=>'', 'username'=>'', 'host'=>'', 'port'=>'');
       $callerid = trim($callerid);
 
-      if($callerid{0} == '"' || $callerid{0} == "'")
+      if($callerid[0] == '"' || $callerid[0] == "'")
       {
-        $d = $callerid{0};
+        $d = $callerid[0];
         $callerid = explode($d, substr($callerid, 1));
         $ret['name'] = array_shift($callerid);
         $callerid = join($d, $callerid);
@@ -1420,7 +1420,7 @@
         {
           if($command)
           {
-            switch($code{0})
+            switch($code[0])
             {
               case '2': $text = substr($text, 0, strlen($text) - 1); break; // backspace
               case '5': $mode = 'LOWERCASE'; break;
@@ -1460,7 +1460,7 @@
     {
       for($i = 0; $i < strlen($text); $i++)
       {
-        switch($text{$i})
+        switch($text[$i])
         {
           case ' ': $ret .= 'SPACE ';
           case ',': $ret .= 'COMMA '; break;
@@ -1495,7 +1495,7 @@
           case '|': $ret .= 'BAR '; break;
           case '_': $ret .= 'UNDERSCORE '; break;
           case '~': $ret .= 'TILDE '; break;
-          default: $ret .= $text{$i} . ' '; break;
+          default: $ret .= $text[$i] . ' '; break;
         }
       }
       return $this->text2wav($ret, $escape_digits, $frequency);
@@ -1564,7 +1564,7 @@
       $ret['code'] = substr($str, 0, 3);
       $str = trim(substr($str, 3));
 
-      if($str{0} == '-') // we have a multiline response!
+      if($str[0] == '-') // we have a multiline response!
       {
         $count = 0;
         $str = substr($str, 1) . "\n";
@@ -1599,14 +1599,14 @@
           if($in_token) // we previously hit a token starting with '(' but not ending in ')'
           {
 	    $tmp = trim($token);
-	    $tmp = $tmp{0} == '(' ? substr($tmp,1):$tmp;
+	    $tmp = $tmp[0] == '(' ? substr($tmp,1):$tmp;
 	    $tmp = substr($tmp,-1) == ')' ? substr($tmp,0,strlen($tmp)-1):$tmp;
 	    $ret['data'] .= ' ' . trim($tmp);
-            if($token{strlen($token)-1} == ')') $in_token = false;
+            if($token[strlen($token)-1] == ')') $in_token = false;
           }
-          elseif($token{0} == '(')
+          elseif($token[0] == '(')
           {
-            if($token{strlen($token)-1} != ')') $in_token = true;
+            if($token[strlen($token)-1] != ')') $in_token = true;
 	    $tmp = trim(substr($token,1));
 	    $tmp = $in_token ? $tmp : substr($tmp,0,strlen($tmp)-1);
 	    $ret['data'] .= ' ' . trim($tmp);
@@ -1768,15 +1768,15 @@
       $ret = '';
       for($i = 0; $i < strlen($message); $i++)
       {
-        $c = ord($message{$i});
+        $c = ord($message[$i]);
         if($c == 10 || $c == 13 || $c == 9)
-          $ret .= $message{$i};
+          $ret .= $message[$i];
         elseif($c < 16)
           $ret .= '\x0' . dechex($c);
         elseif($c < 32 || $c > 127)
           $ret .= '\x' . dechex($c);
         else
-          $ret .= $message{$i};
+          $ret .= $message[$i];
       }
       $message = $ret;
 
