@@ -55,6 +55,7 @@ header('Expires: Sat, 01 Jan 2000 00:00:00 GMT');
 header('Cache-Control: post-check=0, pre-check=0',false);
 header('Pragma: no-cache');
 header('Content-Type: text/html; charset=utf-8');
+header('X-Frame-Options: SAMEORIGIN');
 //header("Content-Security-Policy: default-src 'none';");
 
 // This needs to be included BEFORE the session_start or we fail so
@@ -712,13 +713,13 @@ if ($quietmode) {
 	}
 
 	//send footer
-	$o = FreePBX::create()->View->getScripts();
+	$o = FreePBX::View()->getScripts();
 	$footer['compiled_scripts'] = $o;
 	$footer['js_content'] = load_view($amp_conf['VIEW_POPOVER_JS'], $popover_args);
 	$footer['lang'] = $language;
 	$footer['covert'] 		= in_array($display, array('noauth', 'badrefer')) ? true : false;
 	$footer['extmap'] 				= !$footer['covert']
-		? framework_get_extmap(true)
+		? json_encode(FreePBX::Extensions()->getExtmap())
 		: json_encode(array());
 	$footer['module_name'] = $module_name;
 	$footer['module_page'] = $module_page;
@@ -822,7 +823,7 @@ if ($quietmode) {
 	$footer['compiled_scripts'] = $o;
 	$footer['lang'] = $language;
 	$footer['covert'] 		= in_array($display, array('noauth', 'badrefer')) ? true : false;
-	$footer['extmap'] 				= !$footer['covert'] ? framework_get_extmap(true) : json_encode(array());
+	$footer['extmap'] 				= !$footer['covert'] ? json_encode(FreePBX::Extensions()->getExtmap()) : json_encode(array());
 	$footer['module_name']			= $module_name;
 	$footer['module_page']			= $module_page;
 	$footer['benchmark_starttime']	= $benchmark_starttime;
