@@ -499,7 +499,7 @@ class GPG {
 			$fds[0] = $stdin;
 		}
 
-		$webuser = \FreePBX::Freepbx_conf()->get('AMPASTERISKWEBUSER');
+		$webuser = \FreePBX::Config()->get('AMPASTERISKWEBUSER');
 		$gpgdir = $this->getGpgLocation();
 		$homediropt = "--homedir $gpgdir";
 		$home = preg_replace('/\/\.gnupg$/', '', $gpgdir);
@@ -686,18 +686,20 @@ class GPG {
 		}
 
 		$out = $this->runGPG("--output - ".escapeshellarg($sigfile));
-
+		
 		// Check to see if we don't know about this signature..
-		if (isset($out['status'][2]) && preg_match('/NO_PUBKEY (.+)/', $out['status'][2], $keyarr)) {
-			// We don't. Try to grab it.
-			try {
-				$this->getKey($keyarr[1]);
-			} catch (\Exception $e) {
-				// Couldn't download the key.
-				return array("status" => self::STATE_INVALID);
+		for($i=1;$i<count($out['status']);$i++) {
+			if ( preg_match('/NO_PUBKEY (.+)/', $out['status'][$i], $keyarr) ) {
+				// We don't. Try to grab it.
+				try {
+					$this->getKey($keyarr[1]);
+				} catch (\Exception $e) {
+					// Couldn't download the key.
+					return array("status" => self::STATE_INVALID);
+				}
+				// And now run the validation again.
+				$out = $this->runGPG("--output - ".escapeshellarg($sigfile));
 			}
-			// And now run the validation again.
-			$out = $this->runGPG("--output - ".escapeshellarg($sigfile));
 		}
 
 		$status = $this->checkStatus($out['status']);
@@ -773,7 +775,7 @@ class GPG {
 
 	public function getGpgLocation() {
 		// Re #7429 - Always use the AMPASTERISKWEBUSER homedir for gpg
-		$webuser = \FreePBX::Freepbx_conf()->get('AMPASTERISKWEBUSER');
+		$webuser = \FreePBX::Config()->get('AMPASTERISKWEBUSER');
 
 		if (!$webuser) {
 			throw new \Exception(_("I don't know who I should be running GPG as."));
@@ -788,7 +790,7 @@ class GPG {
 		if (!is_dir($home)) {
 			// Well, that's handy. It doesn't exist. Let's use ASTSPOOLDIR instead, because
 			// that should exist and be writable.
-			$home = \FreePBX::Freepbx_conf()->get('ASTSPOOLDIR');
+			$home = \FreePBX::Config()->get('ASTSPOOLDIR');
 			if (!is_dir($home)) {
 				// OK, I give up.
 				throw new \Exception(sprintf(_("Asterisk home dir (%s) doesn't exist, and, ASTSPOOLDIR doesn't exist. Aborting"),$home));
@@ -843,7 +845,7 @@ class GPG {
 		}
 
 		// Now, who should be running gpg normally?
-		$freepbxuser = \FreePBX::Freepbx_conf()->get('AMPASTERISKWEBUSER');
+		$freepbxuser = \FreePBX::Config()->get('AMPASTERISKWEBUSER');
 		$pwent = posix_getpwnam($freepbxuser);
 		$uid = $pwent['uid'];
 		$gid = $pwent['gid'];
