@@ -16,7 +16,7 @@ class SystemUpdates {
 	public function __construct($cli = false) {
 		$this->cli = $cli;
 		// Can't use functions in class definitions
-		$this->strarr = [ "complete" => _("(Complete)"), "unknown" => _("(Unknown)"), "inprogress" => _("(In Progress)"), "yumerror" => _("(Yum Error)"), "error" => _("General Error") ];
+		$this->strarr = [ "complete" => _("(Complete)"), "unknown" => _("(Unknown)"), "inprogress" => _("(In Progress)"), "yumerror" => _("(YUM Error)"), "error" => _("General Error") ];
 	}
 
 	public function __destruct() {
@@ -61,7 +61,8 @@ class SystemUpdates {
 			return false;
 		}
 		$lic = sysadmin_get_license();
-		return (isset($lic['hostid']));
+		//machineid is used since PHP 7+. for the earlier versions hostid is used
+		return (isset($lic['machineid']));
 	}
 
 	/**
@@ -98,12 +99,24 @@ class SystemUpdates {
 				unlink("/var/spool/asterisk/incron/framework.yum-check-updates");
 			}
 			touch("/var/spool/asterisk/incron/framework.yum-check-updates");
+			$nt = \notifications::create();
+			if($this->checkIfTestingRepoEnabled()) {
+				$nt->add_warning('framework', 'test_repos_enabled', _("Test repos are enabled"), _("'sangoma-devel' rpm is installed.\nyum repo 'sng7-testing' is enabled."), "", false, true);
+			} else {
+				$nt->delete('framework', 'test_repos_enabled');
+			}
+
+			$nt->delete("freepbx", "RPM_BROKEN");
+			if($this->checkBrokenRpm()) {
+				$nt->add_warning("freepbx", "RPM_BROKEN",_("You have some fault RPM versions running"),_("Found some faulty RPM versions please run 'yum update -y' manually to update to the latest version."));
+			}
 			// Wait up to 5 seconds for it to start
 			$endafter = time()+5;
 			while (time() < $endafter) {
 				if ($this->isYumRunning()) {
 					return true;
 				}
+				return true;
 				usleep(100000); // 1/10th of a second.
 			}
 		} finally {
@@ -143,6 +156,11 @@ class SystemUpdates {
 			}
 		} finally {
 			$this->getLock()->release();
+			$nt = \notifications::create();
+			$nt->delete("freepbx", "RPM_BROKEN");
+			if($this->checkBrokenRpm()) {
+				$nt->add_warning("freepbx", "RPM_BROKEN",_("You have some fault RPM versions running"),_("Found some faulty RPM versions please run 'yum update -y' manually to update to the latest version."));
+			}
 		}
 
 		// If we made it here, the updates never started
@@ -346,13 +364,20 @@ class SystemUpdates {
 		$wrapped = null; //https://bugzilla.redhat.com/show_bug.cgi?id=584525
 		foreach ($lines as $line) {
 			// If the line is blank, or, starts with a space, ignore.
-			if (!$line || $line[0] === " ") {
+			if (!$line || trim($line) === "" || $line[0] === " ") {
 				continue;
 			}
 
 			// Ignore any error lines
-			if (strpos($line, "Trying other mirror") !== false || strpos($line, "Operation too slow") !== false) {
-				continue;
+			$txt_ignore = array(
+				"Trying other mirror",
+				"Operation too slow",
+				"is listed more than once in the configuration",
+			);
+			foreach ($txt_ignore as $txt) {
+				if (strpos($line, $txt) !== false) {
+					continue 2;
+				}
 			}
 
 			$linearr = preg_split("/\s+/", $line);
@@ -386,6 +411,20 @@ class SystemUpdates {
 		}
 		return $rpms;
 	}
+	public function checkNotInstalledPkg(array $rpmResult) {
+		$first_word = 'package';
+		$last_word = 'is not installed';
+		if (empty($rpmResult)) {
+			return false;
+		}
+		$lastElem = end($rpmResult);
+		if ($lastElem && preg_match('/^' . $first_word . '\b(.*)\b' . $last_word .'$/' , $lastElem, $matches)) {
+			return true;
+		}
+                return false;
+        }
+
+
 
 	/**
 	 * Run rpm to get the list of current versions
@@ -407,12 +446,18 @@ class SystemUpdates {
 		// Our RPM Command
 		$cmd = '/usr/bin/rpm -q --queryformat "%{NAME}.%{ARCH} %{VERSION}.%{RELEASE}\n" '.join(" ", $rpms);
 		exec($cmd, $output, $ret);
-		if ($ret !== 0 && $ret !== 6) {
-			// 6 = new packages are going to be installed
-			if (function_exists("freepbx_log")) {
-				freepbx_log(FPBX_LOG_CRITICAL, "Update error: Tried to run '$cmd', exit code $ret");
+		if ($ret == 1 && $this->checkNotInstalledPkg($output)) {
+			/* if last package in rpm -q is not installed then it will have 1 as return code
+			 * so we have to check if last line of return output has "package not installed"
+			 * or not, if this is present then ignore the error and continue with processing */
+		} else {
+			if ($ret !== 0 && $ret !== 6) {
+				// 6 = new packages are going to be installed
+				if (function_exists("freepbx_log")) {
+					freepbx_log(FPBX_LOG_CRITICAL, sprintf(_("Update error: Tried to run '%s', exit code %s"), $cmd, $ret));
+				}
+				throw new \Exception(sprintf(_("RPM command errored, Delete /dev/shm/yumwrapper/* and try again. Exit code %s - see FreePBX log for more info."), $ret));
 			}
-			throw new \Exception("RPM command errored, Delete /dev/shm/yumwrapper/* and try again. Exit code $ret - see FreePBX log for more info.");
 		}
 
 		// Map the output of the rpm command to a temporary dict
@@ -493,6 +538,23 @@ class SystemUpdates {
 		// We should have some output that can be displayed to the user
 		if (file_exists("/dev/shm/yumwrapper/yum-update-current.log")) {
 			$retarr['currentlog'] = file("/dev/shm/yumwrapper/yum-update-current.log", FILE_IGNORE_NEW_LINES);
+			$summary = array();
+			$summary[] = "--------------------------------------------------------------------------";
+			$record_summary = false;
+			foreach($retarr['currentlog'] as $index => $line) {
+				$record_summary = (preg_match("~\bTransaction Summary\b~",$line)) ? true : $record_summary;
+				$record_summary = ( preg_match("~\bDownloading packages\b~",$line) ) ? false : $record_summary;
+				if (strpos($line, "No packages marked") === 0){
+					$summary[] = "Transaction Summary : ".$retarr['currentlog'][$index];
+					$retarr['currentlog'] =[];
+					break;
+				}
+				if($record_summary && !(strpos($line, "============") === 0)){
+					$summary[] = $retarr['currentlog'][$index];
+				}
+			}
+			$summary[] = "-------------------------------------------------------------------------- \n \n \n ";
+			$retarr['currentlog'] = array_merge($summary,$retarr['currentlog']);
 		}
 
 		// Has it finished?
@@ -531,6 +593,9 @@ class SystemUpdates {
 			$idle = false;
 			$currentstatus = _("Working");
 		}
+		if($this->checkIfTestingRepoEnabled()) {
+			$html .= show_help(_("This system has test repos enabled."), _('Test Repos Enabled'), false, false);
+		}
 		$html .= "<div class='row'>
 			<div class='col-xs-3'>"._("Current System Update Status:")."</div>
 			<div class='col-xs-5'>$currentstatus</div>
@@ -633,4 +698,44 @@ class SystemUpdates {
 		$this->lock = $factory->createLock('systemupdates',7200);
 		return $this->lock;
 	}
+
+	private function checkIfTestingRepoEnabled() {
+		//check "sangoma-devel" rpm installed
+        $ret = exec("/usr/bin/rpm -qa|grep sangoma-devel");
+        if (!empty($ret)) {
+			//check if the yum repo "sng7-testing" is enabled
+			$ret = exec("/usr/bin/yum repolist | grep -i sng7-testing");
+			if (!empty($ret)) {
+				return true;
+			}
+        }
+        return false;
+	}
+
+	public function checkBrokenRpm()
+	{
+		if(!\FreePBX::Modules()->checkStatus('sysadmin')) {
+			return false;
+		}
+		// in future add the broken rpm version to this array
+		$brokenRpms = [
+			'sysadmin-5.6-5.6.48.sng.noarch',
+			'sysadmin-5.6-5.6.49.sng.noarch',
+			'sysadmin-5.6-5.6.50.sng.noarch',
+			'sysadmin-5.6-5.6.51.sng.noarch',
+		];
+		$rpmNames = ['sysadmin'];
+		if(count($rpmNames) > 1) {
+			$moduleStr = implode("|",$rpmNames);
+			$process = exec("rpm -qa | grep -E '$moduleStr'");
+		} else {
+			$process = exec("rpm -qa | grep '".$rpmNames[0]."'");
+		}
+		foreach ($brokenRpms as $vers) {
+			if (strpos($process, $vers) !== false) {
+				return true;
+			}
+		}
+		return false;
+	}
 }
