@@ -52,6 +52,7 @@ class Chown extends Command {
 		foreach ($conf as $key => $val){
 			${$key} = $val['value'];
 		}
+
 		/*
 		 * These are files Framework is responsible for This list can be
 		 * reduced by moving responsibility to other modules as a hook
@@ -192,14 +193,27 @@ class Chown extends Command {
 		if((empty($this->moduleName) && $this->moduleName != 'framework') && posix_geteuid() == 0 && (!$input->hasOption('file') || !$input->getOption('file'))) {
 			$output->write(_("Setting base permissions..."));
 
-			$this->systemSetRecursivePermissions($ASTVARLIBDIR . '/' . $MOHDIR, 0775, $ampowner, $ampowner, 'rdir');
-			$this->systemSetRecursivePermissions($ASTVARLIBDIR . '/sounds', 0775, $ampowner, $ampowner, 'rdir');
-			$this->systemSetRecursivePermissions($ASTLOGDIR, 0775, $ampowner, $ampowner, 'rdir');
-			$this->systemSetRecursivePermissions($ASTSPOOLDIR, 0775, $ampowner, $ampowner, 'rdir');
-			$this->systemSetRecursivePermissions($AMPWEBROOT, 0775, $ampowner, $ampowner, 'rdir');
-			//$this->systemSetRecursivePermissions('/usr/src/freepbx', 0775, $ampowner, $ampowner, 'rdir');
+			$DIRS[]		= $ASTVARLIBDIR . '/' . $MOHDIR;
+			$DIRS[] 	= $ASTVARLIBDIR . '/sounds';
+			$DIRS[] 	= $ASTLOGDIR;
+			$DIRS[] 	= $ASTSPOOLDIR;
+			$DIRS[] 	= $AMPWEBROOT;
+			// $DIRS[] = '/usr/src/freepbx';
+			$skipped 	= array();
+			$Tstart 	= strtotime("now");
+			foreach($DIRS as $DIR){
+				$bl 	= $this->systemSetRecursivePermissions($DIR, 0775, $ampowner, $ampowner, 'rdir');
+				$skipped= array_merge($skipped,$bl);
+			}
 
-			$output->writeln(_("Done"));
+			foreach($skipped as $data){
+				foreach($data as $skip){
+					$this->d(sprintf(_('%s skipped by configuration'), str_replace("'","", $skip)));
+				}				
+			}
+			$Tend = strtotime("now");
+			$Ttime = $Tend - $Tstart; 
+			$output->writeln(_("Done")." ".sprintf(_("in %d seconds"), $Ttime ));
 		}
 
 		$output->writeln(_("Setting specific permissions..."));
@@ -291,6 +305,9 @@ class Chown extends Command {
 	 * @param  int                  $umask The mode mask (octal)
 	 */
 	private function systemSetRecursivePermissions($path, $mode, $owner='asterisk', $group='asterisk', $rmode = 'rdir', $umask = 0000) {
+		if(empty($path)) {
+			throw new \Exception("Path is empty!");
+		}
 		$blacklist = $this->blacklist;
 		if(!empty($blacklist['files'])) {
 			array_walk($blacklist['files'], function(&$value, $key) {
@@ -326,6 +343,7 @@ class Chown extends Command {
 		}
 
 		exec("find ".escapeshellarg($path)." \( -type f -o -type d \) ".$skip." -exec chown ".escapeshellarg($owner).":".escapeshellarg($group)." {} +");
+        return $blacklist;
 	}
 
 	/**
@@ -357,15 +375,29 @@ class Chown extends Command {
 		$mask = ( $mask & ~0111 );
 		return $mask;
 	}
-
+	
+	/**
+	 * checkPermissions
+	 * Check that we are passing a valid octal number
+	 * @param  mixed $file
+	 * @param  mixed $mode
+	 * @return bool
+	 */
 	private function checkPermissions($file, $mode) {
-		if(decoct(octdec($mode)) == $mode){
-			return true;
-		}else{
+		if(is_string($mode)) {
 			$this->d[] = "<error>".sprintf(_('%s Likely will not work as expected'),$file)."</error>";
-			$this->d[] = "<error>".sprintf(_('Permissions should be set with a leading 0, example 644 should be 0644 File: %s Permission set as: %s'),$file,$mode)."</error>";
+			$this->d[] = "<error>".sprintf(_('Permissions should be set as integer not a string'))."</error>";
 			return false;
 		}
+
+		// 511 is the integer value of 0777
+		if (octdec(decoct($mode)) > 511) {
+			$this->d[] = "<error>".sprintf(_('%s Likely will not work as expected'),$file)."</error>";
+			$this->d[] = "<error>".sprintf(_('Incorrect permissions please check the permission your trying to set'))."</error>";
+			return false;
+		}
+
+  		return true;
 	}
 
 	private function d($message) {
