@@ -9,7 +9,10 @@ use Symfony\Component\Console\Helper\Table;
 use Symfony\Component\Console\Helper\ProgressBar;
 use Symfony\Component\Console\Question\Question;
 use Symfony\Component\Console\Question\ChoiceQuestion;
+use Symfony\Component\Console\Input\ArrayInput;
+use Symfony\Component\Console\Output\NullOutput;
 use Symfony\Component\Console\Command\LockableTrait;
+use FreePBX;
 
 class Moduleadmin extends Command {
 	use LockableTrait;
@@ -20,11 +23,13 @@ class Moduleadmin extends Command {
 	private $pretty = false;
 	private $skipchown = false;
 	private $previousEdge = null;
+    private $previousCache = null;
 	private $tag = null;
 	private $emailbody = [];
 	private $sendemail = false;
 	private $updatemanager = null;
 	private $nt = null;
+	private $skipbreaking = false;
 	public $DEBUG = false;
 
 	public function __destruct() {
@@ -36,8 +41,8 @@ class Moduleadmin extends Command {
 			return;
 		}
 
-		$brand = \FreePBX::Config()->get('DASHBOARD_FREEPBX_BRAND');
-		$ident = \FreePBX::Config()->get('FREEPBX_SYSTEM_IDENT');
+		$brand = $this->FreePBX->Config->get('DASHBOARD_FREEPBX_BRAND');
+		$ident = $this->FreePBX->Config->get('FREEPBX_SYSTEM_IDENT');
 		// We are sending an email.
 		$body = array_merge([
 			sprintf(_("This is an automatic notification from your %s (%s) server."), $brand, $ident),
@@ -56,6 +61,7 @@ class Moduleadmin extends Command {
 			new InputOption('force', 'f', InputOption::VALUE_NONE, _('Force operation (skips dependency and status checks) <warning>WARNING:</warning> Use at your own risk, modules have dependencies for a reason!')),
 			new InputOption('debug', 'd', InputOption::VALUE_NONE, _('Output debug messages to the console (be super chatty)')),
 			new InputOption('edge', '', InputOption::VALUE_NONE, _('Download/Upgrade forcing edge mode')),
+            new InputOption('ignorecache', '', InputOption::VALUE_NONE, _('Ignore cache')),
 			new InputOption('stable', '', InputOption::VALUE_NONE, _('Download/Upgrade forcing stable version')),
 			new InputOption('color', '', InputOption::VALUE_NONE, _('Colorize table based list')),
 			new InputOption('skipchown', '', InputOption::VALUE_NONE, _('Skip the chown operation')),
@@ -65,6 +71,7 @@ class Moduleadmin extends Command {
 			new InputOption('format', '', InputOption::VALUE_REQUIRED, sprintf(_('Format can be: %s'),'json, jsonpretty')),
 			new InputOption('repo', 'R', InputOption::VALUE_REQUIRED | InputOption::VALUE_IS_ARRAY, _('Set the Repos. -R Commercial -R Contributed')),
 			new InputOption('tag', 't', InputOption::VALUE_REQUIRED, _('Download/Upgrade to a specific tag')),
+			new InputOption('skipbreakingcheck', '', InputOption::VALUE_NONE, _('Skip breaking changes checks')),
 			new InputOption('sendemail', '', InputOption::VALUE_NONE, _('Send out finalized email')),
 			new InputOption('onlystdout', '', InputOption::VALUE_NONE, _('Deprecated option')),
 			new InputArgument('args', InputArgument::IS_ARRAY, 'arguments passed to module admin, this is s stopgap', null),))
@@ -72,13 +79,27 @@ class Moduleadmin extends Command {
 	}
 
 	protected function execute(InputInterface $input, OutputInterface $output){
+		$this->FreePBX = FreePBX::Create();
+		/* TODO: Come back to this at some point
+		if(posix_getuid() === 0) {
+			$AMPASTERISKWEBUSER = $this->FreePBX->Config->get('AMPASTERISKWEBUSER');
+			$info = posix_getpwnam($AMPASTERISKWEBUSER);
+			if(empty($info)) {
+				$output->writeln("$AMPASTERISKWEBUSER is not a valid user");
+				return 0;
+			}
+			posix_setuid($info['uid']);
+		}
+		*/
+
 		/* Investigate this at another time
 		if (!$this->lock()) {
 			$output->writeln('The command is already running in another process.');
 			return 0;
 		}
 		*/
-		$this->nt = \FreePBX::Notifications();
+
+		$this->nt = $this->FreePBX->Notifications;
 		$this->updatemanager = new \FreePBX\Builtin\UpdateManager();
 		$this->mf = \module_functions::create();
 		$this->out = $output;
@@ -94,13 +115,25 @@ class Moduleadmin extends Command {
 		if($input->getOption('color')) {
 			$this->color = true;
 		}
-		if ($input->getOption('tag')) {
-			$this->tag = $input->getOption('tag');
-		}
+
 		if ($input->getOption('debug')) {
 			$this->DEBUG = True;
+		}
+
+		if ($input->getOption('skipbreakingcheck')) {
+			$this->skipbreaking = True;
+			if($this->DEBUG){
+				$this->writeln(_('skipbreakingcheck Enabled'));
+			}
 		} else {
-			$this->DEBUG = False;
+			$this->skipbreaking = False;
+			if($this->DEBUG){
+				$this->writeln(_('skipbreakingcheck Disabled'));
+			}
+		}
+
+		if ($input->getOption('tag')) {
+			$this->tag = $input->getOption('tag');
 		}
 		if(!empty($input->getOption('snapshot'))){
 			$ret = array();
@@ -169,24 +202,34 @@ class Moduleadmin extends Command {
 				$this->writeln('<error>'._('Confusing statement. Not sure what you want to do').'</error>');
 				exit(255);
 			}
+			if($input->getOption('ignorecache')){
+				$this->previousCache = \FreePBX::Config()->get('MODULEADMIN_SKIP_CACHE');
+				if($this->previousCache) {
+					$this->writeln('<info>'._('Disable Module Admin caching already enabled, ignoring option').'</info>');
+					$this->previousCache = null;
+				} else {
+					$this->writeln('<info>'._('Disable Module Admin caching temporarily enabled').'</info>');
+					\FreePBX::Config()->update('MODULEADMIN_SKIP_CACHE',1);
+				}
+			}
 			if($input->getOption('edge')) {
-				$this->previousEdge = \FreePBX::Config()->get('MODULEADMINEDGE');
+				$this->previousEdge = $this->FreePBX->Config()->get('MODULEADMINEDGE');
 				if($this->previousEdge) {
 					$this->writeln('<info>'._('Edge repository already enabled, ignoring option').'</info>');
 					$this->previousEdge = null;
 				} else {
 					$this->writeln('<info>'._('Edge repository temporarily enabled').'</info>');
-					\FreePBX::Config()->update('MODULEADMINEDGE',1);
+					$this->FreePBX->Config->update('MODULEADMINEDGE',1);
 				}
 			}
 			if($input->getOption('stable')) {
-				$this->previousEdge = \FreePBX::Config()->get('MODULEADMINEDGE');
+				$this->previousEdge = $this->FreePBX->Config->get('MODULEADMINEDGE');
 				if(!$this->previousEdge) {
 					$this->writeln('<info>'._('Stable repository already enabled, ignoring option').'</info>');
 					$this->previousEdge = null;
 				} else {
 					$this->writeln('<info>'._('Stable repository temporarily enabled').'</info>');
-					\FreePBX::Config()->update('MODULEADMINEDGE',0);
+					$this->FreePBX->Config->update('MODULEADMINEDGE',0);
 				}
 			}
 			try {
@@ -206,7 +249,12 @@ class Moduleadmin extends Command {
 	private function resetMode() {
 		if(!is_null($this->previousEdge)) {
 			$this->writeln("<info>Resetting temporarily repository state</info>");
-			\FreePBX::Config()->update('MODULEADMINEDGE',$this->previousEdge);
+			$this->FreePBX->Config->update('MODULEADMINEDGE',$this->previousEdge);
+		}
+      
+      	if(!is_null($this->previousCache)) {
+			$this->writeln("<info>Resetting temporary state for Disable Module Admin caching</info>");
+			\FreePBX::Config()->update('MODULEADMIN_SKIP_CACHE',$this->previousCache);
 		}
 	}
 
@@ -312,7 +360,7 @@ class Moduleadmin extends Command {
 		if (!$fwconsole) {
 			// Try to find our fwconsole.
 			try {
-				$fwconsole = \FreePBX::Config()->get('AMPSBIN')."/fwconsole";
+				$fwconsole = $this->FreePBX->Config->get('AMPSBIN')."/fwconsole";
 			} catch (\Exception $e) {
 				$fwconsole = "/usr/sbin/fwconsole";
 			}
@@ -335,7 +383,7 @@ class Moduleadmin extends Command {
 		return $fwconsole;
 	}
 
-	private function doForkInstall($modulename, $force) {
+	private function doForkInstall($modulename) {
 
 		$fwconsole = $this->getFwconsolePath();
 
@@ -344,8 +392,11 @@ class Moduleadmin extends Command {
 			1 => STDOUT,
 			2 => STDERR
 		);
-		$force = $force ? "--force" : "";
-		$cmd = "$fwconsole ma install ".escapeshellarg($modulename)." ".$force;
+		$force = $this->force ? "--force" : "";
+
+		$ignoreChecks = $this->skipbreaking ? "--skipbreakingcheck" : "";
+
+		$cmd = "$fwconsole ma install ".escapeshellarg($modulename)." ".$force." ".$ignoreChecks;
 		//release lock, it'll be regained in a bit
 		$this->release();
 		$process = proc_open($cmd, $descriptorspec, $pipes);
@@ -362,9 +413,9 @@ class Moduleadmin extends Command {
 		return $result;
 	}
 
-	private function doInstall($modulename, $force) {
+	private function doInstall($modulename) {
 		$start = time();
-		\FreePBX::Modules()->loadAllFunctionsInc();
+		$this->FreePBX->Modules->loadAllFunctionsInc();
 		$module = $this->mf->getinfo($modulename);
 		$modulestatus = isset($module[$modulename]['status'])?$module[$modulename]['status']:false;
 		if($modulestatus === 1 && !$this->input->getOption('skipdisabled') && !$this->input->getOption('autoenable')){
@@ -374,7 +425,7 @@ class Moduleadmin extends Command {
 			$action = $helper->ask($this->input,$this->out,$question);
 			switch($action){
 				case _("Enable"):
-					$this->mf->enable($modulename, $force);
+					$this->mf->enable($modulename, $this->force, $this->skipbreaking);
 				break;
 				case _("Cancel"):
 					exit;
@@ -383,18 +434,16 @@ class Moduleadmin extends Command {
 		}
 		if($this->input->getOption('autoenable') && $modulestatus === 1){
 			$this->writeln(sprintf(_("Enabling %s because autoenable was passed at the command line"),$modulename));
-			$this->mf->enable($modulename, $force);
+			$this->mf->enable($modulename, $this->force, $this->skipbreaking);
 		}
-		if(!$force && !$this->mf->resolveDependencies($modulename,array($this,'progress'))) {
-			$this->writeln(sprintf(_("Unable to resolve dependencies for module %s:"),$modulename), "error", false);
+		if(!$this->force && !$this->mf->resolveDependencies($modulename,array($this,'progress'))) {
+			$this->writeln(sprintf(_("Unable to resolve dependencies for module %s"),$modulename), "error", false);
 			$this->addToEmail(sprintf(_("Module %s installation failed, could not resolve dependencies"), $name));
 			return false;
 		} else {
-			if (is_array($errors = $this->mf->install($modulename, $force))) {
-				$this->writeln("Unable to install module ${modulename}:", "error", false);
-				$this->writeln(' - '.implode("\n - ",$errors), "error", false);
-				$this->addToEmail(sprintf(_("Module %s installation failed with errors: %s"), $modulename, implode("\n -", $errors)));
-				return false;
+			$result = $this->mf->install($modulename, $this->force, $this->skipbreaking);
+			if(is_array($result) && !empty($result)){
+				return $this->handleErrors($result, $modulename);
 			} else {
 				$this->writeln("Module ".$modulename." version ".$this->mf->_getversion($modulename)." successfully installed");
 				$this->addToEmail(sprintf(_("Module %s installation completed in %s seconds"), $modulename, time()-$start));
@@ -415,7 +464,7 @@ class Moduleadmin extends Command {
 		return true;
 	}
 
-	private function doDownload($modulename, $force) {
+	private function doDownload($modulename) {
 		global $modulexml_path;
 		global $modulerepository_path;
 
@@ -432,8 +481,18 @@ class Moduleadmin extends Command {
 		if (isset($this->tag)) {
 			$xml = $this->mf->getModuleDownloadByModuleNameAndVersion($modulename, $this->tag);
 			if (empty($xml)) {
-				$this->writeln("Unable to update module ${modulename} - ".$this->tag.":", "error", false);
+				$this->writeln("Unable to update module ${modulename} - ".$this->tag.", it does not exist:", "error", false);
 				return false;
+			} else {
+				$data = $this->checkConflicts($xml);
+				if(!empty($data['breaking'])) {
+					foreach($data['issues'] as $issues) {
+						foreach($issues as $issue) {
+							$this->writeln($issue, "error", false);
+						}
+					}
+					return false;
+				}
 			}
 			return $this->doRemoteDownload($xml['downloadurl']);
 		}
@@ -443,7 +502,7 @@ class Moduleadmin extends Command {
 
 		// Try to get the module
 		$start = time();
-		$result = $this->mf->download($modulename, $this->force, array($this,'progress'), $modulerepository_path, $modulexml_path);
+		$result = $this->mf->download($modulename, $this->force, array($this,'progress'), $modulerepository_path, $modulexml_path, $this->skipbreaking);
 		$end = time();
 
 		$elapsed = $end - $start;
@@ -516,6 +575,16 @@ class Moduleadmin extends Command {
 				$this->writeln("Done");
 			break;
 			case "untar":
+				if(isset($this->progress)) {
+					if($this->format == 'json') {
+						$this->writeln(array("read" => $data['total'], "total" => $data['total']),"downloading");
+					} else {
+						$this->progress->finish();
+						$this->writeln("");
+						$this->writeln("Finished downloading");
+					}
+					unset($this->progress);
+				}
 				$this->write("Extracting...");
 			break;
 		}
@@ -525,6 +594,8 @@ class Moduleadmin extends Command {
 		// We're doing security updates.
 		$mf = \module_functions::create();
 		$mods = (array) $mf->get_security();
+		$brand = \FreePBX::Config()->get('DASHBOARD_FREEPBX_BRAND');
+		$ident = \FreePBX::Config()->get('FREEPBX_SYSTEM_IDENT');
 
 		if (!$mods) {
 			// Easy. No security vulnerabilities!
@@ -532,12 +603,12 @@ class Moduleadmin extends Command {
 		}
 
 		// OK, There are security vulnerabilities. Prepare our email.
-		$email_subject = sprintf(_("%s Security Alert (%s)"), $this->brand, $this->machine_id);
-		$email_body = sprintf(_("Your server [%s] discovered the following security issues:"), $this->brand)."\n";
+		$email_subject = sprintf(_("%s Security Alert (%s)"), $brand, $ident);
+		$email_body = sprintf(_("Your server [%s] discovered the following security issues:"), $brand)."\n";
 		$send_email = false;
 
 		set_time_limit(0); // Never time out.
-		$ampsbin = \FreePBX::Config()->get('AMPSBIN');
+		$ampsbin = $this->FreePBX->Config->get('AMPSBIN');
 
 		$notification_title = "";
 		$notification_body = "";
@@ -576,7 +647,7 @@ class Moduleadmin extends Command {
 				continue;
 			}
 
-			$this->out->write("$action $rawname due to security vulnerability ".implode($info['vul'],', ')."...");
+			$this->write("$action $rawname due to security vulnerability ".implode(', ',$info['vul'])."...");
 
 			// Upgrade/install/whatever our mod
 			exec($ampsbin."/fwconsole ma $action ".escapeshellarg($rawname)." --format=json",$out,$ret);
@@ -585,10 +656,11 @@ class Moduleadmin extends Command {
 			if($ret != 0) {
 				$errorvuls[$rawname] = $info;
 			}
-
+			$notification_body = '';
+			$email_body = '';
 			if(!empty($errorvuls)) {
 				// There were issues upgrading some modules.
-				$this->out->writeln("Failed!");
+				$this->writeln("Failed!");
 				$cnt = count($errorvuls);
 				if ($cnt == 1) {
 					$emailbody .= "\n\n"._("WARNING: There was an issue automatically repairing the security vulnerabilities on the following module. This module requires manual attention urgently:")."\n";
@@ -600,7 +672,7 @@ class Moduleadmin extends Command {
 				foreach($errorvuls as $m => $vinfo) {
 					$line = sprintf(
 						_("%s (Cur v. %s) should be upgraded to v. %s to fix security issues: %s")."\n",
-						$m, $vinfo['curver'], $vinfo['minver'], implode($vinfo['vul'],', ')
+						$m, $vinfo['curver'], $vinfo['minver'], implode(', ',$vinfo['vul'])
 					);
 					$notification_body .= $line;
 					$email_body .= "    $line";
@@ -611,12 +683,12 @@ class Moduleadmin extends Command {
 				foreach($mods as $m => $vinfo) {
 					$line = sprintf(
 						_("%s has been automatically upgraded to fix security issues: %s")."\n",
-						$m, implode($vinfo['vul'],', ')
+						$m, implode(', ',$vinfo['vul'])
 					);
 					$success_notification_body .= $line;
 					$email_body .= "    $line";
 				}
-				$this->out->writeln("Done!");
+				$this->writeln("Done!");
 			}
 		}
 		if(!empty($success_notification_title)) {
@@ -630,8 +702,8 @@ class Moduleadmin extends Command {
 		return true;
 	}
 
-	private function doDelete($modulename, $force) {
-		\FreePBX::Modules()->loadAllFunctionsInc();
+	private function doDelete($modulename) {
+		$this->FreePBX->Modules->loadAllFunctionsInc();
 		if (is_array($errors = $this->mf->delete($modulename, $this->force))) {
 			$this->writeln(_("The following error(s) occured:"), "error", false);
 			$this->writeln(' - '.implode("\n - ",$errors), "error", false);
@@ -641,8 +713,8 @@ class Moduleadmin extends Command {
 		}
 	}
 
-	private function doUninstall($modulename, $force) {
-		\FreePBX::Modules()->loadAllFunctionsInc();
+	private function doUninstall($modulename) {
+		$this->FreePBX->Modules->loadAllFunctionsInc();
 		if (is_array($errors = $this->mf->uninstall($modulename, $this->force))) {
 			$this->writeln(_("The following error(s) occured:"), "error", false);
 			$this->writeln(' - '.implode("\n - ",$errors), "error", false);
@@ -652,12 +724,14 @@ class Moduleadmin extends Command {
 		}
 	}
 
-	private function doUpgrade($modulename, $force) {
-		$this->doDownload($modulename, $this->force);
-		$this->doForkInstall($modulename, $this->force);
+	private function doUpgrade($modulename) {
+		$ret = $this->doDownload($modulename);
+		if($ret) {
+			$this->doForkInstall($modulename);
+		}
 	}
 
-	private function doInstallLocal($force) {
+	private function doInstallLocal() {
 		//refresh module cache
 		$this->mf->getinfo(false,false,true);
 		$module_info=$this->mf->getinfo(false, array(MODULE_STATUS_NOTINSTALLED,MODULE_STATUS_NEEDUPGRADE));
@@ -669,15 +743,15 @@ class Moduleadmin extends Command {
 		}
 		if (in_array('core', $modules)){
 			$this->writeln("Installing core...");
-			$this->doForkInstall('core', $this->force);
+			$this->doForkInstall('core');
 		}
 		if (count($modules) > 0) {
 			$this->writeln("Installing: ".implode(', ',$modules));
 			foreach ($modules as $module => $name) {
 				if (($name != 'core')){//we dont want to reinstall core
-					\FreePBX::Modules()->loadAllFunctionsInc(); //get functions from other modules, in case we need them here
+					$this->FreePBX->Modules->loadAllFunctionsInc(); //get functions from other modules, in case we need them here
 					$this->writeln("Installing $name...");
-					$this->doForkInstall($name, $this->force);
+					$this->doForkInstall($name);
 					$this->writeln("");
 				}
 			}
@@ -697,8 +771,13 @@ class Moduleadmin extends Command {
 		$modules_local = $this->mf->getinfo($modulename, array(MODULE_STATUS_ENABLED,MODULE_STATUS_NEEDUPGRADE));
 		$modules_online = $this->mf->getonlinexml();
 		$this->check_active_repos();
+
 		if (isset($modules_local[$modulename])) {
 			if(isset($modules_online[$modulename])) {
+				$data = $this->checkConflicts($modules_online[$modulename]);
+				if(!empty($data['breaking'])) {
+					return $data['issues'];
+				}
 				$upgrade_version = $modules_online[$modulename]['version'];
 			} elseif (!empty($modules_local[$modulename]['updateurl']) && parse_url($modules_local[$modulename]['updateurl'], PHP_URL_SCHEME) === 'https') {
 				$module_update_json = $this->mf->url_get_contents($modules_local[$modulename]['updateurl'], "");
@@ -781,7 +860,7 @@ class Moduleadmin extends Command {
 	 * This is called by the automatica updater, so its output
 	 * is saved for sendemail to possibly use
 	 */
-	private function doUpgradeAll($force) {
+	private function doUpgradeAll() {
 		$modules = $this->getUpgradableModules();
 		if ($modules) {
 			$line = sprintf("Module(s) requiring upgrades: %s", implode(", ", array_keys($modules)));
@@ -805,7 +884,7 @@ class Moduleadmin extends Command {
 				$line = sprintf(_("Upgrading module '%s' from %s to %s"), $name, $arr['local_version'], $arr['online_version']);
 				$this->writeln($line);
 				$this->addToEmail($line);
-				$this->doUpgrade($name, $this->force);
+				$this->doUpgrade($name);
 			}
 			$line = _("All upgrades completed successfully!");
 			$this->writeln($line);
@@ -943,17 +1022,27 @@ class Moduleadmin extends Command {
 			return;
 		}
 		if (posix_getuid() == 0) {
-			$chown = new Chown();
+			$command = $this->getApplication()->find('chown');
+
+			$arguments = array(
+				'command' => 'chown'
+			);
+
 			if(sizeof($args) == 1){
-				$chown->moduleName = is_array($args[0]) ? $args[0]['name'] : $args[0];
+				$arguments['--module'] = is_array($args[0]) ? $args[0]['name'] : $args[0];
 			}
+
+			$out = $this->out;
 			if(!($this->out->getVerbosity() >= OutputInterface::VERBOSITY_VERBOSE)) {
-				$prevVerbosity = $this->out->getVerbosity();
-				$this->out->setVerbosity(OutputInterface::VERBOSITY_QUIET);
+				$this->write("Chowning directories...");
+				$out = new NullOutput();
 			}
-			$chown->execute($this->input, $this->out, !($this->out->getVerbosity() >= OutputInterface::VERBOSITY_VERBOSE));
-			if(!empty($prevVerbosity)) {
-				$this->out->setVerbosity($prevVerbosity);
+
+			$greetInput = new ArrayInput($arguments);
+			$returnCode = $command->run($greetInput, $out);
+
+			if(!($this->out->getVerbosity() >= OutputInterface::VERBOSITY_VERBOSE)) {
+				$this->writeln("Done");
 			}
 		}
 	}
@@ -996,8 +1085,10 @@ class Moduleadmin extends Command {
 		// function, above. Rare, but has happened.
 		if (isset($modules['core'])) {
 			$this->writeln(_("Installing core..."));
-			$this->doDownload('core', $this->force);
-			$this->doForkInstall('core', $this->force);
+			$ret = $this->doDownload('core');
+			if($ret) {
+				$this->doForkInstall('core');
+			}
 			unset($modules['core']);
 		}
 
@@ -1007,23 +1098,26 @@ class Moduleadmin extends Command {
 			$this->addToEmail($line);
 
 			foreach ($modules as $i => $name) {
-
 				// Get functions from other modules, in case we need them here.
 				//
 				// FreePBX 14: This should go away, eventually, as we move to
 				// autoloaded and OO code. However, it's here for the foreseeable
 				// future. Sigh.
 				//
-				\FreePBX::Modules()->loadAllFunctionsInc();
+
+				$this->FreePBX->Modules->loadAllFunctionsInc();
 				$line = sprintf(_("Downloading & Installing '%s'"), $name);
 				$this->writeln($line);
-				$this->doDownload($name, $this->force);
-				$start = time();
-				// Note this will addToEmail if it fails.
-				$this->doForkInstall($name, $this->force);
-				$elapsed = time() - $start;
-				$this->addToEmail(sprintf(_("Module %s installation completed in %s seconds"), $name, $elapsed), $line);
-				$this->writeln("");
+				$ret = $this->doDownload($name);
+				if($ret) {
+					$start = time();
+					// Note this will addToEmail if it fails.
+					$this->doForkInstall($name);
+					$elapsed = time() - $start;
+					$this->addToEmail(sprintf(_("Module %s installation completed in %s seconds"), $name, $elapsed), $line);
+					$this->writeln("");
+				}
+
 			}
 			$line = _("Done. All modules installed.");
 			$this->writeln($line);
@@ -1072,7 +1166,7 @@ class Moduleadmin extends Command {
 	}
 
 	private function updateKeys() {
-		\FreePBX::GPG()->refreshKeys();
+		$this->FreePBX->GPG->refreshKeys();
 	}
 
 	private function showList($online = false) {
@@ -1080,6 +1174,7 @@ class Moduleadmin extends Command {
 		$modules_local = $this->mf->getinfo(false,false,true);
 		$modules = $modules_local;
 		$this->check_active_repos();
+		$showSignCol = ($this->FreePBX->Config->get('SIGNATURECHECK')) ? _('Signature') : null;
 		if ($online) {
 			$modules_online = $this->mf->getonlinexml();
 			if (isset($modules_online)) {
@@ -1139,22 +1234,40 @@ class Moduleadmin extends Command {
 			}
 			$module_version = isset($modules[$name]['dbversion'])?$modules[$name]['dbversion']:'';
 			$module_license = isset($modules[$name]['license'])?$modules[$name]['license']:'';
-			array_push($rows,array($name, $module_version, $status, $module_license));
+			if (!empty($showSignCol)) {
+				if ($status_index == MODULE_STATUS_NOTINSTALLED && !isset($modules_local[$name])) {
+					$module_Signature = '';
+				} elseif (file_exists($this->FreePBX->Config->get('AMPWEBROOT') . "/admin/modules/" . $name . "/module.sig")) {
+					$sigfile = $this->FreePBX->Config->get('AMPWEBROOT') . "/admin/modules/" . $name . "/module.sig";
+					$sig = $this->FreePBX->GPG->checkSig($sigfile);
+					if(in_array($sig['config']['signedwith'], array('B53D215A755231A3','86CE877469D2EAD9'))) {
+						$module_Signature = 'Sangoma';
+					} else {
+						$module_Signature = 'Unknown';
+					}
+				} else {
+					$module_Signature = 'Unsigned';
+				}
+				array_push($rows,array($name, $module_version, $status, $module_license, $module_Signature));
+			} else {
+				array_push($rows,array($name, $module_version, $status, $module_license));
+			}
 		}
+		$headers = (!empty($showSignCol)) ? array(_('Module'), _('Version'), _('Status'),_('License'), $showSignCol) : array(_('Module'), _('Version'), _('Status'),_('License'));
 		if($this->format == 'json') {
 			$this->writeln($rows);
 		} else {
 			$table = new Table($this->out);
 			$table
-				->setHeaders(array(_('Module'), _('Version'), _('Status'),_('License')))
+				->setHeaders($headers)
 				->setRows($rows);
 			$table->render();
 		}
 	}
 
 	private function refreshsignatures() {
-		\FreePBX::GPG();
-		$fpbxmodules = \FreePBX::Modules();
+		$this->FreePBX->GPG;
+		$fpbxmodules = $this->FreePBX->Modules;
 		$list = $fpbxmodules->getActiveModules();
 		$this->writeln(_("Getting Data from Online Server..."));
 		//Leaving this here so that we at least know we can get to the mirror server
@@ -1170,7 +1283,7 @@ class Moduleadmin extends Command {
 		foreach($list as $m) {
 			//Check signature status, then if its online then if its signed online then redownload (through force)
 			$this->writeln(sprintf(_("Checking %s..."),$m['rawname']));
-			$msig = \FreePBX::GPG()->verifyModule($m['rawname']);
+			$msig = $this->FreePBX->GPG->verifyModule($m['rawname']);
 			if ($m['rawname'] == 'builtin') {
 				continue;
 			}
@@ -1182,7 +1295,7 @@ class Moduleadmin extends Command {
 					$modulename = $m['rawname'];
 					$moduleversion = $m['version'];
 					$modules[] = $modulename;
-					$this->doInstallByModuleAndVersion($modulename, $moduleversion, true);
+					$this->doInstallByModuleAndVersion($modulename, $moduleversion, true,true);
 					$this->writeln("\t"._("Verifying GPG..."));
 					$this->mf->updateSignature($modulename);
 					$this->writeln(_("Done"));
@@ -1214,7 +1327,7 @@ class Moduleadmin extends Command {
 	private function updateHooks() {
 		$this->write(_("Updating Hooks..."));
 		try {
-			\FreePBX::Hooks()->updateBMOHooks();
+			$this->FreePBX->Hooks->updateBMOHooks();
 		}catch(\Exception $e) {}
 		$this->writeln(_("Done"));
 	}
@@ -1242,9 +1355,9 @@ class Moduleadmin extends Command {
 		}
 	}
 
-	private function doDisable($modulename, $force) {
-		\FreePBX::Modules()->loadAllFunctionsInc();
-		if (is_array($errors = $this->mf->disable($modulename, $force))) {
+	private function doDisable($modulename) {
+		$this->FreePBX->Modules->loadAllFunctionsInc();
+		if (is_array($errors = $this->mf->disable($modulename, $this->force))) {
 			$this->writeln(_("The following error(s) occured:"), "error", false);
 			$this->writeln(' - '.implode("\n - ",$errors), "error", false);
 			exit(2);
@@ -1253,9 +1366,9 @@ class Moduleadmin extends Command {
 		}
 	}
 
-	private function doEnable($modulename, $force) {
-		\FreePBX::Modules()->loadAllFunctionsInc();
-		if (is_array($errors = $this->mf->enable($modulename, $this->force))) {
+	private function doEnable($modulename) {
+		$this->FreePBX->Modules->loadAllFunctionsInc();
+		if (is_array($errors = $this->mf->enable($modulename, $this->force, $this->skipbreaking))) {
 			$this->writeln(_("The following error(s) occured:"), "error", false);
 			$this->writeln(' - '.implode("\n - ",$errors), "error", false);
 			exit(2);
@@ -1264,9 +1377,9 @@ class Moduleadmin extends Command {
 		}
 	}
 
-	private function tryEnable($modulename, $force) {
-		\FreePBX::Modules()->loadAllFunctionsInc();
-		if (is_array($errors = $this->mf->enable($modulename, $this->force))) {
+	private function tryEnable($modulename) {
+		$this->FreePBX->Modules->loadAllFunctionsInc();
+		if (is_array($errors = $this->mf->enable($modulename, $this->force, $this->skipbreaking))) {
 			$this->writeln(_("The following error(s) occured:"), "error", false);
 			$this->writeln(' - '.implode("\n - ",$errors), "error", false);
 		} else {
@@ -1331,7 +1444,7 @@ class Moduleadmin extends Command {
 	}
 
 	private function handleArgs($args,$output){
-		$devmode = \FreePBX::Config()->get('DEVEL');
+		$devmode = $this->FreePBX->Config->get('DEVEL');
 		$action = array_shift($args);
 		switch($action){
 			case 'updatekeys':
@@ -1343,7 +1456,24 @@ class Moduleadmin extends Command {
 				}
 				foreach($args as $module){
 					//do NOT fork install here!
-					$this->doInstall($module, $this->force);
+					$colon = strpos($module, ':');
+					if($colon !== false){
+						$tmp = explode(':', $module);
+						$module = $tmp[0];
+						if(isset($tmp[1]) && !empty($tmp[1])){
+							$this->tag = $tmp[1];
+						}
+					}
+					if($colon === false){
+						$tag = $this->input->getOption('tag');
+						if($tag){
+							$this->tag = $tag;
+						}
+						if(!$tag){
+							unset($this->tag);
+						}
+					}
+					$this->doInstall($module);
 				}
 				$this->updateHooks();
 				$this->setPerms($action,$args);
@@ -1352,6 +1482,7 @@ class Moduleadmin extends Command {
 				if($devmode) {
 					fatal(_("Can not run this command while 'Developer Mode' is enabled"));
 				}
+
 				$this->check_active_repos();
 				$modules = $this->doInstallAll();
 				$this->updateHooks();
@@ -1371,7 +1502,7 @@ class Moduleadmin extends Command {
 					fatal(_("Missing module name"));
 				}
 				foreach($args as $module){
-					$this->doUninstall($module, $this->force);
+					$this->doUninstall($module);
 				}
 				$this->updateHooks();
 				break;
@@ -1387,7 +1518,24 @@ class Moduleadmin extends Command {
 					if($this->isUrl($module)) {
 						$this->doRemoteDownload($module);
 					} else {
-						$this->doDownload($module, $this->force);
+						$colon = strpos($module, ':');
+						if ($colon !== false) {
+							$tmp = explode(':', $module);
+							$module = $tmp[0];
+							if (isset($tmp[1]) && !empty($tmp[1])) {
+								$this->tag = $tmp[1];
+							}
+						}
+						if ($colon === false) {
+							$tag = $this->input->getOption('tag');
+							if ($tag) {
+								$this->tag = $tag;
+							}
+							if (!$tag) {
+								unset($this->tag);
+							}
+						}
+						$this->doDownload($module);
 					}
 				}
 				$this->setPerms($action,$args);
@@ -1404,13 +1552,32 @@ class Moduleadmin extends Command {
 					if($this->isUrl($module)) {
 						$this->doRemoteDownload($module);
 						if(!empty($this->mf->downloadedRawname)) {
-							$this->doInstall($this->mf->downloadedRawname, $this->force);
+							$this->doInstall($this->mf->downloadedRawname);
 						} else {
 							fatal(_("Could not determine module name"));
 						}
 					} else {
-						$this->doDownload($module, $this->force);
-						$this->doInstall($module, $this->force);
+						$colon = strpos($module, ':');
+						if ($colon !== false) {
+							$tmp = explode(':', $module);
+							$module = $tmp[0];
+							if (isset($tmp[1]) && !empty($tmp[1])) {
+								$this->tag = $tmp[1];
+							}
+						}
+						if ($colon === false) {
+							$tag = $this->input->getOption('tag');
+							if ($tag) {
+								$this->tag = $tag;
+							}
+							if (!$tag) {
+								unset($this->tag);
+							}
+						}
+						$ret = $this->doDownload($module);
+						if($ret) {
+							$this->doInstall($module);
+						}
 					}
 				}
 				$this->updateHooks();
@@ -1426,26 +1593,55 @@ class Moduleadmin extends Command {
 				}
 				$this->check_active_repos();
 				foreach($args as $module){
+					$colon = strpos($module, ':');
+					if ($colon !== false) {
+						$tmp = explode(':', $module);
+						$module = $tmp[0];
+						if (isset($tmp[1]) && !empty($tmp[1])) {
+							$this->tag = $tmp[1];
+						}
+					}
+					if ($colon === false) {
+						$tag = $this->input->getOption('tag');
+						if ($tag) {
+							$this->tag = $tag;
+						}
+						if (!$tag) {
+							unset($this->tag);
+						}
+					}
 					$state = $this->isModuleUpgradeable($module);
-					switch($state) {
-						case -4:
-							$this->writeln(sprintf(_('%s does not have a valid JSON update file, unable to upgrade'),$module));
-						break;
-						case -3:
-							$this->writeln(sprintf(_('%s is not a locally installed module, unable to upgrade'),$module));
-						break;
-						case -2:
-							$this->writeln(sprintf(_('%s does not exist online, unable to upgrade'),$module));
-						break;
-						case -1:
-							$this->writeln(sprintf(_('%s is newer than online version, unable to upgrade'),$module));
-						break;
-						case 0:
-							$this->writeln(sprintf(_('%s is the same as the online version, unable to upgrade'),$module));
-						break;
-						case 1:
-							$this->doUpgrade($module, $this->force);
-						break;
+					if(is_array($state)) {
+						foreach($state as $m => $msgs) {
+							if(is_array($msgs)) {
+								foreach($msgs as $msg) {
+									$this->writeln('<error>'.$msg.'</error>');
+								}
+							} else {
+								$this->writeln('<error>'.$msgs.'</error>');
+							}
+						}
+					} else {
+						switch($state) {
+							case -4:
+								$this->writeln('<error>'.sprintf(_('%s does not have a valid JSON update file, unable to upgrade').'</error>',$module));
+							break;
+							case -3:
+								$this->writeln('<error>'.sprintf(_('%s is not a locally installed module, unable to upgrade').'</error>',$module));
+							break;
+							case -2:
+								$this->writeln('<error>'.sprintf(_('%s does not exist online, unable to upgrade').'</error>',$module));
+							break;
+							case -1:
+								$this->writeln('<error>'.sprintf(_('%s is newer than online version, unable to upgrade').'</error>',$module));
+							break;
+							case 0:
+								$this->writeln('<error>'.sprintf(_('%s is the same as the online version, unable to upgrade').'</error>',$module));
+							break;
+							case 1:
+								$this->doUpgrade($module);
+							break;
+						}
 					}
 				}
 				$this->updateHooks();
@@ -1457,7 +1653,7 @@ class Moduleadmin extends Command {
 					fatal(_("Can not run this command while 'Developer Mode' is enabled"));
 				}
 				$this->check_active_repos();
-				$modules = $this->doUpgradeAll($this->force);
+				$modules = $this->doUpgradeAll();
 				$this->updateHooks();
 				foreach($modules as $module) {
 					$this->setPerms($action,array($module));
@@ -1484,8 +1680,9 @@ class Moduleadmin extends Command {
 					$this->updateSecurity();
 				}
 				$this->updatemanager->securityEmail();
-				if($settings['auto_module_updates'] == 'enabled') {
-					$this->updatemanager->updateEmail();
+				$moduleupdates = $settings['auto_module_updates'];
+				if($moduleupdates == 'enabled' || $moduleupdates == 'emailonly') {
+					$this->updatemanager->updateEmail($moduleupdates == 'enabled');
 				}
 				$this->updatemanager->unsignedEmail();
 				break;
@@ -1533,7 +1730,7 @@ class Moduleadmin extends Command {
 					fatal(_("Missing module name"));
 				}
 				foreach($args as $module){
-					$this->doDelete($module, $this->force);
+					$this->doDelete($module);
 				}
 				$this->updateHooks();
 				break;
@@ -1542,7 +1739,7 @@ class Moduleadmin extends Command {
 					fatal(_("Missing module name"));
 				}
 				foreach($args as $module){
-					$this->doDisable($module, $this->force);
+					$this->doDisable($module);
 				}
 				$this->updateHooks();
 				break;
@@ -1551,7 +1748,7 @@ class Moduleadmin extends Command {
 					fatal(_("Missing module name"));
 				}
 				foreach($args as $module){
-					$this->doEnable($module, $this->force);
+					$this->doEnable($module);
 				}
 				$this->updateHooks();
 				break;
@@ -1559,7 +1756,7 @@ class Moduleadmin extends Command {
 				$modules = $this->listDisabled();
 				foreach($modules as $module){
 					$this->writeln(sprintf(_('Attempting to Enable %s'),$module));
-					$this->tryEnable($module, $this->force);
+					$this->tryEnable($module);
 				}
 				$this->writeln(_('This action understands somethings may be disabled for a reason.'));
 				$this->writeln(_('Please review the output above for any errors while enabling modules'));
@@ -1668,9 +1865,9 @@ class Moduleadmin extends Command {
 			return false;
 		}
 
-		\FreePBX::Modules()->loadAllFunctionsInc();
+		$this->FreePBX->Modules->loadAllFunctionsInc();
 		$this->doRemoteDownload($xml['downloadurl']);
-		$this->doForkInstall($modulename, $this->force);
+		$this->doForkInstall($modulename);
 		return true;
 	}
 
@@ -1685,4 +1882,31 @@ class Moduleadmin extends Command {
 		$this->emailbody[] = $line;
 	}
 
+	public function handleErrors($errors, $modulename){
+		$this->writeln("Unable to install module ${modulename}:", "error", false);
+		$this->writeln(' - ' . implode("\n - ", $errors), "error", false);
+		$this->addToEmail(sprintf(_("Module %s installation failed with errors: %s"), $modulename, implode("\n -", $errors)));
+		return false;
+	}
+
+	/**
+	 * Check module conflicts
+	 *
+	 * @param mixed $module String or Array
+	 * @return array
+	 */
+	public function checkConflicts($module) {
+		if($this->force) {
+			return [
+				'breaking' => false
+			];
+		}
+		if($this->skipbreaking === true) {
+			return [
+				'breaking' => false
+			];
+		}
+		return $this->FreePBX->Modules->checkConflicts($module);
+	}
+
 }
