@@ -57,22 +57,11 @@ if (!function_exists('json_last_error_msg')) {
 function freepbx_log_security($txt) {
 	$path = FreePBX::Config()->get('ASTLOGDIR');
 	$log_file = $path.'/freepbx_security.log';
+	/** Monolog */
+	$logger = FreePBX::Logger()->getDriver('freepbx_security');
 
-	$tz = date_default_timezone_get();
-	if (!$tz) {
-		$tz = 'America/Los_Angeles';
-	}
-	date_default_timezone_set($tz);
-	$tstamp		= date("Y-m-d H:i:s");
-
-	// Don't append if the file is greater than ~2G since some systems fail
-	//
-	$size = file_exists($log_file) ? sprintf("%u", filesize($log_file)) + strlen($txt) : 0;
-	if ($size > 2000000000) {
-		unlink($log_file);
-	}
 	if(!FreePBX::Config()->get('AMPDISABLELOG') && FreePBX::Config()->get('LOG_OUT_MESSAGES')) {
-		file_put_contents($log_file, "[$tstamp] $txt\n", FILE_APPEND);
+		$logger->notice($txt);
 	}
 }
 /**
@@ -83,82 +72,9 @@ function freepbx_log_security($txt) {
  * @param  string   The error message
  */
 function freepbx_log($level, $message) {
-	global $amp_conf;
-
-	$php_error_handler = false;
-	$bt = debug_backtrace();
-
-	if (isset($bt[1]) && $bt[1]['function'] == 'freepbx_error_handler') {
-		$php_error_handler = true;
-	} elseif (isset($bt[1]) && $bt[1]['function'] == 'out' || $bt[1]['function'] == 'die_freepbx') {
-		$file_full = $bt[1]['file'];
-		$line = $bt[1]['line'];
-	} elseif (basename($bt[0]['file']) == 'notifications.class.php') {
-		$file_full = $bt[2]['file'];
-		$line = $bt[2]['line'];
-	} else {
-		$file_full = $bt[0]['file'];
-		$line = $bt[0]['line'];
-	}
-
-	if (!$php_error_handler) {
-		$file_base = basename($file_full);
-		$file_dir  = basename(dirname($file_full));
-		$txt = sprintf("[%s] (%s/%s:%s) - %s\n", $level, $file_dir, $file_base, $line, $message);
-	} else {
-		// PHP Error Handler provides it's own formatting
-		$txt = sprintf("[%s-%s\n", $level, $message);
-	}
-
-	// if it is not set, it's probably an initial installation so we want to log something
-	if (!isset($amp_conf['AMPDISABLELOG']) || !$amp_conf['AMPDISABLELOG']) {
-		$log_type = isset($amp_conf['AMPSYSLOGLEVEL']) ? $amp_conf['AMPSYSLOGLEVEL'] : 'FILE';
-		switch ($log_type) {
-			case 'LOG_EMERG':
-			case 'LOG_ALERT':
-			case 'LOG_CRIT':
-			case 'LOG_ERR':
-			case 'LOG_WARNING':
-			case 'LOG_NOTICE':
-			case 'LOG_INFO':
-			case 'LOG_DEBUG':
-				syslog(constant($log_type),"FreePBX - $txt");
-				break;
-			case 'SQL':     // Core will remove these settings once migrated,
-			case 'LOG_SQL': // default to FILE during any interim steps.
-			case 'FILE':
-			default:
-				// during initial install, there may be no log file provided because the script has not fully bootstrapped
-				// so we will default to a pre-install log file name. We will make a file name mandatory with a proper
-				// default in FPBX_LOG_FILE
-				$log_file	= isset($amp_conf['FPBX_LOG_FILE']) ? $amp_conf['FPBX_LOG_FILE'] : '/tmp/freepbx_pre_install.log';
-
-				// PHP Throws an error on install running of install_amp because the tiemzone isn't set. This is something that
-				// should be done in the php.ini file but we will make an attempt to set it to something if we can't derive it
-				// from the date_default_timezone_get() command which goes through heuristics of guessing.
-				//
-				$tz = date_default_timezone_get();
-				if (!$tz) {
-					$tz = 'America/Los_Angeles';
-				}
-				date_default_timezone_set($tz);
-				$tstamp = date("Y-M-d H:i:s");
-
-				// Don't append if the file is greater than ~2G since some systems fail
-				//
-				$size = file_exists($log_file) ? sprintf("%u", filesize($log_file)) + strlen($txt) : 0;
-				if ($size < 2000000000) {
-					$dn = dirname($log_file);
-					if((file_exists($log_file) && is_writable($log_file)) || (!file_exists($log_file) && is_dir($dn) && is_writable($dn))) {
-						file_put_contents($log_file, "[$tstamp] $txt", FILE_APPEND);
-					} else {
-						return false;
-					}
-				}
-				break;
-		}
-	}
-	return true;
+	$freepbx = FreePBX::Create();
+	$logger = $freepbx->Logger;
+	return $logger->log($level, $message);
 }
 
 /**
@@ -262,7 +178,7 @@ function needreload() {
  * Check to see if Apply Changes/Need Reload flag has been set
  * @return bool true if reload needed, otherwise false
  */
-function check_reload_needed() {
+function check_reload_needed($implicit_grant=false) {
 	global $db;
 	global $amp_conf;
 	$sql = "SELECT value FROM admin WHERE variable = 'need_reload'";
@@ -271,8 +187,10 @@ function check_reload_needed() {
 		die_freepbx($sql.$row->getMessage());
 	}
 	//check from amp user if we are allowed to execute apply changes
-	if(!isset($_SESSION["AMP_user"]) || !is_object($_SESSION["AMP_user"]) || !(get_class($_SESSION["AMP_user"]) == 'ampuser') || !$_SESSION["AMP_user"]->checkSection(99)) {
-		return false;
+	if(!$implicit_grant){
+		if(!isset($_SESSION["AMP_user"]) || !is_object($_SESSION["AMP_user"]) || !(get_class($_SESSION["AMP_user"]) == 'ampuser') || !$_SESSION["AMP_user"]->checkSection(99)) {
+			return false;
+		}
 	}
 	return ($row[0] == 'true' || $amp_conf['DEVELRELOAD']);
 }
@@ -658,27 +576,18 @@ function edit_crontab($remove = '', $add = '') {
  */
 function dbug_write($txt, $check = false){
 	global $amp_conf;
-
 	// dbug can be used prior to bootstrapping and initialization, so we set
 	// it if not defined here to a default.
 	//
 	if (!isset($amp_conf['FPBXDBUGFILE'])) {
-		$amp_conf['FPBXDBUGFILE'] = '/var/log/asterisk/freepbx_debug';
+		$amp_conf['FPBXDBUGFILE'] = $amp_conf['ASTLOGDIR'].'/freepbx_debug';
 	}
-
-// If not check set max size just under 2G which is the php limit before it gets upset
-	if($check) { $max_size = 52428800; } else { $max_size = 2000000000; }
-	//optionaly ensure that dbug file is smaller than $max_size
-	$size = file_exists($amp_conf['FPBXDBUGFILE']) ? sprintf("%u", filesize($amp_conf['FPBXDBUGFILE'])) + strlen($txt) : 0;
-	if ($size > $max_size) {
-		file_put_contents($amp_conf['FPBXDBUGFILE'], $txt);
-	} else {
-		file_put_contents($amp_conf['FPBXDBUGFILE'], $txt, FILE_APPEND);
-	}
-
+	$dbugfile = $amp_conf['FPBXDBUGFILE'];
+	FreePBX::Logger()->createLogDriver('dbug', $dbugfile, \FreePBX\Logger::DEBUG, true)->debug($txt);
 	if($amp_conf['PHP_CONSOLE']) {
 		PhpConsole\Connector::getInstance()->getDebugDispatcher()->dispatchDebug($txt, 'dbug');
 	}
+	/** Monolog */
 }
 
 /**
@@ -860,8 +769,8 @@ function dbug_printtree($dir, $indent = "\t") {
  * @return string
  */
 function fpbx_which($app) {
-	$freepbx_conf = freepbx_conf::create();
-	$which = $freepbx_conf->get_conf_setting('WHICH_' . $app);
+	$freepbx_conf = \FreePBX::Config();
+	$which = $freepbx_conf->get('WHICH_' . $app);
 
 	//if we have the location cached return it
 	if (!empty($which) && file_exists($which) && is_executable($which)) {
@@ -873,43 +782,21 @@ function fpbx_which($app) {
 	//otherwise, search for it
 	//ist of posible plases to find which
 
-	$which = array(
-			'which',
-			'/usr/bin/which' //centos/mac osx
-	);
+	$paths = [
+		"/usr/local/sbin",
+		"/usr/local/bin",
+		"/sbin",
+		"/bin",
+		"/usr/sbin",
+		"/usr/bin",
+		"/root/bin"
+	];
 
-	//TODO: Remove which in 14 no needed - Rob Thomas
-	$location = '';
-	foreach ($which as $w) {
-		exec($w . ' ' . $app . ' 2>&1', $path, $ret);
+	$find = new Symfony\Component\Process\ExecutableFinder();
+	$location = $find->find($app,null,$paths);
 
-		//exit if we have a positive find
-		if ($ret === 0) {
-			$location = $path[0];
-			break;
-		}
-	}
-
-	if(empty($location)) {
-		$paths = array(
-			"/usr/local/sbin",
-			"/usr/local/bin",
-			"/sbin",
-			"/bin",
-			"/usr/sbin",
-			"/usr/bin",
-			"/root/bin"
-		);
-		foreach($paths as $path) {
-			if (file_exists($path."/".$app) && is_executable($path."/".$app)) {
-				$location = $path."/".$app;
-				break;
-			}
-		}
-	}
-
-	if(!empty($location) && $freepbx_conf->conf_setting_exists('WHICH_' . $app)) {
-		$freepbx_conf->set_conf_values(array('WHICH_' . $app => $location), true,true);
+	if(!empty($location) && $freepbx_conf->exists('WHICH_' . $app)) {
+		$freepbx_conf->set('WHICH_' . $app, $location, true,true);
 		return $location;
 	} elseif(!empty($location) && !$freepbx_conf->conf_setting_exists('WHICH_' . $app)) {
 		//if we have a path add it to freepbx settings
@@ -1202,91 +1089,7 @@ function generate_message_banner($message,$type='info',$details=array(),$link=''
  * allows FreePBX to update the manager credentials primarily used by Advanced Settings and Backup and Restore.
  */
 function fpbx_ami_update($user=false, $pass=false, $writetimeout = false) {
-	global $amp_conf, $astman, $db, $bootstrap_settings;
-	$conf_file = $amp_conf['ASTETCDIR'] . '/manager.conf';
-	$ret = $ret2 = $ret3 = 0;
-	$output = array();
-
-	if(strpos($amp_conf['ASTETCDIR'],"..") === false && !file_exists($conf_file)) {
-		return;
-	}
-
-	if ($user !== false && $user != '') {
-		$sed_arg = escapeshellarg('s/\s*\[general\].*$/TEMPCONTEXT/;s/\[.*\]/\[' . $amp_conf['AMPMGRUSER'] . '\]/;s/^TEMPCONTEXT$/\[general\]/');
-		exec("sed -i.bak $sed_arg $conf_file", $output, $ret);
-		if ($ret) {
-			freepbx_log(FPBX_LOG_ERROR,sprintf(_("Failed changing AMI user to [%s], internal failure details follow:"),$amp_conf['AMPMGRUSER']));
-			foreach ($output as $line) {
-				freepbx_log(FPBX_LOG_ERROR,sprintf(_("AMI failure details:"),$line));
-			}
-		}
-	}
-
-	if ($pass !== false && $pass != '') {
-		unset($output);
-		$sed_arg = escapeshellarg('s/secret\s*=.*$/secret = ' . $amp_conf['AMPMGRPASS'] . '/');
-		exec("sed -i.bak $sed_arg $conf_file", $output2, $ret2);
-		if ($ret2) {
-			freepbx_log(FPBX_LOG_ERROR,sprintf(_("Failed changing AMI password to [%s], internal failure details follow:"),$amp_conf['AMPMGRPASS']));
-			foreach ($output2 as $line) {
-				freepbx_log(FPBX_LOG_ERROR,sprintf(_("AMI failure details:"),$line));
-			}
-		}
-
-		// We've changed the password, let's update the notification
-		//
-		$nt = notifications::create($db);
-		$freepbx_conf =& freepbx_conf::create();
-		if ($amp_conf['AMPMGRPASS'] == $freepbx_conf->get_conf_default_setting('AMPMGRPASS')) {
-			if (!$nt->exists('core', 'AMPMGRPASS')) {
-				$nt->add_warning('core', 'AMPMGRPASS', _("Default Asterisk Manager Password Used"), _("You are using the default Asterisk Manager password that is widely known, you should set a secure password"));
-			}
-		} else {
-			$nt->delete('core', 'AMPMGRPASS');
-		}
-	}
-
-	//attempt to set writetimeout
-	unset($output);
-	if ($writetimeout) {
-		$sed_arg = escapeshellarg('s/writetimeout\s*=.*$/writetimeout = ' . $amp_conf['ASTMGRWRITETIMEOUT'] . '/');
-		exec("sed -i.bak $sed_arg $conf_file", $output3, $ret3);
-		if ($ret3) {
-			freepbx_log(FPBX_LOG_ERROR,sprintf(_("Failed changing AMI writetimout to [%s], internal failure details follow:"),$amp_conf['ASTMGRWRITETIMEOUT']));
-			foreach ($output3 as $line) {
-				freepbx_log(FPBX_LOG_ERROR,sprintf(_("AMI failure details:"),$line));
-			}
-		}
-	}
-	if ($ret || $ret2 || $ret3) {
-		dbug("aborting early because previous errors");
-		return false;
-	}
-	if (is_object($astman) && method_exists($astman, "connected") && $astman->connected()) {
-		$ast_ret = $astman->Command('module reload manager');
-	} else {
-		unset($output);
-		dbug("no astman connection so trying to force through linux command line");
-		exec(fpbx_which('asterisk') . " -rx 'module reload manager'", $output, $ret2);
-		if ($ret2) {
-			freepbx_log(FPBX_LOG_ERROR,_("Failed to reload AMI, manual reload will be necessary, try: [asterisk -rx 'module reload manager']"));
-		}
-	}
-	if (is_object($astman) && method_exists($astman, "connected") && $astman->connected()) {
-		$astman->disconnect();
-	}
-	if (!is_object($astman) || !method_exists($astman, "connected")) {
-		//astman isn't initiated so escape
-		return true;
-	}
-	if (!$res = $astman->connect($amp_conf["ASTMANAGERHOST"] . ":" . $amp_conf["ASTMANAGERPORT"], $amp_conf["AMPMGRUSER"] , $amp_conf["AMPMGRPASS"], $bootstrap_settings['astman_events'])) {
-		// couldn't connect at all
-		freepbx_log(FPBX_LOG_CRITICAL,"Connection attmempt to AMI failed");
-	} else {
-		$bmo = FreePBX::create();
-		$bmo->astman = $astman;
-	}
-	return true;
+	return \FreePBX::Framework()->amiUpdate($user, $pass, $writetimeout);
 }
 
 /**
@@ -1608,6 +1411,23 @@ function freepbxGetSanitizedRequest($definition = FILTER_SANITIZE_STRING, $add_e
 	return $request;
 }
 
+/**
+ * IsAsteriskSupported
+ *
+ * @param  string $version
+ * @return array
+ */
+function IsAsteriskSupported($version){
+	$min 				= "11";
+	$max 				= "21";
+	$result["status"] 	= true;
+	if(version_compare($version, $min, "lt") || version_compare($version, $max, "ge")) {
+		$result["status"] 	= false;
+		$result["message"] 	= _("Supported Asterisk versions:")." 11, 12, 13, 14, 15, 16, 17, 18, 19, 20.";
+	}
+	return $result;	
+}
+
 /**
  * @method versions_min
  *
@@ -1629,3 +1449,55 @@ function version_min($origin, $list){
 	}
 	return NULL;
 }
+
+/**
+ * checkFreeSpace
+ * @pram $allowedSpace = 2GB
+ * @return boolean
+ */
+function checkFreeSpace($allowedSpace)
+{
+	$availableSpace = disk_free_space("/");
+	$availableSpace = round($availableSpace/1073741824,2); //available space in GB
+	if ($availableSpace <= $allowedSpace) {
+		return array('status'=>false, 'available_space'=> $availableSpace);
+	} else {
+		return array('status'=>true, 'available_space'=> $availableSpace);
+	}
+}
+
+/**
+ * Fetch the from Email address to use while sending the email
+ * If Sysadmin is installed and has the fromEmail then use that one
+ * else use Advanced Settings define 'AMPUSERMANEMAILFROM' field
+ */
+function fetchFromEmail($isWelcomeEmail=false) {
+	$from = '';
+	$femail = \FreePBX::Config()->get('AMPUSERMANEMAILFROM');
+	if (!function_exists('sysadmin_get_storage_email') &&
+			\FreePBX::Modules()->checkStatus('sysadmin') &&
+			file_exists(FreePBX::Config()->get('AMPWEBROOT').'/admin/modules/sysadmin/functions.inc.php')) {
+		include \FreePBX::Config()->get('AMPWEBROOT').'/admin/modules/sysadmin/functions.inc.php';
+	}
+
+	if(function_exists('sysadmin_get_storage_email')) {
+		$emails = sysadmin_get_storage_email();
+		if(!empty($emails['fromemail'])) {
+			$femail = $emails['fromemail'];
+		}
+	}
+	/* validate if from email is a valid email */
+	if (isset($femail) && !empty($femail) &&
+			filter_var($femail, FILTER_VALIDATE_EMAIL)) {
+		$from = $femail;
+	} else {
+		/* for backward support where userman using below default from email for welcome emails*/
+		if ($isWelcomeEmail) {
+			$from = 'freepbx@freepbx.org';
+		} else {
+			$from = get_current_user() . '@' . gethostname();
+		}
+	}
+	return $from;
+}
+
