@@ -31,7 +31,7 @@ function callrecording_destinations() {
 function callrecording_destination_popovers() {
 	global $module_page;
 	if ($module_page != 'callrecording') {
-		$ret['callrecording'] = 'Call Recording';
+		$ret['callrecording'] = _('Call Recording');
 	} else {
 		$ret = array();
 	}
@@ -68,16 +68,14 @@ function callrecording_get_config($engine) {
 	global $core_conf;
 	switch ($engine) {
 	case 'asterisk':
-		if(version_compare($version, "13.0", "lt")) {
-			$ext->addGlobal('MIXMON_BEEP', '');
+
+		$period =  FreePBX::Config()->get_conf_setting('CALLREC_BEEP_PERIOD');
+		if (!empty($period)) {
+			$ext->addGlobal('MIXMON_BEEP', "B($period)");
 		} else {
-			$period =  FreePBX::Config()->get_conf_setting('CALLREC_BEEP_PERIOD');
-			if (!empty($period)) {
-				$ext->addGlobal('MIXMON_BEEP', "B($period)");
-			} else {
-				$ext->addGlobal('MIXMON_BEEP', '');
-			}
+			$ext->addGlobal('MIXMON_BEEP', '');
 		}
+
 		$record_status =  FreePBX::Config()->get_conf_setting('CALL_REC_OPTION');
 		if($record_status){
 			$ext->addGlobal('MONITOR_REC_OPTION', 'b');
@@ -129,7 +127,6 @@ function callrecording_get_config($engine) {
 		; ARG2: called_exten
 		; ARG3: action (if we know it)
 		;       force (== always), yes, dontcare, no, never
-		;
 		*/
 
 
@@ -164,11 +161,6 @@ function callrecording_get_config($engine) {
 		$ext->add($context, $exten, 'recorderror', new ext_playback('something-terribly-wrong,error'));
 		$ext->add($context, $exten, '', new ext_hangup());
 
-		// If we don't have a current mode, and we were explicitly given a command, we can set our current mode
-		// to that. This is what we CURRENTLY think we should be doing. This may change if it's an exten.
-		//  .. Disabled. Legacy code, should be unneeded now.
-		// $ext->add($context, $exten, 'checkaction', new ext_execif('$["${REC_POLICY_MODE}"="" & "${ARG3}"!=""]','Set','__REC_POLICY_MODE=${TOUPPER(${ARG3})}'));
-
 		// Now jump to the dialplan handler. If it doesn't exist, do the generic test (rg, force, q use these).
 		$ext->add($context, $exten, 'checkaction',  new ext_gotoif('$[${DIALPLAN_EXISTS('.$context.',${ARG1})}]', $context.',${ARG1},1'));
 
@@ -197,7 +189,7 @@ function callrecording_get_config($engine) {
 
 		$ext->add($context, $exten, 'delayed', new ext_noop('Detected legacy "delayed" entry. Mapping to "yes"'));
 		// YES: Start recording if we haven't been told otherwise.
-		$ext->add($context, $exten, 'yes', new ext_execif('$["${REC_POLICY_MODE}" = "NEVER" | "${REC_POLICY_MODE}" = "NO" | "${REC_STATUS}" = "RECORDING"]', 'Return'));
+		$ext->add($context, $exten, 'yes', new ext_execif('$["${REC_POLICY_MODE_SAVE}" = "never" | "${REC_POLICY_MODE_SAVE}" = "NEVER" | "${REC_POLICY_MODE}" = "NEVER" | "${REC_POLICY_MODE}" = "NO" | "${REC_STATUS}" = "RECORDING"]', 'Return'));
 		$ext->add($context, $exten, '', new ext_set('__REC_POLICY_MODE', 'YES'));
 		$ext->add($context, $exten, '', new ext_goto('startrec'));
 
@@ -215,8 +207,10 @@ function callrecording_get_config($engine) {
 		if(version_compare($version, "12.0", "lt")) {
 			$ext->add($context, $exten, '', new ext_set('AUDIOHOOK_INHERIT(MixMonitor)','yes'));
 		}
-		$ext->add($context, $exten, '', new ext_set('__CALLFILENAME','${ARG2}-${ARG3}-${FROMEXTEN}-${TIMESTR}-${UNIQUEID}'));
-		$ext->add($context, $exten, '', new ext_mixmonitor('${MIXMON_DIR}${YEAR}/${MONTH}/${DAY}/${CALLFILENAME}.${MON_FMT}','a${MONITOR_REC_OPTION}i(LOCAL_MIXMON_ID)${MIXMON_BEEP}','${MIXMON_POST}'));
+		$ext->add($context, $exten, '', new ext_execif('$["${FROMQUEUEEXTEN}" != ""]', 'Set', 'RECFROMEXTEN=${FROMQUEUEEXTEN}'));
+		$ext->add($context, $exten, '', new ext_execif('$["${FROMQUEUEEXTEN}" == ""]', 'Set', 'RECFROMEXTEN=${FROMEXTEN}'));
+		$ext->add($context, $exten, '', new ext_set('__CALLFILENAME','${ARG2}-${ARG3}-${RECFROMEXTEN}-${TIMESTR}-${UNIQUEID}'));
+		$ext->add($context, $exten, '', new ext_mixmonitor('${MIXMON_DIR}${YEAR}/${MONTH}/${DAY}/${CALLFILENAME}.${MON_FMT}','a${MONITOR_REC_OPTION}i(${LOCAL_MIXMON_ID})${MIXMON_BEEP}','${MIXMON_POST}'));
 		$ext->add($context, $exten, '', new ext_set('__MIXMON_ID', '${LOCAL_MIXMON_ID}'));
 		$ext->add($context, $exten, '', new ext_set('__RECORD_ID', '${CHANNEL(name)}'));
 		$ext->add($context, $exten, '', new ext_set('__REC_STATUS','RECORDING'));
@@ -290,7 +284,19 @@ function callrecording_get_config($engine) {
 
 		// Originator of the call wins. Always out/internal.
 		$ext->add($context, $exten, 'caller', new ext_set('RECMODE','${DB(AMPUSER/${FROMEXTEN}/recording/out/internal)}'));
-		$ext->add($context, $exten, '', new ext_execif('$[!${LEN(${RECMODE})}]','Set', 'RECMODE=dontcare'));
+		$ext->add($context, $exten, '', new ext_set('CALLERRECMODE','${DB(AMPUSER/${FROMEXTEN}/recording/out/internal)}'));
+		// check callee set higher preference mode
+		$ext->add($context, $exten, '', new ext_set('CALEERECMODE','${DB(AMPUSER/${ARG2}/recording/in/internal)}'));
+		$ext->add($context, $exten, '', new ext_gotoif('$[!${LEN(${CALEERECMODE})}]','processnormal'));
+		// check caller mode preference is greater than caller eg: Force,Never
+		$ext->add($context, $exten, '', new ext_execif('$["${CALLER_PRI}" = "${CALLEE_PRI}" & "${CALEERECMODE}"="never"]','Set','RECMODE=${CALEERECMODE}'));
+		// check caller mode preference is greater than caller eg: No and callee Force
+		$ext->add($context, $exten, '', new ext_execif('$["${CALLER_PRI}" = "${CALLEE_PRI}" & "${CALEERECMODE}"="force" & "${CALLERRECMODE}"="no"]','Set','RECMODE=${CALEERECMODE}'));
+		// caller never and callee force : result calle ->force 
+		$ext->add($context, $exten, '', new ext_execif('$["${CALLER_PRI}" = "${CALLEE_PRI}" & "${CALEERECMODE}"="force" & "${CALLERRECMODE}"="never"]','Set','RECMODE=${CALEERECMODE}'));
+		// caller force and callee never : result calle ->Never 
+		$ext->add($context, $exten, '', new ext_execif('$["${CALLER_PRI}" = "${CALLEE_PRI}" & "${CALEERECMODE}"="never" & "${CALLERRECMODE}"="force"]','Set','RECMODE=${CALEERECMODE}'));
+		$ext->add($context, $exten, 'processnormal', new ext_execif('$[!${LEN(${RECMODE})}]','Set', 'RECMODE=dontcare'));
 		// If we don't care, then the callee gets to pick.
 		$ext->add($context, $exten, '', new ext_execif('$["${RECMODE}"="dontcare"]','Set', 'RECMODE=${CALLEE}'));
 		$ext->add($context, $exten, '', new ext_gosub('1', 'recordcheck', false, '${RECMODE},${CALLTYPE},${ARG2}'));
@@ -352,7 +358,7 @@ function callrecording_get_config($engine) {
 			$ext->add($context, $exten, '', new ext_set('AUDIOHOOK_INHERIT(MixMonitor)','yes'));
 		}
 		$ext->add($context, $exten, '', new ext_set('MONITOR_FILENAME','${MIXMON_DIR}${YEAR}/${MONTH}/${DAY}/${CALLFILENAME}'));
-		$ext->add($context, $exten, '', new ext_mixmonitor('${MONITOR_FILENAME}.${MON_FMT}','${MONITOR_OPTIONS}${MIXMON_BEEP}','${MIXMON_POST}'));
+		$ext->add($context, $exten, '', new ext_mixmonitor('${MONITOR_FILENAME}.${MON_FMT}','${EVAL(${MONITOR_OPTIONS})}${MIXMON_BEEP}','${EVAL(${MIXMON_POST})}'));
 		$ext->add($context, $exten, '', new ext_set('__REC_STATUS','RECORDING'));
 		$ext->add($context, $exten, '', new ext_set('CDR(recordingfile)','${CALLFILENAME}.${MON_FMT}'));
 		$ext->add($context, $exten, '', new ext_return(''));
@@ -381,7 +387,7 @@ function callrecording_get_config($engine) {
 		$ext->add($context, $exten, '', new ext_execif('$["${REC_STATUS}"="STOPPED"]','Playback','beep&beep'));
 		$ext->add($context, $exten, '', new ext_gotoif('$["${REC_STATUS}"="STOPPED"]','end'));
 		$ext->add($context, $exten, '', new ext_gotoif('$["${REC_STATUS}"="RECORDING"]','startrec'));
-		$ext->add($context, $exten, 'startrec', new ext_mixmonitor('${MIXMON_DIR}${YEAR}/${MONTH}/${DAY}/${CALLFILENAME}.${MON_FMT}','ai(LOCAL_MIXMON_ID)${MIXMON_BEEP}','${MIXMON_POST}'));
+		$ext->add($context, $exten, 'startrec', new ext_mixmonitor('${MIXMON_DIR}${YEAR}/${MONTH}/${DAY}/${CALLFILENAME}.${MON_FMT}','ai(LOCAL_MIXMON_ID)${MIXMON_BEEP}','${EVAL(${MIXMON_POST})}'));
 		$ext->add($context, $exten, '', new ext_execif('$["${REC_STATUS}"="RECORDING"]','Playback','beep'));
 		$ext->add($context, $exten, 'denied', new ext_execif('$["${ONETOUCH_REC_SCRIPT_STATUS:0:6}"="DENIED"]','Playback','access-denied'));
 		$ext->add($context, $exten, 'end', new ext_macroexit());
@@ -455,74 +461,23 @@ function callrecording_hookGet_config($engine) {
 /**  Get a list of all callrecording
  */
 function callrecording_list() {
-	global $db;
-	$sql = "SELECT callrecording_id, description, callrecording_mode, dest FROM callrecording ORDER BY description ";
-	$results = $db->getAll($sql, DB_FETCHMODE_ASSOC);
-	if(DB::IsError($results)) {
-		die_freepbx($results->getMessage()."<br><br>Error selecting from callrecording");
-	}
-	return $results;
+	return FreePBX::Callrecording()->listAll();
 }
 
 function callrecording_get($callrecording_id) {
-	global $db;
-	$sql = "SELECT callrecording_id, description, callrecording_mode, dest FROM callrecording WHERE callrecording_id = ".$db->escapeSimple($callrecording_id);
-	$row = $db->getRow($sql, DB_FETCHMODE_ASSOC);
-	if(DB::IsError($row)) {
-		die_freepbx($row->getMessage()."<br><br>Error selecting row from callrecording");
-	}
-
-	return $row;
+	return FreePBX::Callrecording()->getRecording($callrecording_id);
 }
 
 function callrecording_add($description, $callrecording_mode, $dest) {
-	global $db;
-	global $amp_conf;
-
-	/*
-		Test if description already exists.
-		It's possible to add another one through page refresh.
-		So, for avoid to add the same record, we launch this test.
-	*/
-	$sql = "SELECT * FROM callrecording WHERE description = :description";
-	$stm = $db->prepare($sql);
-	$stm->execute(array(":description" => $description));	
-	$ret = $stm->fetch(\PDO::FETCH_ASSOC);
-	if(empty($ret["description"])){
-		$sql = "INSERT INTO callrecording (description, callrecording_mode, dest) VALUES (".
-			"'".$db->escapeSimple($description)."', ".
-			"'".$db->escapeSimple($callrecording_mode)."', ".
-			"'".$db->escapeSimple($dest)."')";
-		$result = $db->query($sql);
-		if(DB::IsError($result)) {
-			die_freepbx($result->getMessage().$sql);
-		}
-		if(method_exists($db,'insert_id')) {
-			$id = $db->insert_id();
-		} else {
-			$id = $amp_conf["AMPDBENGINE"] == "sqlite3" ? sqlite_last_insert_rowid($db->connection) : mysql_insert_id($db->connection);
-		}
-		return($id);		
-	}
-	return;
+	return FreePBX::Callrecording()->add($callrecording_id, $description, $callrecording_mode, $dest);
 }
 
 function callrecording_delete($callrecording_id) {
-	global $db;
-	$sql = "DELETE FROM callrecording WHERE callrecording_id = ?";
-	$result = $db->query($sql,array($callrecording_id));
-	if(DB::IsError($result)) {
-		die_freepbx($result->getMessage().$sql);
-	}
+	return FreePBX::Callrecording()->delete($callrecording_id);
 }
 
 function callrecording_edit($callrecording_id, $description, $callrecording_mode, $dest) {
-	global $db;
-	$sql = "UPDATE callrecording SET description = ?, callrecording_mode = ?, dest = ? WHERE callrecording_id = ?";
-	$result = $db->query($sql,array($description,$callrecording_mode,$dest,$callrecording_id));
-	if(DB::IsError($result)) {
-		die_freepbx($result->getMessage().$sql);
-	}
+	return FreePBX::Callrecording()->edit($callrecording_id, $description, $callrecording_mode, $dest);
 }
 
 function callrecording_hook_core($viewing_itemid, $target_menuid){
@@ -538,9 +493,6 @@ function callrecording_hook_core($viewing_itemid, $target_menuid){
 			$opts		= explode('/', $extdisplay);
 			$extension	= $opts['0'];
 			$cidnum		= isset($opts['1']) ? $opts['1'] : '';
-		}else{
-			$extension 	= $extension;
-			$cidnum		= $cidnum;
 		}
 
 		//update if we have enough info
@@ -558,22 +510,14 @@ function callrecording_hook_core($viewing_itemid, $target_menuid){
 		$request = $_REQUEST;
 		$request['route_id'] = isset($request['id'])?$request['id']:'';
 		$route_id	= isset($request['route_id']) ? $request['route_id'] : (isset($request['extdisplay']) ? $request['extdisplay'] : '');
-		if (!empty($_SESSION['callrecordingAddRoute'])) {
-			$callrecording = $_SESSION['callrecordingAddRoute'];
-		} else {
-			$callrecording = callrecording_display_get($target_menuid, $route_id);
-		}
+		$callrecording = callrecording_display_get($target_menuid, $route_id);
 		$helptext = _("This sets the call recording behavior for calls going out this route. Please read the wiki for information on what these settings mean.");
 		break;
 	}
 	$html = '';
-	//if ($target_menuid == 'did'){
 	if ($target_menuid == 'did' || $target_menuid == 'routing') {
 		global $tabindex;
 		$html = '<!--CALL RECORDING HOOK-->';
-		$html .= '<div class="well well-info">';
-		$html .= _("Note that the meaning of these options has changed.")." <a href='http://wiki.freepbx.org/display/FPG/Call+Recording+walk+through'>"._("Please read the wiki for futher information on these changes.").'</a>';
-		$html .= '</div>';
 		$html .= '<div class="element-container">';
 		$html .= '<div class="row">';
 		$html .= '<div class="col-md-12">';
@@ -617,9 +561,6 @@ function callrecording_hook_core($viewing_itemid, $target_menuid){
 	return $html;
 }
 
-//Moved in to BMO hook, not aliased as this shouldnt be called by anything
-//function callrecording_hookProcess_core($viewing_itemid, $request) {}
-
 
 function callrecording_adjustroute($route_id,$action,$callrecording='') {
 	global $db;
@@ -627,23 +568,19 @@ function callrecording_adjustroute($route_id,$action,$callrecording='') {
 	$route_id = $db->escapeSimple($route_id);
 	$callrecording = $db->escapeSimple($callrecording);
 
-	//dbug("in adjustroute with route_id: $route_id, action: $action, callrecording: $callrecording");
 	switch ($action) {
 	case 'delroute':
 		callrecording_display_delete($dispname,$route_id);
 		break;
-		case 'addroute';
+	case 'addroute';
 		if ($callrecording != '') {
-			// we don't have the route_id yet, it hasn't been inserted yet :(, put it in the session
-			// and when returned it will be available on the redirect_standard
-			$_SESSION['callrecordingAddRoute'] = $callrecording;
+			callrecording_display_update($dispname, $callrecording, $route_id);
 		}
 		break;
-		case 'delayed_insert_route';
+	case 'delayed_insert_route';
 		callrecording_display_update($dispname, $callrecording, $route_id);
 		break;
-		case 'editroute';
-		//dbug("in editroute ready to insert dispnam: $dispname, route: $route_id, mode $callrecording");
+	case 'editroute';
 		if ($callrecording != '') {
 			callrecording_display_update($dispname, $callrecording, $route_id);
 		} else {
@@ -656,7 +593,6 @@ function callrecording_adjustroute($route_id,$action,$callrecording='') {
 function callrecording_display_get($display, $extension=null,$cidnum=null){
 	global $db;
 
-	//dbug("display_get with display: $display, exten $extension, cid $cidnum", $_REQUEST);
 	switch ($display) {
 	case 'did':
 		if($extension || $cidnum || (isset($_REQUEST['extdisplay']) && $_REQUEST['extdisplay']=='/') || (isset($_REQUEST['display']) && $_REQUEST['display']=='did')){
@@ -671,9 +607,7 @@ function callrecording_display_get($display, $extension=null,$cidnum=null){
 			if ($cidnum !== null) {
 				$params[] = $cidnum;
 			}
-			//dbug("executing getOne code: $sql", $params);
 			$mode=$db->getOne($sql, $params);
-			//$mode=$db->getOne($sql, array($display, $extension, $cidnum));
 		}else{
 			$sql="SELECT callrecording_module.*,incoming.pricid FROM callrecording_module, incoming WHERE callrecording_module.cidnum=incoming.cidnum AND callrecording_module.extension=incoming.extension AND callrecording_module.display = '$display'";
 			$mode=$db->getAll($sql, DB_FETCHMODE_ASSOC);
@@ -687,9 +621,7 @@ function callrecording_display_get($display, $extension=null,$cidnum=null){
 			if ($extension !== null) {
 				$params[] = $extension;
 			}
-			//dbug("executing getOne code: $sql", $params);
 			$mode=$db->getOne($sql, $params);
-			//$mode=$db->getOne($sql, array($display, $extension, $cidnum));
 		} else {
 			$sql="SELECT extension as route_id, callrecording FROM callrecording_module WHERE display = '$display'";
 			$mode=$db->getAll($sql, DB_FETCHMODE_ASSOC);
@@ -712,8 +644,6 @@ function callrecording_display_update($display,$recording_code=null,$extension=n
 	if ($cidnum !== null) {
 		$params[] = $cidnum;
 	}
-	//dbug("executing delete code: $sql", $params);
-	//$db->query($sql,array($display,$extension,$cidnum));
 	$db->query($sql, $params);
 	if(isset($recording_code) && $recording_code!=''){
 		$sql="INSERT INTO callrecording_module (display,extension,cidnum,callrecording) VALUES (?, ?, ?,?)";
