@@ -151,6 +151,41 @@ class Cdr implements \BMO {
 	public function ucpDelGroup($id,$display,$data) {
 	}
 
+	/* UCP template to get the user assigned vm extension details
+	* @defaultexten is the default_extensionof the userman userid
+	* @userid is userman user id
+	* @widget is an array we need to replace few item based on the userid
+	*/
+	public function getWidgetListByModule($defaultexten, $userid,$widget) {
+		// if the widget_type_id is not defaultextension and widget_type_id is not in extensions
+		// then return only the defaultexten details
+		$widgets = array();
+		$widget_type_id = $widget['widget_type_id'];// this will be an extension number
+		$enabled = $this->FreePBX->Ucp->getCombinedSettingByID($userid,'Cdr','enable');
+		if (!$enabled) {
+			return false;
+		}
+		$extensions = $this->FreePBX->Ucp->getCombinedSettingByID($userid,'Cdr','assigned');
+		$extensions = is_array($extensions)?$extensions:[];
+		if(in_array($widget_type_id,$extensions)){
+			// nothing to do return the same widget
+			return $widget;
+		}else {// sent the default extension
+			$data = $this->FreePBX->Core->getDevice($defaultexten);
+			if(empty($data) || empty($data['description'])) {
+				$data = $this->FreePBX->Core->getUser($defaultexten);
+				$name = $data['name'];
+			} else {
+				$name = $data['description'];
+			}
+			$widget['widget_type_id'] = $defaultexten;
+			$widget['name'] = $name;
+			return $widget;
+		}
+	return false;
+	}
+
+
 	public function ucpAddGroup($id, $display, $data) {
 		$this->ucpUpdateGroup($id,$display,$data);
 	}
@@ -266,7 +301,7 @@ class Cdr implements \BMO {
 			$cdrassigned = array('self');
 		}
 		foreach(core_users_list() as $list) {
-			$ausers[$list[0]] = $list[1] . " &#60;".$list[0]."&#62;";
+			$ausers[$list[0]] = sprintf("%s <%s>", $list[1], $list[0]);
 		}
 		$html[0] = array(
 			"title" => _("Call History"),
@@ -295,6 +330,10 @@ class Cdr implements \BMO {
 
 	}
 
+	public function getDbTable() {
+		return $this->db_table;
+	}
+
 	public function ajaxRequest($req, &$setting) {
 		$setting['authenticate'] = true;
 		$setting['allowremote'] = false;
@@ -369,6 +408,35 @@ class Cdr implements \BMO {
 		return $recording;
 	}
 
+	public function getAllCalls($page=1,$orderby='date',$order='desc',$search='',$limit=100) {
+		$start = ($limit * ($page - 1));
+		$end = $limit;
+		switch($orderby) {
+			case 'description':
+				$orderby = 'clid';
+			break;
+			case 'duration':
+				$orderby = 'duration';
+			break;
+			case 'date':
+			default:
+				$orderby = 'timestamp';
+			break;
+		}
+		$order = ($order == 'desc') ? 'desc' : 'asc';
+		if(!empty($search)) {
+			$sql = "SELECT *, UNIX_TIMESTAMP(calldate) As timestamp FROM ".$this->db_table." WHERE (clid LIKE :search OR src LIKE :search OR dst LIKE :search) ORDER by $orderby $order LIMIT $start,$end";
+			$sth = $this->cdrdb->prepare($sql);
+			$sth->execute(array(':search' => '%'.$search.'%'));
+		} else {
+			$sql = "SELECT *, UNIX_TIMESTAMP(calldate) As timestamp FROM ".$this->db_table." ORDER by $orderby $order LIMIT $start,$end";
+			$sth = $this->cdrdb->prepare($sql);
+			$sth->execute();
+		}
+		$calls = $sth->fetchAll(\PDO::FETCH_ASSOC);
+		return $calls;
+	}
+
 	/**
 	 * Get all CDR call records
 	 * @param int  $extension The extension
@@ -378,7 +446,7 @@ class Cdr implements \BMO {
 	 * @param string  $search    The search string to use
 	 * @param integer $limit     The number of results to return
 	 */
-	public function getCalls($extension,$page=1,$orderby='date',$order='desc',$search='',$limit=100) {
+	public function getCalls($extension,$page=1,$orderby='date',$order='desc',$search='',$limit=100,$fromAPI=false) {
 		$start = ($limit * ($page - 1));
 		$end = $limit;
 		switch($orderby) {
@@ -411,6 +479,8 @@ class Cdr implements \BMO {
 			if(empty($call['src']) && preg_match('/\/(.*)\-/',$call['channel'],$matches)) {
 				$call['src'] = $matches[1];
 			}
+			//This Check $fromAPI to avoid to send the unwanted data to DPMA Call Log API only.
+			if(!$fromAPI) {
 			if($call['duration'] > 59) {
 				$min = floor($call['duration'] / 60);
 				if($min > 59) {
@@ -425,6 +495,7 @@ class Cdr implements \BMO {
 			$call['recordingformat'] = !empty($call['recordingfile']) ? strtolower(pathinfo($call['recordingfile'],PATHINFO_EXTENSION)) : '';
 			$call['recordingfile'] = $this->processPath($call['recordingfile']);
 			$call['requestingExtension'] = $extension;
+			}
 		}
 		return $calls;
 	}
