@@ -88,6 +88,11 @@ foreach ($_POST as $k => $v) {
 	$_POST[$k] = preg_replace('/;/', ' ', $dbcdr->escapeSimple($v));
 }
 
+//if need_csv is true then need_html should be true
+if (isset($_POST['need_csv']) ) {
+	$_POST['need_html']='true';
+}
+
 $h_step = 30;
 if(!isset($_POST['need_csv'])) {
 ?>
@@ -167,7 +172,7 @@ for ( $i = 2000; $i <= date('Y'); $i++) {
 <tr>
 <td><label for="Report Type"><?php echo _("Report Type")?> : </label></td>
 <td>
-<input <?php if ( (empty($_POST['need_html']) && empty($_POST['need_chart']) && empty($_POST['need_chart_cc']) && empty($_POST['need_csv'])) || ( ! empty($_POST['need_html']) &&  $_POST['need_html'] == 'true' ) ) { echo 'checked="checked"'; } ?> type="checkbox" name="need_html" value="true" /> : <?php echo _("CDR search")?><br />
+<input <?php if ( (empty($_POST['need_html']) && empty($_POST['need_chart']) && empty($_POST['need_chart_cc']) && empty($_POST['need_csv'])) || ( ! empty($_POST['need_html']) &&  $_POST['need_html'] == 'true' ) ) { echo 'checked="checked"'; } ?> checked='checked' type="checkbox" name="need_html" value="true" /> : <?php echo _("CDR search")?><br />
 <input <?php if ( ! empty($_POST['need_csv']) && $_POST['need_csv'] == 'true' ) { echo 'checked="checked"'; } ?> type="checkbox" name="need_csv" value="true" /> : <?php echo _("CSV File")?><br/>
 <input <?php if ( ! empty($_POST['need_chart']) && $_POST['need_chart'] == 'true' ) { echo 'checked="checked"'; } ?> type="checkbox" name="need_chart" value="true" /> : <?php echo _("Call Graph")?><br />
 <!--
@@ -573,11 +578,12 @@ foreach ($mod_vars as $key => $val) {
 					if (count($values) > 1) {
 						foreach ($values as $key_like => $value_like) {
 							if ($key_like == 0) {
-								$$key = "AND $key $pre_like LIKE '%$value_like%'";
+								$$key = "AND ($key $pre_like LIKE '%$value_like%'";
 							} else {
  								$$key .= " OR $key $pre_like LIKE '%$value_like%'";
 							}
 						}
+						$$key .= ")";
 					} else {
 						$$key = "AND $key $pre_like LIKE '%$val[0]%'";
 					}
@@ -591,11 +597,12 @@ foreach ($mod_vars as $key => $val) {
 					if (count($values) > 1) {
 						foreach ($values as $key_like => $value_like) {
 							if ($key_like == 0) {
-								$$key = "AND $key $pre_like LIKE '%$value_like'";
+								$$key = "AND ($key $pre_like LIKE '%$value_like'";
 							} else {
 								$$key .= " OR $key $pre_like LIKE '%$value_like'";
 							}
 						}
+						$$key .= ")";
 					} else {
 						$$key = "AND $key $pre_like LIKE '%$val[0]'";
 					}
@@ -635,11 +642,12 @@ foreach ($mod_vars as $key => $val) {
 					if (count($values) > 1) {
 						foreach ($values as $key_like => $value_like) {
 							if ($key_like == 0) {
-								$$key = "AND $key $pre_like LIKE '$value_like%'";
+								$$key = "AND ($key $pre_like LIKE '$value_like%'";
 							} else {
 								$$key .= " OR $key $pre_like LIKE '$value_like%'";
 							}
 						}
+						$$key .= ")";
 					} else {
 						$$key = "AND $key $pre_like LIKE '$val[0]%'";
 					}
@@ -666,7 +674,7 @@ $group = empty($_POST['group']) ? 'day' : $_POST['group'];
 if (isset($cnum)) {
   $cnum_length = strlen($cnum);
   $cnum_type = substr($cnum, 0 ,strpos($cnum , 'cnum') -1);
-  $cnum_remaining = substr($cnum, strpos($cnum , 'cnum'));
+  $cnum_remaining = substr(trim($cnum,"()"), strpos($cnum , 'cnum'));
   $src = str_replace('AND cnum', '', $cnum);
 
   $cnum = "$cnum_type ($cnum_remaining OR src $src)";
@@ -675,17 +683,16 @@ if (isset($cnum)) {
 if (isset($dst)) {
   $dst_length = strlen($dst);
   $dst_type = substr($dst, 0 ,strpos($dst , 'dst') -1);
-  $dst_remaining = substr($dst, strpos($dst , 'dst'));
+  $dst_remaining = substr(trim($dst,"()"), strpos($dst , 'dst'));
   $dstchannel = str_replace('AND dst', '', $dst);
 
   $dst = "$dst_type ($dst_remaining OR dstchannel $dstchannel)";
 }
-
 // Build the "WHERE" part of the query
 $where = "WHERE $date_range $cnum $outbound_cnum $cnam $dst_cnam $did $dst $userfield $accountcode $disposition $duration";
 
 if ( isset($_POST['need_csv']) && $_POST['need_csv'] == 'true' ) {
-	$query = "(SELECT calldate, clid, did, src, dst, dcontext, channel, dstchannel, lastapp, lastdata, duration, billsec, disposition, amaflags, accountcode, uniqueid, userfield, cnum, cnam, outbound_cnum, outbound_cnam, dst_cnam FROM $db_name.$db_table_name $where $order $sort LIMIT $result_limit)";
+	$query = "(SELECT calldate, clid, did, src, dst, dcontext, channel, dstchannel, lastapp, lastdata, duration, billsec, disposition, amaflags, accountcode, uniqueid, userfield, cnum, cnam, outbound_cnum, outbound_cnam, dst_cnam, recordingfile, linkedid, peeraccount, sequence FROM $db_name.$db_table_name $where $order $sort LIMIT $result_limit)";
 	$resultcsv = $dbcdr->getAll($query, DB_FETCHMODE_ASSOC);
 	cdr_export_csv($resultcsv);
 }
@@ -784,10 +791,10 @@ if ( $tot_calls_raw ) {
 			$tcid = $row['clid'];
 		}
 		//cdr_formatSrc($cid, $tcid);
-		if ($row['cnum'] != '' || $row['cnum'] != '') {
+		if ($row['cnam'] != '' || $row['cnum'] != '') {
 			cdr_formatCallerID($row['cnam'], $row['cnum'], $row['channel']);
 		} else {
-			cdr_formatSrc($row['src'], $row['clid']);
+			cdr_formatSrc(str_replace('"" ','',$row['clid']), str_replace('"" ','',$row['clid']));
 		}
 		cdr_formatCallerID($row['outbound_cnam'], $row['outbound_cnum'], $row['dstchannel']);
 		cdr_formatDID($row['did']);
@@ -1114,9 +1121,17 @@ function cdr_formatSrc($src, $clid) {
 }
 
 function cdr_formatCallerID($cnam, $cnum, $channel) {
-	$dcnum = $cnum == '' && $cnam == '' ? '' : htmlspecialchars('<' . $cnum . '>');
-	$dcnam = htmlspecialchars($cnam == '' ? '' : '"' . $cnam . '" ');
-	echo '<td title="' ._("Channel") . ": " . $channel . '">' . $dcnam . $dcnum . '</td>';
+	if(preg_match("/\p{Hebrew}/u", utf8_decode($cnam))){
+		$cnam = utf8_decode($cnam);
+		$dcnum = $cnum == '' && $cnam == '' ? '' : htmlspecialchars('<' . $cnum . '>');
+		$dcnam = htmlspecialchars($cnam == '' ? '' : '"' . $cnam . '" ');
+		echo '<td title="' ._("Channel") . ": " . $channel . '">' . $dcnum .' '. $dcnam . '</td>';
+	}
+	else{
+		$dcnum = $cnum == '' && $cnam == '' ? '' : htmlspecialchars('<' . $cnum . '>');
+		$dcnam = htmlspecialchars($cnam == '' ? '' : '"' . $cnam . '" ');
+		echo '<td title="' ._("Channel") . ": " . $channel . '">' . $dcnam . $dcnum . '</td>';
+	}
 }
 
 function cdr_formatDID($did) {
@@ -1203,6 +1218,9 @@ function cdr_formatRecordingFile($recordingfile, $basename, $id, $uid) {
 }
 
 function cdr_formatCNAM($cnam) {
+	if(preg_match("/\p{Hebrew}/u", utf8_decode($cnam))){
+		$cnam = utf8_decode($cnam);
+	}
 	$cnam = htmlspecialchars($cnam);
 	echo '<td title="' . _("Caller ID Name") . ": " . $cnam . '">' . $cnam . "</td>";
 }
