@@ -129,6 +129,15 @@ class Cdr extends Modules{
 	* @return bool True if pass
 	*/
 	function ajaxRequest($command, $settings) {
+		$enabled = $this->UCP->getCombinedSettingByID($this->user['id'], 'Cdr', 'enable');
+		if (!$enabled) {
+			return false;
+		}
+		$assigned = $this->UCP->getCombinedSettingByID($this->user['id'], 'Cdr', 'assigned');
+		if ($command =='grid' && !in_array($_REQUEST['extension'],$assigned)) {
+			return false;
+		}
+
 		switch($command) {
 			case 'grid':
 				return true;
@@ -157,13 +166,16 @@ class Cdr extends Modules{
 		$return = array("status" => false, "message" => "");
 		switch($_REQUEST['command']) {
 			case "grid":
-				$limit = $_REQUEST['limit'];
+				$limit = filter_var($_REQUEST['limit'], FILTER_SANITIZE_NUMBER_INT);
 				$ext = $_REQUEST['extension'];
+				if (!$this->_checkExtension($ext)) {
+					return array("status" => false, "message" => _("The extension isn't associated with the user account"));
+				}
 				$order = $_REQUEST['order'];
 				$orderby = !empty($_REQUEST['sort']) ? $_REQUEST['sort'] : "date";
 				$search = !empty($_REQUEST['search']) ? $_REQUEST['search'] : "";
 				$pages = $this->cdr->getPages($ext,$search,$limit);
-				$offset = $_REQUEST['offset'];
+				$offset = filter_var($_REQUEST['offset'], FILTER_SANITIZE_NUMBER_INT);
 				$page = ($offset / $limit) + 1;
 				$total = $this->cdr->getTotalCalls($ext,$search);
 				$data = $this->postProcessCalls($this->cdr->getCalls($ext,$page,$orderby,$order,$search,$limit),$ext);
@@ -173,6 +185,9 @@ class Cdr extends Modules{
 				);
 			break;
 			case 'gethtml5':
+				if (!$this->_checkExtension($_REQUEST['ext'])) {
+					return array("status" => false, "message" => _("The extension isn't associated with the user account"));
+				}
 				$media = $this->UCP->FreePBX->Media();
 				$record = $this->UCP->FreePBX->Cdr->getRecordByIDExtension($_REQUEST['id'],$_REQUEST['ext']);
 				if(!file_exists($record['recordingfile'])) {
