@@ -37,8 +37,8 @@ class PJSip extends \FreePBX\modules\Core\Drivers\Sip {
 		return array(
 			"rawName" => "pjsip",
 			"hardware" => "pjsip_generic",
-			"prettyName" => _("Generic PJSIP Device"),
-			"shortName" => "PJSIP",
+			"prettyName" => sprintf(_("Generic %s Device"),'SIP (PJSIP) [chan_pjsip]'),
+			"shortName" => "SIP [chan_pjsip]",
 			"description" => _("A new SIP channel driver for Asterisk, chan_pjsip is built on the PJSIP SIP stack. A collection of resource modules provides the bulk of the SIP functionality")
 		);
 	}
@@ -94,6 +94,10 @@ class PJSip extends \FreePBX\modules\Core\Drivers\Sip {
 				"value" => "yes",
 				"flag" => $flag++
 			),
+			"user_eq_phone" => array(
+				"value" => "no",
+				"flag" => $flag++
+			),
 			"namedcallgroup" => array(
 				"value" => $this->freepbx->Config->get('DEVICE_CALLGROUP'),
 				"flag" => $flag++
@@ -174,6 +178,18 @@ class PJSip extends \FreePBX\modules\Core\Drivers\Sip {
 				"value" => "",
 				"flag" => $flag++
 			),
+			"rtp_timeout" => array(
+				"value" => "0",
+				"flag" => $flag++
+			),
+			"rtp_timeout_hold" => array(
+				"value" => "0",
+				"flag" => $flag++
+			),
+			"timers_min_se" => array(
+				"value" => "90",
+				"flag" => $flag++
+			),
 			"direct_media" => array(
 				"value" => "yes",
 				"flag" => $flag++
@@ -189,6 +205,18 @@ class PJSip extends \FreePBX\modules\Core\Drivers\Sip {
 			"max_video_streams" => array(
 				"value" => "1",
 				"flag" => $flag++
+			),
+			"webrtc" => array(
+				"value" => "no",
+				"flag" => $flag++
+			),
+			"defaultuser" => array(
+					"value" => "",
+					"flag" => $flag++
+			),
+			"message_context" => array(
+					"value" => "",
+					"flag" => $flag++
 			)
 		);
 		return array(
@@ -249,13 +277,18 @@ class PJSip extends \FreePBX\modules\Core\Drivers\Sip {
 			$tt = _("With this option enabled, Asterisk will attempt to negotiate the use of bundle. If negotiated this will result in multiple RTP streams being carried over the same underlying transport. Note that enabling bundle will also enable the rtcp_mux option");
 			$tmparr['bundle'] = array('prompttext' => _('Enable RTP bundling'), 'value' => 'no', 'tt' => $tt, 'select' => $select, 'level' => 1, 'type' => 'radio');
 			unset($select);
-		}
 
-		if (version_compare($this->version, '15.0', 'ge')) {
 			$tt = _("Maximum number of allowed audio streams for the endpoint");
 			$tmparr['max_audio_streams'] = array('prompttext' => _('Max audio streams'), 'value' => '1', 'tt' => $tt, 'level' => 1);
+
 			$tt = _("Maximum number of allowed video streams for the endpoint");
 			$tmparr['max_video_streams'] = array('prompttext' => _('Max video streams'), 'value' => '1', 'tt' => $tt, 'level' => 1);
+
+			$select[] = array('value' => 'no', 'text' => _('No'));
+			$select[] = array('value' => 'yes', 'text' => _('Yes'));
+			$tt = _("Defaults and enables some options that are relevant to WebRTC");
+			$tmparr['bundle'] = array('prompttext' => _('Enable WebRTC defaults'), 'value' => 'no', 'tt' => $tt, 'select' => $select, 'level' => 1, 'type' => 'radio');
+			unset($select);
 		}
 
 		$select[] = array('value' => 'no', 'text' => _('None'));
@@ -274,6 +307,10 @@ class PJSip extends \FreePBX\modules\Core\Drivers\Sip {
 		$tmparr['timers'] = array('prompttext' => _('Session Timers'), 'value' => 'yes', 'tt' => $tt, 'select' => $select, 'level' => 1);
 		unset($select);
 
+		$tt = _("Minimum session timer expiration period. Time in seconds. This does not matter if Session Timers is set to No");
+		$tmparr['timers_min_se'] = array('prompttext' => _('Timer Expiration Period'), 'value' => '90', 'tt' => $tt, 'level' => 1);
+		unset($select);
+
 		$select[] = array('value' => 'no', 'text' => _('No'));
 		$select[] = array('value' => 'yes', 'text' => _('Yes'));
 		$tt = _("Determines whether media may flow directly between endpoints.");
@@ -284,6 +321,12 @@ class PJSip extends \FreePBX\modules\Core\Drivers\Sip {
 		$select[] = array('value' => 'yes', 'text' => _('Yes'));
 		$tt = _("Determines whether encryption should be used if possible but does not terminate the session if not achieved. This option only applies if Media Encryption is not set to None.").' [media_encryption_optimistic]';
 		$tmparr['media_encryption_optimistic'] = array('prompttext' => _('Allow Non-Encrypted Media (Opportunistic SRTP)'), 'value' => 'no', 'tt' => $tt, 'select' => $select, 'level' => 1, 'type' => 'radio');
+		unset($select);
+
+		$select[] = array('value' => 'no', 'text' => _('No'));
+		$select[] = array('value' => 'yes', 'text' => _('Yes'));
+		$tt = _('Some SIP phones (Mitel/Aastra, Snom) expect a sip/frag "200 OK" after REFER has been accepted. If set to no then asterisk will not send the progress details, but immediately will send "200 OK".').' [refer_blind_progress]';
+		$tmparr['refer_blind_progress'] = array('prompttext' => _('Refer Blind Progress'), 'value' => 'yes', 'tt' => $tt, 'select' => $select, 'level' => 1, 'type' => 'radio');
 
 		$tt = _("The number of in-use channels which will cause busy to be returned as device state. This should be left at 0 unless you know what you are doing");
 		$tmparr['device_state_busy_at'] = array('prompttext' => _('Device State Busy at'), 'value' => '0', 'tt' => $tt, 'level' => 1);
@@ -302,8 +345,19 @@ class PJSip extends \FreePBX\modules\Core\Drivers\Sip {
 		$tmparr['minimum_expiration'] = array('prompttext' => _('Minimum Expiration'), 'value' => '60', 'tt' => $tt, 'level' => 1);
 		unset($select);
 
+		$tt = _("Maximum number of seconds without receiving RTP (while off hold) before terminating call.");
+		$tmparr['rtp_timeout'] = array('prompttext' => _('RTP Timeout'), 'value' => '0', 'tt' => $tt, 'level' => 1);
+		unset($select);
+
+		$tt = _("Maximum number of seconds without receiving RTP (while on hold) before terminating call.");
+		$tmparr['rtp_timeout_hold'] = array('prompttext' => _('RTP Hold Timeout'), 'value' => '0', 'tt' => $tt, 'level' => 1);
+		unset($select);
+
 		$tmparr['outbound_proxy'] = array('prompttext' => _('Outbound Proxy'), 'value' => '', 'level' => 1);
 
+		$tt = _("Context where SIP MESSAGEs from this endpoint will be processed");
+		$tmparr['message_context'] = array('prompttext' => _('Messages Context'), 'value' => '', 'tt' => $tt, 'level' => 1);
+
 		//Use the transport engine, don't cross migrate anymore, it just doesn't work
 		$transports = $this->getActiveTransports();
 		$transports = is_array($transports)?$transports:array();
@@ -358,15 +412,43 @@ class PJSip extends \FreePBX\modules\Core\Drivers\Sip {
 			$conf['pjsip.transports.conf'][$transport] = $tmparr;
 		}
 
-		$conf['pjsip.conf']['global'] = array(
+		$ver_list = array("13.24.0", "16.1.0", "17.0.0", "18.0.0");
+        if(version_min($this->freepbx->Config->get('ASTVERSION'), $ver_list) == true){
+  			$use_callerid_contact = \FreePBX::create()->Sipsettings->getConfig('pjsip_use_callerid_contact');
+			$use_callerid_contact = (empty($use_callerid_contact))? "no": $use_callerid_contact;
+        	$conf['pjsip.conf']['global'] = array(
+              'type=global',
+              'user_agent='.$this->freepbx->Config->get('SIPUSERAGENT') . '-' . getversion() . "(" . $this->version . ")",
+              'use_callerid_contact='.$use_callerid_contact,
+          	);
+        }
+        else{
+        	$conf['pjsip.conf']['global'] = array(
 			'type=global',
-			'user_agent='.$this->freepbx->Config->get('SIPUSERAGENT') . '-' . getversion() . "(" . $this->version . ")"
-		);
+			'user_agent='.$this->freepbx->Config->get('SIPUSERAGENT') . '-' . getversion() . "(" . $this->version . ")",
+			);
+        }
+
+		$debug = $this->freepbx->Sipsettings->getConfig('pjsip_debug');
+		if($debug) {
+			$conf['pjsip.conf']['global']['debug'] = (empty($debug) || $debug === 'yes') ? 'yes' : 'no';
+		}
+		$pjsip_keepAliveinterval = $this->freepbx->Sipsettings->getConfig('pjsip_keep_alive_interval');
+		if(isset($pjsip_keepAliveinterval)) {
+			$conf['pjsip.conf']['global']['keep_alive_interval'] = !empty($pjsip_keepAliveinterval) ? $pjsip_keepAliveinterval : '90';
+		}
 		if(!empty($this->_global) && is_array($this->_global)) {
 			foreach($this->_global as $el) {
 				$conf['pjsip.conf']['global'][] = "{$el['key']}={$el['value']}";
 			}
 		}
+		$asteriskVersions=array("13.25.0", "16.2.0", "17.0.0", "18.0.0");
+		if(version_min($this->freepbx->Config->get('ASTVERSION'), $asteriskVersions) == true){
+			$taskprocessor_overload_trigger = $this->freepbx->Sipsettings->getConfig('taskprocessor_overload_trigger');
+			$conf['pjsip.conf']['global']['taskprocessor_overload_trigger'] =  (empty($taskprocessor_overload_trigger)) ? 'pjsip_only' : $taskprocessor_overload_trigger;
+		}else{
+			unset($conf['pjsip.conf']['global']['taskprocessor_overload_trigger']);
+		}
 		$conf['pjsip.conf']['global'][] = "#include pjsip_custom_post.conf";
 		$trunks = $this->getAllTrunks();
 		foreach($trunks as $trunk) {
@@ -408,10 +490,16 @@ class PJSip extends \FreePBX\modules\Core\Drivers\Sip {
 					'forbidden_retry_interval' => (empty($trunk['forbidden_retry_interval']))? "0" : $trunk['forbidden_retry_interval'],
 					'max_retries' => $retries,
 					'expiration' => $trunk['expiration'],
-					'line' => 'yes',
-					'endpoint' => str_replace(' ', '', $tn),
 					'auth_rejection_permanent' => ($trunk['auth_rejection_permanent'] == 'on') ? 'yes' : 'no'
 				);
+
+				if ($trunk['pjsip_line'] == 'true') {
+					$conf['pjsip.registration.conf'][$tn]['line'] = 'yes';
+					$conf['pjsip.registration.conf'][$tn]['endpoint'] = str_replace(' ', '', $tn);
+				} else {
+					$conf['pjsip.registration.conf'][$tn]['line'] = 'no';
+				}
+
 				if(!empty($trunk['contact_user'])) {
 					$conf['pjsip.registration.conf'][$tn]['contact_user'] = $trunk['contact_user'];
 				}
@@ -461,7 +549,7 @@ class PJSip extends \FreePBX\modules\Core\Drivers\Sip {
 				if ($trunk['authentication'] == "inbound" || $trunk['authentication'] == "both" || empty($trunk['username'])) {
 					$conf['pjsip.auth.conf'][$tn]['username'] = $tn;
 				} else {
-					$conf['pjsip.auth.conf'][$tn]['username'] = $trunk['username'];
+					$conf['pjsip.auth.conf'][$tn]['username'] = isset($trunk['auth_username']) && !empty($trunk['auth_username']) ? $trunk['auth_username'] : $trunk['username'];
 				}
 			}
 
@@ -511,10 +599,10 @@ class PJSip extends \FreePBX\modules\Core\Drivers\Sip {
 				'disallow' => 'all',
 				'allow' => $this->filterValidCodecs(!empty($trunk['codecs']) ? $trunk['codecs'] : 'ulaw'), // '&' is invalid in pjsip
 				'aors' => !empty($trunk['aors']) ? $trunk['aors'] : $tn,
-				'send_connected_line' => !empty($trunk['send_connected_line']) ? $trunk['send_connected_line'] : 'yes',
+				'send_connected_line' => !empty($trunk['send_connected_line']) ? $trunk['send_connected_line'] : 'yes'
 			);
-
-			$ver_list = array("13.24.0", "16.1.0"); // include all versions to test.
+			
+			$ver_list = array("13.24.0", "16.1.0", "18.0.0"); // include all versions to test.
 			if(version_min($this->freepbx->Config->get('ASTVERSION'), $ver_list) == false){
 				unset($conf['pjsip.endpoint.conf'][$tn]['send_connected_line']);
 			}
@@ -550,6 +638,10 @@ class PJSip extends \FreePBX\modules\Core\Drivers\Sip {
 				$conf['pjsip.endpoint.conf'][$tn]['contact_user'] = $trunk['contact_user'];
 			}
 
+			if(!empty($trunk['user_eq_phone'])) {
+				$conf['pjsip.endpoint.conf'][$tn]['user_eq_phone'] = $trunk['user_eq_phone'];
+			}
+
 			if(!empty($trunk['dtmfmode'])) {
 				// PJSIP Has a limited number of dtmf settings. If we don't know what it is, set it to RFC.
 				$validdtmf = array("rfc4733","inband","info");
@@ -730,24 +822,6 @@ class PJSip extends \FreePBX\modules\Core\Drivers\Sip {
 	public function writeConfig($conf) {
 		$ast_sip_driver = $this->freepbx->Config->get_conf_setting('ASTSIPDRIVER');
 
-		// We check for 'is_numeric' in case the Asterisk Version can't be parsed, for whatever
-		// reason - it will return the string 'UNABLE-TO-PARSE'. If that happens, don't change
-		// anything, because it probably was working, and changing things will break them.
-		//
-		// However, if they HAVE switched from somethign that supports pjsip to something
-		// that doesn't support pjsip, we need to disable pjsip, otherwise Asterisk won't
-		// start.
-		if(is_numeric($this->version) && version_compare($this->version, '12', 'lt')) {
-			// Asterisk 11 doesn't support pjsip
-			// If we're trying to use pjsip on 11, disable it.  If there are devices or
-			// trunks trying to use chan_pjsip, we complain loudly about it in
-			// core_devices_configpageload
-			if($ast_sip_driver == 'chan_pjsip' || $ast_sip_driver == 'both') {
-				$this->freepbx->Config->set_conf_values(array('ASTSIPDRIVER' => 'chan_sip'), true, true);
-				$ast_sip_driver = "chan_sip";
-			}
-		}
-
 		$this->freepbx->ModulesConf->removenoload("res_pjproject.so");
 		if($ast_sip_driver === 'both') {
 			$this->freepbx->ModulesConf->removenoload("chan_sip.so");
@@ -896,6 +970,9 @@ class PJSip extends \FreePBX\modules\Core\Drivers\Sip {
 				$localnets = is_array($localnets)?$localnets:array();
 				if ($localnets) {
 					foreach($localnets as $arr) {
+						if(empty($arr['net']) || empty($arr['mask'])) {
+							continue;
+						}
 						$transport[$t]['local_net'][] = $arr['net']."/".$arr['mask'];
 					}
 				}
@@ -1073,6 +1150,11 @@ class PJSip extends \FreePBX\modules\Core\Drivers\Sip {
 		if (!empty($config['direct_media'])) {
 			$endpoint[] = "direct_media=".$config['direct_media'];
 		}
+
+		if (!empty($config['vmexten'])) {
+			$endpoint[] = "voicemail_extension=".$config['vmexten'];
+		}
+
 		//http://issues.freepbx.org/browse/FREEPBX-12151
 		if(isset($config['mailbox'])) {
 			$mwisub = !empty($config['mwi_subscription']) ? $config['mwi_subscription'] : (version_compare($this->version,'13.9.1','ge') ? "auto" : "unsolicited");
@@ -1130,6 +1212,9 @@ class PJSip extends \FreePBX\modules\Core\Drivers\Sip {
 			if (!empty($config['max_video_streams'])) {
 				$endpoint[] = "max_video_streams=".$config['max_video_streams'];
 			}
+			if (!empty($config['webrtc'])) {
+				$endpoint[] = "webrtc=".$config['webrtc'];
+			}
 		}
 
 		if (!empty($config['bundle']) && version_compare($this->version,'15.0','ge')) {
@@ -1147,8 +1232,16 @@ class PJSip extends \FreePBX\modules\Core\Drivers\Sip {
 		if (!empty($config['trustrpid'])) {
 			$endpoint[] = "trust_id_inbound=".$config['trustrpid'];
 		}
+
+		// Any Asterisk version
+		if (!empty($config['user_eq_phone'])) {
+			$endpoint[] = "user_eq_phone=".$config['user_eq_phone']; // any Asterisk version
+		}
+		else{
+			$endpoint[] = "user_eq_phone=no";
+		}
 		
-		$ver_list = array("13.24.0", "16.1.0", "17.0.0");
+		$ver_list = array("13.24.0", "16.1.0", "17.0.0", "18.0.0");
 		if(version_min($this->freepbx->Config->get('ASTVERSION'), $ver_list) == true){
 			if (!empty($config['send_connected_line'])) {
 				$endpoint[] = "send_connected_line=".$config['send_connected_line'];
@@ -1169,6 +1262,10 @@ class PJSip extends \FreePBX\modules\Core\Drivers\Sip {
 			$endpoint[] = "timers=".$config['timers'];
 		}
 
+		if (!empty($config['timers_min_se'])) {
+			$endpoint[] = "timers_min_se=".$config['timers_min_se'];
+		}
+
 		if (!empty($config['outbound_proxy'])) {
 			$endpoint[] = "outbound_proxy=".$config['outbound_proxy'];
 		}
@@ -1177,6 +1274,17 @@ class PJSip extends \FreePBX\modules\Core\Drivers\Sip {
 			$endpoint[] = "media_encryption_optimistic=".$config['media_encryption_optimistic'];
 		}
 
+		if (!empty($config['refer_blind_progress'])) {
+			$endpoint[] = "refer_blind_progress=".$config['refer_blind_progress'];
+		}
+
+		if (!empty($config['refer_blind_progress'])) {
+			$endpoint[] = "refer_blind_progress=".$config['refer_blind_progress'];
+		}
+
+		$endpoint[] = "rtp_timeout=".(!empty($config['rtp_timeout']) ? $config['rtp_timeout'] : $chan_sip_settings['rtptimeout']);
+		$endpoint[] = "rtp_timeout_hold=".(!empty($config['rtp_timeout_hold']) ? $config['rtp_timeout_hold'] : $chan_sip_settings['rtpholdtimeout']);
+
 		if(!empty($config['device_state_busy_at']) && is_numeric($config['device_state_busy_at']) && $config['device_state_busy_at'] > 0) {
 			$endpoint[] = "device_state_busy_at=".$config['device_state_busy_at'];
 		}
@@ -1221,6 +1329,10 @@ class PJSip extends \FreePBX\modules\Core\Drivers\Sip {
 			$endpoint[] = 'record_off_feature=apprecord';
 		}
 
+		if (!empty($config['message_context'])) {
+                        $endpoint[] = "message_context=".$config['message_context'];
+                }
+
 		// Auth
 		$auth[] = "auth_type=userpass";
 		$auth[] = "password=".$config['secret'];
@@ -1442,7 +1554,10 @@ class PJSip extends \FreePBX\modules\Core\Drivers\Sip {
 	 * @param {array} $dispvars Display Variables
 	 */
 	public function getDisplayVars($trunkid, $dispvars) {
-		$sipSettingsCodecs = $this->freepbx->Sipsettings->getCodecs('audio',true);
+		$audio_codecs 		= $this->freepbx->Sipsettings->getCodecs('audio',true);
+		$video_codecs 		= $this->freepbx->Sipsettings->getCodecs('video',true);	
+		$sipSettingsCodecs 	= array_merge($audio_codecs, $video_codecs);
+
 		if(!empty($trunkid)) {
 			$get = $this->db->prepare("SELECT keyword, data FROM pjsip WHERE id = :id");
 			$get->bindParam(':id', str_replace('OUT_','',$trunkid));
@@ -1466,12 +1581,12 @@ class PJSip extends \FreePBX\modules\Core\Drivers\Sip {
 
 		} else {
 			$dispvars = array(
-				"auth_rejection_permanent" => "on",
+				"auth_rejection_permanent" => "off",
 				"expiration" => 3600,
 				"retry_interval" => 60,
-				"forbidden_retry_interval" => 10,
-				"fatal_retry_interval" => 0,
-				"max_retries" => 10,
+				"forbidden_retry_interval" => 30,
+				"fatal_retry_interval" => 30,
+				"max_retries" => 10000,
 				"context" => "from-pstn",
 				"transport" => null,
 				"codecs" => $sipSettingsCodecs,
