@@ -18,8 +18,8 @@ class Sip extends techDriver {
 		return array(
 			"rawName" => "sip",
 			"hardware" => "sip_generic",
-			"prettyName" => _("Generic CHAN SIP Driver"),
-			"shortName" => "Chan_SIP",
+			"prettyName" => sprintf(_("Generic %s Device"),'SIP (Legacy) [chan_pjsip]'),
+			"shortName" => "SIP (Legacy) [chan_sip]",
 			"description" => _("The legacy SIP channel driver in Asterisk")
 		);
 	}
@@ -139,6 +139,10 @@ class Sip extends techDriver {
 				"value" => "no",
 				"flag" => $flag++
 			),
+			"webrtc" => array(
+				"value" => "",
+				"flag" => $flag++
+			),
 		);
 		return array(
 			"dial" => $dial,
@@ -194,30 +198,12 @@ class Sip extends techDriver {
 						if ($protocol == "ws" || $protocol == "wss") {
 							continue;
 						}
-						// Is this the default port for this protocol?
-						$defaultport = false;
-						if ($protocol == "udp" && $port == 5060) {
-							$defaultport = true;
-						} elseif ($protocol == "tcp" && $port == 5060) {
-							$defaultport = true;
-						} elseif ($protocol == "tls" && $port == 5061) {
-							$defaultport = true;
-						}
-
 						// If the bind address is 0.0.0.0 (or :: or [::]), we don't need to say
 						// that it's listening on a specific address.
-						if ($ip == "0.0.0.0" || $ip == "::" || $ip = "[::]") {
-							if ($defaultport) {
-								$pports[] = sprintf(_("Port %s (%s)"), $port, strtoupper($protocol));
-							} else {
-								$pports[] = sprintf(_("Port %s (%s - this is a <strong>NON STANDARD</strong> port)"), $port, strtoupper($protocol));
-							}
+						if ($ip == "0.0.0.0" || $ip == "::" || $ip == "[::]") {
+							$pports[] = sprintf(_("Port %s (%s)"), $port, strtoupper($protocol));
 						} else {
-							if ($defaultport) {
-								$pports[] = sprintf(_("Interface %s, Port %s (%s)"), $ip, $port, strtoupper($protocol));
-							} else {
-								$pports[] = sprintf(_("Interface %s, Port %s (%s - this is a <strong>NON STANDARD</strong> port)"), $ip, $port, strtoupper($protocol));
-							}
+							$pports[] = sprintf(_("Interface %s with Port %s (%s)"), $ip, $port, strtoupper($protocol));
 						}
 					}
 				}
@@ -342,7 +328,7 @@ class Sip extends techDriver {
 		$tt = _("Whether Asterisk should trust the RPID settings from this device. Usually should be yes for CONNECTEDLINE() functionality to work if supported by the endpoint.").'[trustrpid]';
 		$tmparr['trustrpid'] = array('prompttext' => _('Trust RPID'), 'value' => $this->freepbx->Config->get_conf_setting('DEVICE_SIP_TRUSTRPID'), 'tt' => $tt, 'select' => $select, 'level' => 1, 'type' => 'radio');
 
-		$ver_list = array("13.24.0", "16.1.0", "17.0.0");
+		$ver_list = array("13.24.0", "16.1.0", "17.0.0", "18.0.0");
 		if ($deviceInfo['tech'] == "pjsip" && version_min($this->freepbx->Config->get('ASTVERSION'), $ver_list) == true) {
 			unset($select);
 			$select[] 	= array('value' => 'no', 'text' => _('No'));
@@ -352,6 +338,13 @@ class Sip extends techDriver {
 			$tmparr['send_connected_line'] = array('prompttext' => _('Send Connected Line'), 'value' => empty($scl_val) ? 'yes': $scl_val, 'tt' => $tt, 'select' => $select, 'level' => 1, 'type' => 'radio');
 		}
 
+		unset($select);
+		$select[] 	= array('value' => 'no', 'text' => _('No'));
+		$select[] 	= array('value' => 'yes', 'text' => _('Yes'));
+		$tt 		= _("Determines whether a user=phone parameter is placed into the request URI if the user is determined to be a phone number.").'[user_eq_phone]';
+		$uep_val 	= $this->freepbx->Config->get_conf_setting('user_eq_phone');
+		$tmparr['user_eq_phone'] = array('prompttext' => _('user = Phone'), 'value' => empty($uep_val) ? 'no': $uep_val, 'tt' => $tt, 'select' => $select, 'level' => 1, 'type' => 'radio');
+
 		unset($select);
 		$select[] = array('value' => 'no', 'text' => _('No'));
 		$select[] = array('value' => 'yes', 'text' => _('Send Remote-Party-ID header'));
