@@ -119,7 +119,7 @@ $globals_convert['TONEZONE'] = 'us';
 
 // LAUNCH_AGI_AS_FASTAGI
 //
-$set['value'] = false;
+$set['value'] = true;
 $set['defaultval'] =& $set['value'];
 $set['options'] = '';
 $set['readonly'] = 0;
@@ -133,6 +133,20 @@ $set['description'] = "When enabled all AGI() calls that launch local scripts wi
 $set['type'] = CONF_TYPE_BOOL;
 $freepbx_conf->define_conf_setting('LAUNCH_AGI_AS_FASTAGI',$set);
 
+// DIALPARTIESDIALPLAN
+//
+$set['value'] = false;
+$set['defaultval'] =& $set['value'];
+$set['options'] = '';
+$set['readonly'] = 0;
+$set['hidden'] = 0;
+$set['level'] = 0;
+$set['module'] = '';
+$set['name'] = 'Use Experimental Dialparties Dialplan';
+$set['description'] = "Use experimental dialparties dialplan. When enabled the system will no longer use dialparties.agi and will instead use strictly dialplan, this may improve performance on heavy systems";
+$set['type'] = CONF_TYPE_BOOL;
+$freepbx_conf->define_conf_setting('DIALPARTIESDIALPLAN',$set);
+
 // OUTBOUND_CID_UPDATE
 //
 $set['value'] = true;
@@ -164,8 +178,8 @@ $set['name'] = 'Display Dialed Number on Calling Phone';
 $set['description'] = "When set to true and when CONNECTEDLINE() capabilities are configured and supported by your handset, the number actually dialled will be updated on your handset in the CNUM field. This allows you to see the final manipulation of your number after outbound route and trunk dial manipulation rules have been applied. For example, if you have configured 7 digit dialing on a North America dialplan, the ultimate 10 or 11 digit transmission will be displayed back. Any 'Outbound Dial Prefixes' configured at the trunk level will NOT be shown as these are foten analog line pauses (w) or other characters that distort the CNUM field on updates.";
 $set['type'] = CONF_TYPE_BOOL;
 $freepbx_conf->define_conf_setting('OUTBOUND_DIAL_UPDATE',$set);
-//Added Emergencydevice table
-outn(_("Adding EmergencyDevicetable.."));
+//Added emergencydevices table
+outn(_("Adding emergencydevices table.."));
 $sql = "CREATE TABLE IF NOT EXISTS `emergencydevices` (
   `id` VARCHAR(20)  NOT NULL,
   `tech` VARCHAR(10) NOT NULL ,
@@ -178,7 +192,7 @@ $sql = "CREATE TABLE IF NOT EXISTS `emergencydevices` (
 )";
 $check = $db->query($sql);
 if(DB::IsError($check)) {
-	out(_("Failed to create emergencydevices Table"));
+	out(_("Failed to create emergencydevices table"));
 }
 out(_("Done"));
 
@@ -567,6 +581,7 @@ function _initialize_zonelist() {
  array ( "name" => "Hong Kong", "iso" => "hk", "conf" => "ringcadence = 400,200,400,3000\ndial = 350+440\nbusy = 480+620/500,0/500\nring = 440+480/400,0/200,440+480/400,0/3000\ncongestion = 480+620/250,0/250\ncallwaiting = 440/300,0/10000\ndialrecall = !350+440/100,!0/100,!350+440/100,!0/100,!350+440/100,!0/100,350+440\nrecord = 1400/500,0/15000\ninfo = !950/330,!1400/330,!1800/330,0\nstutter = !350+440/100,!0/100,!350+440/100,!0/100,!350+440/100,!0/100,!350+440/100,!0/100,!350+440/100,!0/100,!350+440/100,!0/100,350+440\n"),
  array ( "name" => "Hungary",  "iso" => "hu", "conf" => "ringcadence = 1250,3750\ndial = 425\nbusy = 425/300,0/300\nring = 425/1250,0/3750\ncongestion = 425/300,0/300\ncallwaiting = 425/40,0/1960\ndialrecall = 425+450\nrecord = 1400/400,0/15000\ninfo = !950/330,!1400/330,!1800/330,!0/1000,!950/330,!1400/330,!1800/330,!0/1000,!950/330,!1400/330,!1800/330,!0/1000,0\nstutter = 350+375+400\n"),
  array ( "name" => "India",  "iso" => "in", "conf" => "ringcadence = 400,200,400,2000\ndial = 400*25\nbusy = 400/750,0/750\nring = 400*25/400,0/200,400*25/400,0/2000\ncongestion = 400/250,0/250\ncallwaiting = 400/200,0/100,400/200,0/7500\ndialrecall = !350+440/100,!0/100,!350+440/100,!0/100,!350+440/100,!0/100,350+440\nrecord = 1400/500,0/15000\ninfo = !950/330,!1400/330,!1800/330,0/1000\nstutter = !350+440/100,!0/100,!350+440/100,!0/100,!350+440/100,!0/100,!350+440/100,!0/100,!350+440/100,!0/100,!350+440/100,!0/100,350+440\n"),
+ array ( "name" => "Indonesia",  "iso" => "id", "conf" => "ringcadence = 1000,4000\ndial = 425\nbusy = 425/500,0/500\nring = 425/1000,0/4000\ncongestion = 425/250,0/250\ncallwaiting = 425/150,0/150,425/150,0/10000\ninfo = !950/330,!1400/330,!1800/330,!0/1000\n"),
  array ( "name" => "Iran",  "iso" => "ir", "conf" => "ringcadence = 1000,4000\ndial = 425\nbusy = 425/500,0/500\nring = 425/1000,0/4000\ncongestion = 425/240,0/240\ncallwaiting = 425/200,0/200,425/200,0/10000\ndialrecall = 425 record = 1400/80,0/15000\ninfo = 950/330,1400/330,1800/330,0/1000\nstutter = 400+425\n"),
  array ( "name" => "Ireland", "iso" => "ie", "conf" => "ringcadance = 400,200,400,2000\ndial = 425\nbusy = 425/450,0/450\nring = 400+450/400,0/200,400+450/400,0/2000\ncongestion = 425/375,0/375\ncallwaiting = 425/200,0/200,425/200,0/4000\ndialrecall = 440\nrecord = 1400/500,0/10000\ninfo = 950/330,1400/330,1800/330\n"),
  array ( "name" => "Israel",  "iso" => "il", "conf" => "ringcadence = 1000,3000\ndial = 414\nbusy = 414/500,0/500\nring = 414/1000,0/3000\ncongestion = 414/250,0/250\ncallwaiting = 414/100,0/100,414/100,0/100,414/600,0/3000 \ndialrecall = !414/100,!0/100,!414/100,!0/100,!414/100,!0/100,414\nrecord = 1400/500,0/15000\ninfo = 1000/330,1400/330,1800/330,0/1000\nstutter = !414/160,!0/160,!414/160,!0/160,!414/160,!0/160,!414/160,!0/160,!414/160,!0/160,!414/160,!0/160,!414/160,!0/160,!414/160,!0/160,!414/160,!0/160,!414/160,!0/160,414 \n"),
@@ -925,12 +940,12 @@ $set['category'] = 'Dialplan and Operational';
 $set['value'] = $INTERNALALERTINFO;
 $set['defaultval'] = $set['value'];
 $set['name'] = 'Internal Alert Info';
-$set['description'] = "Alert Info to use on Extension to Extension Calls. 'Inherit' will use the previously set Alert Info,'Unset' will remove the previously set Alert Info.";
+$set['description'] = "Alert Info to use on Extension to Extension Calls. 'Inherit' will use the previously set Alert Info,'Unset' will remove the previously set Alert Info. <br> <b>Sangoma's S series phones ringtones</b>: ring1, ring2, ring3, ring4, ring5. </br> <b>Sangoma's D series phones ringtones</b>: Alarm, Chimes, Digium, GuitarStrum, Jingle, Office, Office2, RotaryPhone, SteelDrum, Techno, Theme, Tweedle, Twinkle, Vibe.";
 $set['hidden'] = 0;
 $set['emptyok'] = 1;
 $set['readonly'] = 0;
 $set['level'] = 0;
-$set['options'] = array("inherit","unset","ring1","ring2","ring3","ring4","ring5");
+$set['options'] = array("inherit","unset","ring1","ring2","ring3","ring4","ring5","Alarm","Chimes","Digium","GuitarStrum","Jingle","Office","Office2","RotaryPhone","SteelDrum","Techno","Theme","Tweedle","Twinkle","Vibe");
 $set['module'] = '';
 $set['type'] = CONF_TYPE_CSELECT;
 $freepbx_conf->define_conf_setting('INTERNALALERTINFO',$set);
@@ -1009,6 +1024,9 @@ if(!empty($info['core']['dbversion']) && version_compare_freepbx($info['core']['
 		out(_("error occured"));
 	}
 }
+if(!empty($info['core']['dbversion']) && version_compare_freepbx($info['core']['dbversion'], "15.0.9.23" , "<=")) {
+	$freepbx_conf->update('DIALPARTIESDIALPLAN', false);
+}
 
 if(FreePBX::Modules()->checkStatus("sysadmin")) {
 	touch("/var/spool/asterisk/incron/core.logrotate");
@@ -1050,6 +1068,25 @@ if(!\FreePBX::Core()->getConfig('migratesendrpid')) {
 	\FreePBX::Core()->setConfig('migratesendrpid',true);
 }
 
+$sth = \FreePBX::Database()->prepare("SELECT DISTINCT route_id FROM outbound_route_sequence");
+$sth->execute();
+$routeSeqs = $sth->fetchAll(\PDO::FETCH_COLUMN);
+
+$sth = \FreePBX::Database()->prepare("SELECT route_id FROM outbound_routes");
+$sth->execute();
+$outboundRoutes = $sth->fetchAll(\PDO::FETCH_COLUMN);
+
+
+$routeDiff = array_diff($outboundRoutes,$routeSeqs);
+
+if(!empty($routeDiff)) {
+	\FreePBX::Notifications()->add_warning('core', 'MISSING_ROUTES', _('Missing outbound routes have been found'), _('Due to bug FREEPBX-18620 routes were created without valid references which made them invisible. They are now visible and you are advised to edit/delete routes you dont know about'), "?display=routing", true, true);
+	foreach($routeDiff as $route) {
+		$routing = new FreePBX\modules\Core\Components\Outboundrouting();
+		$routing->setOrder($route, 'new');
+	}
+}
+
 
 outn(_("Removing encoding on incoming routes alertinfo values..."));
 $sth = \FreePBX::Database()->prepare("SELECT * FROM incoming WHERE `alertinfo` != ''");
