@@ -24,30 +24,13 @@ if (function_exists('music_list')){
 if (function_exists('timeconditions_timegroups_drawgroupselect')){
 	$optionalelems .= load_view(__DIR__.'/timecond.php', array('time_group_id' => $time_group_id, 'time_mode' => $time_mode,'timezone' => $timezone,'calendar_id' => $calendar_id,'calendar_group_id' => $calendar_group_id));
 }
-$routepriority = core_routing_list();
-$routeseqopts = '';
-//Routing select box
-if ($route_seq != 0) {
-	$routeseqopts .= '<option value="0"'.($route_seq == 0 ? ' SELECTED' : '').'>'.sprintf(_('First before %s'),$routepriority[0]['name'])."</option>\n";
-}
 
-$last_seq = isset($last_seq)?$last_seq:count($routepriority)-1;
-$route_names = array();
-foreach ($routepriority as $key => $route) {
-	if($request['id'] != $route['route_id']){
-		$route_names[] = $route['name'];
-	}
-	if ($key == 0 && $route_seq != 0) continue;
-	if ($key == ($route_seq+1)) continue;
-	if ($route_seq == $key) {
-		$routeseqopts .= '				<option value="'.$key.'" SELECTED>'._('---No Change---')."</option>\n";
-	} else {
-		$routeseqopts .= '				<option value="'.$key.'">'.sprintf(_('Before %s'),$route['name'])."</option>\n";
-	}
-}
-if ($extdisplay == '' | $route_seq != $last_seq) {
-	$routeseqopts .= '<option value="bottom"'.($route_seq == count($routepriority) ? ' SELECTED' : '').'>'.sprintf(_('Last after %s'),$routepriority[$last_seq]['name'])."</option>\n";
-}
+//Notification tab email settings
+$emailfrom = $emailInfo['emailfrom'];
+$emailto = $emailInfo['emailto'];
+$emailsubject = $emailInfo['emailsubject'];
+$emailbody = $emailInfo['emailbody'];
+
 //Hooks....
 //$module_hook = moduleHook::create();
 //if (!empty($module_hook->hookHtml)) {
@@ -132,7 +115,7 @@ if(!$amp_conf['ENABLEOLDDIALPATTERNS']) {
 		$dpinput[] = '	</div>';
 		$dpinput[] = '<td>';
 		$dpinput[] = '		<a href="#"  id="routerowadd'.$idx.'"><i class="fa fa-plus"></i></a>';
-		$dpinput[] = '		<a href="#"  id="routerowdel'.$idx.'"><i class="fa fa-trash"></i></a>';
+		$dpinput[] = '		<a href="#"  id="routerowdel'.$idx.'"><i class="fa fa-trash-o"></i></a>';
 		$dpinput[] = '</td>';
 		$dpinput[] = '</tr>';
 	}
@@ -174,7 +157,7 @@ if(!$amp_conf['ENABLEOLDDIALPATTERNS']) {
 	$dpinput[] = '	</div>';
 	$dpinput[] = '<td>';
 	$dpinput[] = '		<a href="#"  id="routerowadd'.$idx.'"><i class="fa fa-plus"></i></a>';
-	$dpinput[] = '		<a href="#"  id="routerowdel'.$idx.'"><i class="fa fa-trash"></i></a>';
+	$dpinput[] = '		<a href="#"  id="routerowdel'.$idx.'"><i class="fa fa-trash-o"></i></a>';
 	$dpinput[] = '</td>';
 	$dpinput[] = '</tr>';
 	$dprows = implode(PHP_EOL, $dpinput);
@@ -193,12 +176,15 @@ if(!$amp_conf['ENABLEOLDDIALPATTERNS']) {
 }
 //trunk html
 $trunks = array();
-foreach (core_trunks_listbyid() as $temp) {
+foreach (core_trunks_listbyid(true) as $temp) {
 	$trunks[$temp['trunkid']] = $temp['name'];
 	$trunkstate[$temp['trunkid']] = $temp['disabled'];
+	$trunkdisplay[$temp['trunkid']] = isset($temp['routedisplay']) ? $temp['routedisplay'] : 'on';
 }
 $key = -1;
-$positions=count($trunkpriority);
+if (is_array($trunkpriority)) {
+	$positions=count($trunkpriority);
+}
 $trunkhtml = '<table class = "table">';
 $trunkhtml .= '<tbody id="routetrunks">';
 if(!empty($trunkpriority) && is_array($trunkpriority)) {
@@ -207,18 +193,18 @@ if(!empty($trunkpriority) && is_array($trunkpriority)) {
 				$trunkhtml .= '<td>';
 				$trunkhtml .= '<div class="input-group">';
 				$trunkhtml .= '<span class="input-group-addon move" id="basic-addon'.$key.'"><i class="fa fa-arrows"></i></span>';
-			 	$trunkhtml .= '<select id="trunkpri'.$key.'" name="trunkpriority['.$key.']" class="form-control '. ($trunkstate[$trunk]=='off'?"":'text-danger').'">';
+			 	$trunkhtml .= '<select id="trunkpri'.$key.'" name="trunkpriority['.$key.']" class="form-control '. ($trunkstate[$trunk]=='off' &&  $trunkdisplay[$trunk] == 'on'?"":'text-danger').'">';
 				$trunkhtml .= '<option value=""></option>';
 				foreach ($trunks as $name=>$display_description) {
-					if ($trunkstate[$name] == 'off') {
+					if ($trunkstate[$name] == 'off' && $trunkdisplay[$trunk] == 'on') {
 						$trunkhtml .= '<option id="trunk'.$key.'" name="trunk'.$key.'" value="'.$name.'" '.($name == $trunk ? "selected" : "").'>'.str_replace('AMP:', '', $display_description).'</option>';
-					} else {
+					} else if ($trunkdisplay[$trunk] == 'on') {
 						$trunkhtml .= '<option id="trunk'.$key.'" class="text-danger" name="trunk'.$key.'" value="'.$name.'" '.($name == $trunk ? "selected" : "").'>'.str_replace('AMP:', '', $display_description).'</option>';
 					}
 				}
 
 				$trunkhtml .= '</select>';
-				$trunkhtml .= '<span class="input-group-btn"><button type="button" class="btn btn-default deltrunkrow"><i class = "fa fa-trash"></i></button></span>';
+				$trunkhtml .= '<span class="input-group-btn"><button type="button" class="btn btn-default deltrunkrow"><i class = "fa fa-trash-o"></i></button></span>';
 				$trunkhtml .= '</div>';
 				$trunkhtml .= '</td>';
 				$trunkhtml .= '</tr>';
@@ -243,7 +229,7 @@ for ($i=0; $i < $num_new_boxes; $i++) {
 		}
 	}
 	$trunkhtml .= '</select>';
-	$trunkhtml .= '<span class="input-group-btn"><button type="button" class="btn btn-default deltrunkrow"><i class = "fa fa-trash"></i></button></span>';
+	$trunkhtml .= '<span class="input-group-btn"><button type="button" class="btn btn-default deltrunkrow"><i class = "fa fa-trash-o"></i></button></span>';
 	$trunkhtml .= '</div>';
 	$trunkhtml .= '</td>';
 	$trunkhtml .= '</tr>';
@@ -254,6 +240,7 @@ for ($i=0; $i < $num_new_boxes; $i++) {
 ?>
 <form enctype="multipart/form-data" class="fpbx-submit" autocomplete="off" id="routeEdit" name="routeEdit" action="?display=routing" method="POST" data-fpbx-delete="config.php?display=<?php echo urlencode($display) ?>&id=<?php echo urlencode($extdisplay) ?>&action=delroute">
 	<input type="hidden" id="extdisplay" name="extdisplay" value="<?php echo $extdisplay ?>"/>
+	<input type="hidden" id="route_seq" name="route_seq" value="<?php echo $route_seq?>">
 	<input type="hidden" id="id" name="id" value="<?php echo $extdisplay ?>"/>
 	<input type="hidden" id="action" name="action" value="<?php echo $formAction ?>"/>
 	<input type="hidden" id="repotrunkdirection" name="repotrunkdirection" value="">
@@ -264,6 +251,7 @@ for ($i=0; $i < $num_new_boxes; $i++) {
   <li role="presentation" data-name="routesettings" class="change-tab active"><a href="#routesettings" data-toggle="tab"><?php echo _("Route Settings")?></a></li>
   <li role="presentation" data-name="dialpatterns" class="change-tab"><a href="#dialpatterns" data-toggle="tab"><?php echo _("Dial Patterns")?></a></li>
   <li role="presentation" data-name="importexport" class="change-tab"><a href="#importexport" data-toggle="tab"><?php echo _("Import/Export Patterns")?></a></li>
+  <li role="presentation" data-name="notifications" class="change-tab"><a href="#notifications" data-toggle="tab"><?php echo _("Notifications")?></a></li>
     <?php echo $hooks['hookTabs'] ?>
   <?php echo $hooktab ?>
 </ul>
@@ -401,32 +389,6 @@ for ($i=0; $i < $num_new_boxes; $i++) {
 			<!--OPTIONAL ELEMENTS-->
 			<?php echo $optionalelems?>
 			<!--END OPTIONAL ELEMENTS-->
-			<!--ROUTE POSITION-->
-			<div class="element-container">
-				<div class="row">
-					<div class="col-md-12">
-						<div class="row">
-							<div class="form-group">
-								<div class="col-md-3">
-									<label class="control-label" for="route_seq"><?php echo _("Route Position")?></label>
-									<i class="fa fa-question-circle fpbx-help-icon" data-for="route_seq"></i>
-								</div>
-								<div class="col-md-9">
-									<select name="route_seq" class="form-control">
-									<?php echo $routeseqopts ?>
-									</select>
-								</div>
-							</div>
-						</div>
-					</div>
-				</div>
-				<div class="row">
-					<div class="col-md-12">
-						<span id="route_seq-help" class="help-block fpbx-help-block"><?php echo _("Where to insert this route or relocate it relative to the other routes.")?></span>
-					</div>
-				</div>
-			</div>
-			<!--END ROUTE POSITION-->
 			<!--TRUNK PRIORITY-->
 			<div class="element-container">
 				<div class="row">
@@ -543,6 +505,163 @@ for ($i=0; $i < $num_new_boxes; $i++) {
 				</div>
 			</div>
 			<!--END IMPORT/EXPORT-->
+			<!--NOTIFICATIONS-->
+			<div class="tab-pane" id="notifications">
+				<div class="well well-info">
+					<?php echo _("Use caution when enabling Notifications for routes with high traffic to avoid excessive calls, and emails.")?>
+				</div>
+				<!--NOTIFICATION WHEN-->
+				<div class="element-container">
+					<div class="row">
+						<div class="col-md-12">
+							<div class="row">
+								<div class="form-group">
+									<div class="col-md-3">
+										<label class="control-label" for="notification_on"><?php echo _("Notification when") ?></label>
+										<i class="fa fa-question-circle fpbx-help-icon" data-for="notification_on"></i>
+									</div>
+									<div class="col-md-9 radioset">
+										<span class="radioset">
+										<input type="radio" name="notification_on" id="call" value="call" <?php echo (empty($notification_on) || $notification_on == "call"?"CHECKED":"") ?>>
+										<label for="call"><?php echo _("Call successful");?></label>
+										<input type="radio" name="notification_on" id="pattern" value="pattern" <?php echo ($notification_on == "pattern"?"CHECKED":"") ?>>
+										<label for="pattern"><?php echo _("Dial pattern matched");?></label>
+										</span>
+									</div>
+								</div>
+							</div>
+						</div>
+					</div>
+					<div class="row">
+						<div class="col-md-12">
+							<span id="notification_on-help" class="help-block fpbx-help-block"><?php echo _("If the option 'Call successful' is selected, an email notification will be sent on post answer. Selecting 'Dial pattern matched' will send notification email as soon as the dialled pattern is getting matched, so in this case, the notification may not have correct information about trunk & sent outbound caller id.")?></span>
+						</div>
+					</div>
+				</div>
+				<!--NOTIFICATION WHEN-->
+				<!--Email To-->
+				<div class="element-container">
+				  <div class="row">
+					<div class="col-md-12">
+					  <div class="row">
+						<div class="form-group">
+						  <div class="col-md-3">
+							<label class="control-label" for="emailto"><?php echo _("Email To") ?></label>
+							<i class="fa fa-question-circle fpbx-help-icon" data-for="emailto"></i>
+						  </div>
+						  <div class="col-md-9">
+							<input type="text" class="form-control" id="emailto" name="emailto" value="<?php echo isset($emailto)?$emailto:''?>">
+						  </div>
+						</div>
+					  </div>
+					</div>
+				  </div>
+				  <div class="row">
+					<div class="col-md-12">
+					  <span id="emailto-help" class="help-block fpbx-help-block"><?php echo _("The email address you want the notifications sent to.  Leave this blank to disable email notifications.")?></span>
+					</div>
+				  </div>
+				</div>
+				<!--END Email To-->
+				<!--Email From-->
+				<div class="element-container">
+				  <div class="row">
+					<div class="col-md-12">
+					  <div class="row">
+						<div class="form-group">
+						  <div class="col-md-3">
+							<label class="control-label" for="emailfrom"><?php echo _("Email From") ?></label>
+							<i class="fa fa-question-circle fpbx-help-icon" data-for="emailfrom"></i>
+						  </div>
+						  <div class="col-md-9">
+							<input type="text" class="form-control" id="emailfrom" name="emailfrom" value="<?php echo isset($emailfrom)?$emailfrom:''?>">
+						  </div>
+						</div>
+					  </div>
+					</div>
+				  </div>
+				  <div class="row">
+					<div class="col-md-12">
+					  <span id="emailfrom-help" class="help-block fpbx-help-block"><?php echo _("The email address you want the notification emails to appear  to come from.  Keep in mind this may need to be a real email box, and  the address of this server may need to be added to your SPF records to  ensure delivery of the emails.")?></span>
+					</div>
+				  </div>
+				</div>
+				<!--END Email From-->
+				<!--Email Subject-->
+				<div class="element-container">
+				  <div class="row">
+					<div class="col-md-12">
+					  <div class="row">
+						<div class="form-group">
+						  <div class="col-md-3">
+							<label class="control-label" for="emailsubject"><?php echo _("Email Subject") ?></label>
+							<i class="fa fa-question-circle fpbx-help-icon" data-for="emailsubject"></i>
+						  </div>
+						  <div class="col-md-9">
+							<input type="text" class="form-control" id="emailsubject" name="emailsubject" value="<?php echo isset($emailsubject)?$emailsubject:''?>">
+						  </div>
+						</div>
+					  </div>
+					</div>
+				  </div>
+				  <div class="row">
+					<div class="col-md-12">
+					  <span id="emailsubject-help" class="help-block fpbx-help-block"><?php echo _("Saving with a blank value will restore the default.
+						<br />Available Variables for Email Subject and Body:
+						<br />
+						<b>{{CALLUID}}</b> - Call's Unique ID<br />
+						<b>{{ROUTENAME}}</b> - Outbound Route's name<br />
+						<b>{{DIALEDNUMBER}}</b> - Destination number as it will be sent to the trunk<br />
+						<b>{{DIALEDNUMBERRAW}}</b> - Destination number as it was dialed<br /> 
+						<b>{{CALLERNAME}}</b> - Caller's name<br />
+						<b>{{CALLERNUMBER}}</b> - Caller's number<br />
+						<b>{{CALLERALL}}</b> - example:  \"CALLERNAME\" &lt;CALLERNUMBER&gt;<br />
+						<b>{{OUTGOINGCALLERIDNAME}}</b> - Outgoing CallerID name<br />
+						<b>{{OUTGOINGCALLERIDNUMBER}}</b> - Outgoing CallerID number<br />
+						<b>{{OUTGOINGCALLERIDALL}}</b> - example: \"OUTGOINGCALLERIDNAME\" &lt;OUTGOINGCALLERIDNUMBER&gt;<br />
+						<b>{{TRUNKNAME}}</b> - Trunk name<br />
+						<b>{{MONTH}}</b> - 2 digit month<br />
+						<b>{{DAY}}</b> - 2 digit day<br />
+						<b>{{YEAR}}</b> - 4 digit year<br />
+						<b>{{TIME}}</b> - example: 17:30:00<br />
+						<b>{{TIMEAMPM}}</b> - example: 5:30:00 PM<br />
+						<b>{{TZFULL}}</b> - Full Timezone.  ex. America/New_York<br />
+						<b>{{TZSHORT}}</b> - Short Timezone.  ex. UTC, PST, +12
+						");
+						echo isset($hooksAdditionalContent['emailHelpText']) ? $hooksAdditionalContent['emailHelpText'] : '';
+						?></span>
+					</div>
+				  </div>
+				</div>
+				<!--END Email Subject-->
+				<!--Email Body-->
+				<div class="element-container">
+				  <div class="row">
+					<div class="col-md-12">
+					  <div class="row">
+						<div class="form-group">
+						  <div class="col-md-3">
+							<label class="control-label" for="emailbody"><?php echo _("Email Body") ?></label>
+							<i class="fa fa-question-circle fpbx-help-icon" data-for="emailbody"></i>
+						  </div>
+						  <div class="col-md-9">
+							<textarea class="form-control" rows="20" id="emailbody" name="emailbody"><?php echo isset($emailbody)?htmlentities($emailbody):''?></textarea>
+						  </div>
+						</div>
+					  </div>
+					</div>
+				  </div>
+				  <div class="row">
+					<div class="col-md-12">
+					  <span id="emailbody-help" class="help-block fpbx-help-block"><?php echo _("Saving with a blank value will restore the default.
+							<br>See Email Subject Help message for available variables.")?></span>
+					</div>
+				  </div>
+				</div>
+				<!--END Email Body-->
+			</div>
+			<!--END NOTIFICATIONS-->
+			<?php echo $hooks['hookContent']?>
 			<!--Hooks in the "Additional Settings tab -->
 			<div class="tab-pane" id="additionalsettings">
 				<?php //echo $module_hook->hookHtml; ?>
@@ -611,6 +730,12 @@ for ($i=0; $i < $num_new_boxes; $i++) {
 					<input type="checkbox" id="fwld">
 					<label for="fwld"><?php echo _("Long Distance")?></label>
 					</div>
+					<div class = "form-group radioset">
+						<input type="checkbox" id="fwinfoeu">
+						<label for="fwinfoeu"><?php echo _("EU Information")?></label>
+						<input type="checkbox" id="fwemergencyeu" checked>
+						<label for="fwemergencyeu"><?php echo _("EU Emergency")?></label>
+					</div>
 					<div id ="lpresults"></div>
 				</div>
 				<div class="modal-footer">
@@ -630,4 +755,9 @@ if(!empty($route_names)){
 	echo "routing_names = " . json_encode($route_names) . ";";
 }
 ?>
+$(document).ready(function(){
+	if($("#paging").length){
+		$("#paging").detach().appendTo("#notifications");
+	}
+})
 </script>
