@@ -60,6 +60,22 @@
 			</div>
 		</div>
 		<!--END USERNAME-->
+		<div class="element-container">
+			<div class="row">
+				<div class="form-group">
+					<div class="col-md-3">
+						<label class="control-label" for="secret"><?php echo _("Auth username") ?></label>
+						 <i class="fa fa-question-circle fpbx-help-icon" data-for="auth_username"></i>
+					</div>
+					<div class="col-md-9">
+						<input type="text" class="form-control" name="auth_username" id="auth_username" value="<?php echo $auth_username ?>">
+					</div>
+				</div>
+				<div class="col-md-12">
+					<span id="auth_username-help" class="help-block fpbx-help-block"><?php echo _("This need to be set only  when auth username not same as the username. ")?> </span>
+				</div>
+			</div>
+		</div>
 		<!--SECRET-->
 		<div class="element-container">
 			<div class="row">
@@ -257,9 +273,36 @@
 				</div>
 			</div>
 		</div><!--END TRANSPORT-->
-		<?php 
-		$ver_list = array("13.24.0", "16.1.0");
+		<?php
+		$ver_list = array("13.24.0", "16.1.0", "18.0.0");
 		if(version_min(\FreePBX::Config()->get('ASTVERSION'),$ver_list) == true) { ?>
+		<!--PJSIP TRUNK LINE ENABLE / DISABLE-->
+		<div class="element-container">
+			<div class="row">
+				<div class="col-md-12">
+					<div class="row">
+						<div class="form-group">
+							<div class="col-md-3">
+								<label class="control-label" for="pjsip line"><?php echo _("Send Line in Registration") ?></label>
+								<i class="fa fa-question-circle fpbx-help-icon" data-for="pjsip_linew"></i>
+							</div>
+							<div class="col-md-9 radioset">
+								<input type="radio" name="pjsip_line" id="pjsip_lineyes" value="true" <?php echo ($pjsip_line == "true" || empty($pjsip_line)?"CHECKED":"") ?>>
+								<label for="pjsip_lineyes"><?php echo _("Yes")?></label>
+								<input type="radio" name="pjsip_line" id="pjsip_lineno" value="false" <?php echo ($pjsip_line == "false" ?"CHECKED":"") ?>>
+								<label for="pjsip_lineno"><?php echo _("No")?></label>
+							</div>
+						</div>
+					</div>
+				</div>
+			</div>
+			<div class="row">
+				<div class="col-md-12">
+					<span id="pjsip_linew-help" class="help-block fpbx-help-block"><?php echo _("If this option is enabled, a “line” parameter is added to the outgoing “Contact” header during registration.")?></span>
+				</div>
+			</div>
+		</div>
+		<!--PJSIP TRUNK LINE ENABLE / DISABLE-->
 		<!--SEND CONNECTED LINE-->
 		<div class="element-container">
 			<div class="row">
@@ -498,6 +541,33 @@
 			</div>
 		</div>
 		<!--END OUTBOUND PROXY-->
+		<!--USER EQ PHONE-->
+		<div class="element-container">
+			<div class="row">
+				<div class="col-md-12">
+					<div class="row">
+						<div class="form-group">
+							<div class="col-md-3">
+								<label class="control-label" for="user_eq_phone"><?php echo _("User = Phone") ?></label>
+								<i class="fa fa-question-circle fpbx-help-icon" data-for="user_eq_phone"></i>
+							</div>
+							<div class="col-md-9 radioset">
+								<input type="radio" name="user_eq_phone" id="user_eq_phoneyes" value="yes" <?php echo ($user_eq_phone == "yes") ? "CHECKED" : "" ?>>
+								<label for="user_eq_phoneyes"><?php echo _("Yes")?></label>
+								<input type="radio" name="user_eq_phone" id="user_eq_phoneno" value="no" <?php echo ($user_eq_phone == "no" || empty($user_eq_phone)) ? "CHECKED" : "" ?>>
+								<label for="user_eq_phoneno"><?php echo _("No")?></label>
+							</div>
+						</div>
+					</div>
+				</div>
+			</div>
+			<div class="row">
+				<div class="col-md-12">
+					<span id="user_eq_phone-help" class="help-block fpbx-help-block"><?php echo _("Determines whether a user=phone parameter is placed into the request URI if the user is determined to be a phone number.")?></span>
+				</div>
+			</div>
+		</div>
+		<!--END USER EQ PHONE-->
 		<!--CONTACT USER-->
 		<div class="element-container">
 			<div class="row">
@@ -1247,17 +1317,21 @@ function checkAuthButtons() {
 	// username/secret is disabled.
 	var a = $("input[name=authentication]:checked").val();
 	if (a === "off") {
-		$("#username,#secret").attr('placeholder', '<?php echo _("Authentication Disabled"); ?>');
+		$("#username,#secret,#auth_username").attr('placeholder', '<?php echo _("Authentication Disabled"); ?>');
 		if ($("#username").val().length) {
 			// It's set to something. Remove it.
 			$("#username").data("origval", $("#username").val());
 		}
+		if ($("#auth_username").val().length) {
+			// It's set to something. Remove it.
+			$("#auth_username").data("origval", $("#auth_username").val());
+		}
 		if ($("#secret").val().length) {
 			// It's set to something. Remove it.
 			$("#secret").data("origval", $("#secret").val());
 		}
 		$("#secret").removeClass("clicktoedit");
-		$("#username,#secret").val("").prop("readonly", true);
+		$("#username,#secret,#auth_username").val("").prop("readonly", true);
 		$("#registrationnone").click();
 	} else if (a === "inbound" || a === "both") {
 		// Username is not settable, as it is the trunk name that is used for auth
@@ -1267,7 +1341,12 @@ function checkAuthButtons() {
 			// It's set to something. Remove it.
 			$("#username").data("origval", $("#username").val());
 		}
-		$("#username").attr('placeholder', '<?php echo _("Username is trunk name"); ?>').prop("readonly", true).val("");
+		if ($("#auth_username").val().length) {
+			// It's set to something. Remove it.
+			$("#auth_username").data("origval", $("#auth_username").val());
+		}
+		$("#username,#auth_username").attr('placeholder', '<?php echo _("username is trunk name"); ?>').prop("readonly", true).val("");
+		$("#auth_username").attr('placeholder', '<?php echo _("Auth Username is trunk name"); ?>').prop("readonly", true).val("");
 		$("#secret").prop("readonly", false).attr('placeholder', '');
 		if (typeof $("#secret").data("origval") !== "undefined" && $("#secret").data("origval") !== false) {
 			$("#secret").val($("#secret").data("origval"));
@@ -1276,7 +1355,7 @@ function checkAuthButtons() {
 	} else {
 		// Make sure they're not readonly...
 		$("#secret").addClass("clicktoedit");
-		$("#username,#secret").prop("readonly", false).attr('placeholder', '');
+		$("#username,#secret,#auth_username").prop("readonly", false).attr('placeholder', '');
 		// If they had anything previously, put them back.
 		if (typeof $("#username").data("origval") !== "undefined" && $("#username").data("origval") !== false) {
 			$("#username").val($("#username").data("origval"));
@@ -1286,6 +1365,10 @@ function checkAuthButtons() {
 			$("#secret").val($("#secret").data("origval"));
 			$("#secret").data("origval", false);
 		}
+		if (typeof $("#auth_username").data("origval") !== "undefined" && $("#auth_username").data("origval") !== false) {
+			$("#auth_username").val($("#auth_username").data("origval"));
+			$("#auth_username").data("origval", false);
+		}
 	}
 }
 
