@@ -1,5 +1,7 @@
 <?php
 namespace FreePBX\modules;
+use BMO;
+use PDO;
 // vim: set ai ts=4 sw=4 ft=php:
 if(!function_exists('music_list')) {
 	include(__DIR__.'/functions.inc.php');
@@ -42,6 +44,15 @@ class Music implements \BMO {
 		}
 	}
 
+	public function setDatabase($pdo){
+		$this->db = $pdo;
+		return $this;
+	}
+	public function resetDatabase(){
+		$this->db = $this->FreePBX->Database;
+		return $this;
+	}
+
 	public function showPage() {
 		$mh = $this;
 		$heading = _("On Hold Music");
@@ -126,7 +137,26 @@ class Music implements \BMO {
 		needreload();
 	}
 
+	/**
+	 * Add Category by generated id
+	 *
+	 * @param string $name The Category name
+	 * @param string $type The Category type
+	 * @return void
+	 */
 	public function addCategory($name,$type) {
+		$this->addCategoryById(null, $name,$type);
+	}
+
+	/**
+	 * Add Category by ID
+	 *
+	 * @param integer $id The Category ID
+	 * @param string $name The Category name
+	 * @param string $type The Category type
+	 * @return void
+	 */
+	public function addCategoryById($id, $name,$type) {
 		$name = iconv("UTF-8", "ISO-8859-1//TRANSLIT", $name);
 		$name = preg_replace("/\s+|'+|`+|\\+|\/+|\"+|<+|>+|\?+|\*|\.+|&+|\|/","",$name);
 		$name = basename($name);
@@ -138,9 +168,9 @@ class Music implements \BMO {
 			);
 			return;
 		}
-		$sql = "INSERT INTO music (`category`, `type`) VALUES (?,?)";
+		$sql = "INSERT INTO music (`id`, `category`, `type`) VALUES (?, ?,?)";
 		$sth = $this->db->prepare($sql);
-		$sth->execute(array($name,$type));
+		$sth->execute(array($id,$name,$type));
 
 		if(!file_exists($this->mohpath . "/" . $name)) {
 			mkdir($this->mohpath . "/" . $name);
@@ -254,50 +284,6 @@ class Music implements \BMO {
 	}
 
 	public function install() {
-		$table = $this->FreePBX->Database->migrate("music");
-		$cols = array(
-			"id" => array(
-				"type" => "integer",
-				"primaryKey" => true,
-				"autoincrement" => true
-			),
-			"category" => array(
-				"type" => "string",
-				"length" => 190,
-				"notnull" => false,
-			),
-			"type" => array(
-				"type" => "string",
-				"length" => 100,
-				"notnull" => false,
-			),
-			"random" => array(
-				"type" => "boolean",
-				"notnull" => false,
-				"default" => 0
-			),
-			"application" => array(
-				"type" => "string",
-				"length" => 255,
-				"notnull" => false,
-			),
-			"format" => array(
-				"type" => "string",
-				"length" => 10,
-				"notnull" => false,
-			),
-		);
-		$indexes = array(
-			"category_UNIQUE" => array(
-				"type" => "unique",
-				"cols" => array(
-					"category"
-				)
-			)
-		);
-		$table->modify($cols,$indexes);
-		unset($table);
-
 		$freepbx_conf = $this->FreePBX->Config;
 		if ($freepbx_conf->conf_setting_exists('AMPMPG123')) {
 			$freepbx_conf->remove_conf_setting('AMPMPG123');
@@ -321,7 +307,7 @@ class Music implements \BMO {
 		$sql = "SELECT * FROM music WHERE category = 'default'";
 		$sth = $this->db->prepare($sql);
 		$sth->execute();
-		$default = $sth->fetch(\PDO::FETCH_ASSOC);
+		$default = $sth->fetch(PDO::FETCH_ASSOC);
 		if(empty($default)) {
 			$random = file_exists($this->mohpath."/.random") ? 1 : 0;
 			$sql = "INSERT INTO music (`category`, `type`, `random`) VALUES (?,?,?)";
@@ -334,7 +320,7 @@ class Music implements \BMO {
 			$sql = "SELECT * FROM music WHERE category = ?";
 			$sth = $this->db->prepare($sql);
 			$sth->execute(array($category));
-			$c = $sth->fetch(\PDO::FETCH_ASSOC);
+			$c = $sth->fetch(PDO::FETCH_ASSOC);
 			if(!empty($c)) {
 				continue;
 			}
@@ -364,12 +350,7 @@ class Music implements \BMO {
 	public function uninstall() {
 
 	}
-	public function backup(){
 
-	}
-	public function restore($backup){
-
-	}
 
 	/**
 	 * Get all categories
@@ -379,7 +360,7 @@ class Music implements \BMO {
 		$sql = "SELECT * FROM music";
 		$sth = $this->db->prepare($sql);
 		$sth->execute();
-		$cats = $sth->fetchAll(\PDO::FETCH_ASSOC);
+		$cats = $sth->fetchAll(PDO::FETCH_ASSOC);
 		return !empty($cats) ? $cats : array();
 	}
 
@@ -714,7 +695,7 @@ class Music implements \BMO {
 		$sql = "SELECT * FROM music WHERE category = ?";
 		$sth = $this->db->prepare($sql);
 		$sth->execute(array($name));
-		return $sth->fetch(\PDO::FETCH_ASSOC);
+		return $sth->fetch(PDO::FETCH_ASSOC);
 	}
 
 	/**
@@ -726,7 +707,7 @@ class Music implements \BMO {
 		$sql = "SELECT * FROM music WHERE id = ?";
 		$sth = $this->db->prepare($sql);
 		$sth->execute(array($id));
-		return $sth->fetch(\PDO::FETCH_ASSOC);
+		return $sth->fetch(PDO::FETCH_ASSOC);
 	}
 
 	/**
