@@ -13,6 +13,7 @@ class Pm2 extends \FreePBX_Helpers implements \BMO {
 	private $npmver = "2.15.11";
 	private $pm2Home = "/tmp";
 	private $nodeloc = "/tmp";
+	private static $pm2app = false;
 
 	public function __construct($freepbx = null) {
 		$this->astman = $freepbx->astman;
@@ -162,10 +163,10 @@ class Pm2 extends \FreePBX_Helpers implements \BMO {
 		}
 
 		try {
-			$this->runPM2Command("update");
+			$this->pm2Apps()->runPM2Command("update");
 		} catch(\Exception $e) {}
 
-		$this->runPM2Command("ping");
+		$this->pm2Apps()->runPM2Command("ping");
 	}
 
 	public function uninstall() {
@@ -214,7 +215,7 @@ class Pm2 extends \FreePBX_Helpers implements \BMO {
 			throw new \Exception("There is already a process by that name running!");
 		}
 		try {
-			$this->runPM2Command("delete ".$name);
+			$this->pm2Apps()->runPM2Command("delete ".$name);
 		} catch(\Exception $e) {}
 		$processParts = explode(" ",$process,2);
 		$force = ($force) ? '-f' : '';
@@ -224,7 +225,7 @@ class Pm2 extends \FreePBX_Helpers implements \BMO {
 		$outLog = $PM2DISABLELOG ? '/dev/null' : $astlogdir."/".$name."_out.log";
 		$cwd = dirname($process);
 		$args = !empty($processParts[1]) ? ' -- '.$processParts[1] : '';
-		$this->runPM2Command("start ".$processParts[0]." ".$force." --update-env --name ".escapeshellarg($name)." -e ".escapeshellarg($errorLog)." -o ".escapeshellarg($outLog)." --log ".escapeshellarg("/dev/null")." --merge-logs --log-date-format 'YYYY-MM-DD HH:mm Z'".$args, $cwd, $environment);
+		$this->pm2Apps()->runPM2Command("start ".$processParts[0]." ".$force." --update-env --name ".escapeshellarg($name)." -e ".escapeshellarg($errorLog)." -o ".escapeshellarg($outLog)." --log ".escapeshellarg("/dev/null")." --merge-logs --log-date-format 'YYYY-MM-DD HH:mm Z'".$args, $cwd, $environment);
 		return $this->getStatus($name);
 	}
 
@@ -235,7 +236,7 @@ class Pm2 extends \FreePBX_Helpers implements \BMO {
 			throw new \Exception("There is already a process by that name running!");
 		}
 		try {
-			$this->runPM2Command("delete ".$name);
+			$this->pm2Apps()->runPM2Command("delete ".$name);
 		} catch(\Exception $e) {}
 		$processParts = explode(" ",$process,2);
 		$force = ($force) ? '-f' : '';
@@ -244,7 +245,7 @@ class Pm2 extends \FreePBX_Helpers implements \BMO {
 		$errorLog = $PM2DISABLELOG ? '/dev/null' : $astlogdir."/".$name."_err.log";
 		$outLog = $PM2DISABLELOG ? '/dev/null' : $astlogdir."/".$name."_out.log";
 		$args = !empty($processParts[1]) ? ' -- '.$processParts[1] : '';
-		$this->runPM2Command("start ".$processParts[0]." ".$force." --update-env --name ".escapeshellarg($name)." -e ".escapeshellarg($errorLog)." -o ".escapeshellarg($outLog)." --log ".escapeshellarg("/dev/null")." --merge-logs --log-date-format 'YYYY-MM-DD HH:mm Z'".$args, $directory, $environment);
+		$this->pm2Apps()->runPM2Command("start ".$processParts[0]." ".$force." --update-env --name ".escapeshellarg($name)." -e ".escapeshellarg($errorLog)." -o ".escapeshellarg($outLog)." --log ".escapeshellarg("/dev/null")." --merge-logs --log-date-format 'YYYY-MM-DD HH:mm Z'".$args, $directory, $environment);
 		return $this->getStatus($name);
 	}
 
@@ -259,7 +260,7 @@ class Pm2 extends \FreePBX_Helpers implements \BMO {
 		if(empty($out)) {
 			throw new \Exception("There is no process by that name");
 		}
-		$this->runPM2Command("stop ".$name);
+		$this->pm2Apps()->runPM2Command("stop ".$name);
 	}
 
 	/**
@@ -273,7 +274,7 @@ class Pm2 extends \FreePBX_Helpers implements \BMO {
 		if(empty($out)) {
 			throw new \Exception("There is no process by that name");
 		}
-		$this->runPM2Command("restart ".$name." --update-env");
+		$this->pm2Apps()->runPM2Command("restart ".$name." --update-env");
 	}
 
 	/**
@@ -287,7 +288,7 @@ class Pm2 extends \FreePBX_Helpers implements \BMO {
 		if(empty($out)) {
 			throw new \Exception("There is no process by that name");
 		}
-		$this->runPM2Command("delete ".$name);
+		$this->pm2Apps()->runPM2Command("delete ".$name);
 	}
 
 	/**
@@ -295,7 +296,7 @@ class Pm2 extends \FreePBX_Helpers implements \BMO {
 	 * @method update
 	 */
 	public function update() {
-		$this->runPM2Command("update",'',array(),true);
+		$this->pm2Apps()->runPM2Command("update",'',array(),true);
 	}
 
 	/**
@@ -308,12 +309,12 @@ class Pm2 extends \FreePBX_Helpers implements \BMO {
 		if(empty($out)) {
 			throw new \Exception("There is no process by that name");
 		}
-		$this->runPM2Command("reset ".$name);
+		$this->pm2Apps()->runPM2Command("reset ".$name);
 	}
 
 	public function reloadLogs() {
 		try {
-			$this->runPM2Command("reloadLogs",'',array(),false,5,5);
+			$this->pm2Apps()->runPM2Command("reloadLogs",'',array(),false,5,5);
 		} catch(\Exception $e) {
 			//https://github.com/Unitech/pm2/issues/3521
 			if(get_class($e) == "Symfony\Component\Process\Exception\ProcessTimedOutException") {
@@ -329,59 +330,25 @@ class Pm2 extends \FreePBX_Helpers implements \BMO {
 	 * @return array        Array of processes
 	 */
 	public function listProcesses() {
-		$output = $this->runPM2Command("jlist");
+		$output = $this->pm2Apps()->runPM2Command("jlist");
 		$processes = json_decode($output,true);
 		//check for errors because of this:
 		// [PM2] Spawning PM2 daemon with pm2_home=/home/asterisk/.pm2
 		// [PM2] PM2 Successfully daemonized
 		if(json_last_error() !== JSON_ERROR_NONE) {
-			$output = $this->runPM2Command("jlist");
+			$output = $this->pm2Apps()->runPM2Command("jlist");
 			$processes = json_decode($output,true);
 		}
 		$processes = (!empty($processes) && is_array($processes)) ? $processes : array();
 		$final = array();
 		foreach($processes as $process) {
-			$process['pm2_env']['created_at_human_diff'] = ($process['pm2_env']['status'] == 'online') ? $this->get_date_diff(time(),(int)round($process['pm2_env']['created_at']/1000)) : 0;
-			$process['monit']['human_memory'] = $this->human_filesize($process['monit']['memory']);
+			$process['pm2_env']['created_at_human_diff'] = ($process['pm2_env']['status'] == 'online') ? $this->pm2Apps()->get_date_diff(time(),(int)round($process['pm2_env']['created_at']/1000)) : 0;
+			$process['monit']['human_memory'] = $this->pm2Apps()->human_filesize($process['monit']['memory']);
 			$final[] = $process;
 		}
 		return $final;
 	}
 
-	/**
-	 * Run a command against PM2
-	 * @method runPM2Command
-	 * @param  string        $cmd    The command to run
-	 * @param  boolean       $stream Whether to stream the output or return it
-	 */
-	private function runPM2Command($cmd,$cwd='',$environment=array(),$stream=false,$timeout=240,$idleTimeout=null) {
-		if(!file_exists($this->nodeloc."/node_modules/pm2/bin/pm2")){
-			throw new \Exception("pm2 binary does not exist run fwconsole ma install pm2 and try again", 1);
-		}
-		if(!is_executable($this->nodeloc."/node_modules/pm2/bin/pm2")) {
-			chmod($this->nodeloc."/node_modules/pm2/bin/pm2",0755);
-		}
-		$command = $this->generateRunAsAsteriskCommand($this->nodeloc."/node_modules/pm2/bin/pm2 ".$cmd,$cwd,$environment);
-		$process = new Process($command);
-		$process->setIdleTimeout($timeout);
-		if(!empty($idleTimeout)) {
-			$process->setTimeout($idleTimeout);
-		}
-		if(!$stream) {
-			$process->mustRun();
-			return $process->getOutput();
-		} else {
-			$process->setTty(true);
-			$process->run(function ($type, $buffer) {
-				if (Process::ERR === $type) {
-					echo $buffer;
-				} else {
-					echo $buffer;
-				}
-			});
-		}
-	}
-
 	public function chownFreepbx() {
 		$files = array();
 		$files[] = array('type' => 'execdir',
@@ -417,7 +384,7 @@ class Pm2 extends \FreePBX_Helpers implements \BMO {
 	public function installNodeDependencies($cwd='',$callback=null,$environment=array(),$production=true) {
 		$cwd = !empty($cwd) ? $cwd : $this->nodeloc;
 		if($this->freepbx->Config->get('PM2USECACHE')) {
-			$command = $this->generateRunAsAsteriskCommand('npm-cache -v',$cwd,$environment);
+			$command = $this->pm2Apps()->generateRunAsAsteriskCommand('npm-cache -v',$cwd,$environment);
 			$process = new Process($command);
 			try {
 				$process->mustRun();
@@ -425,10 +392,10 @@ class Pm2 extends \FreePBX_Helpers implements \BMO {
 					$callback("Found npm-cache v".$process->getOutput());
 				}
 			} catch (ProcessFailedException $e) {
-				$command = $this->generateRunAsAsteriskCommand('npm install -g npm-cache',$cwd,$environment);
+				$command = $this->pm2Apps()->generateRunAsAsteriskCommand('npm install -g npm-cache',$cwd,$environment);
 				exec($command);
 
-				$command = $this->generateRunAsAsteriskCommand('npm-cache -v',$cwd,$environment);
+				$command = $this->pm2Apps()->generateRunAsAsteriskCommand('npm-cache -v',$cwd,$environment);
 				$process = new Process($command);
 				try {
 					$process->mustRun();
@@ -461,9 +428,9 @@ class Pm2 extends \FreePBX_Helpers implements \BMO {
 
 		$prod = ($production) ? ' --only=production' : '';
 		if($this->freepbx->Config->get('PM2USECACHE')) {
-			$command = $this->generateRunAsAsteriskCommand('npm-cache install'.$prod,$cwd,$environment);
+			$command = $this->pm2Apps()->generateRunAsAsteriskCommand('npm-cache install'.$prod,$cwd,$environment);
 		} else {
-			$command = $this->generateRunAsAsteriskCommand('npm install'.$prod,$cwd,$environment);
+			$command = $this->pm2Apps()->generateRunAsAsteriskCommand('npm install'.$prod,$cwd,$environment);
 		}
 		if(!$PM2DISABLELOG) {
 			$log = fopen($cwd."/logs/install.log", "a");
@@ -499,85 +466,6 @@ class Pm2 extends \FreePBX_Helpers implements \BMO {
 		return $process->isSuccessful();
 	}
 
-	/**
-	 * Generate run command string
-	 * @method generateRunAsAsteriskCommand
-	 * @param  string                       $command The command to run
-	 * @param  string                       $environment Array of environment variables to run
-	 * @return string                                The finalized command
-	 */
-	public function generateRunAsAsteriskCommand($command,$cwd='',$environment=array()) {
-		$cwd = !empty($cwd) ? $cwd : $this->nodeloc;
-		$webuser = $this->freepbx->Config->get('AMPASTERISKWEBUSER');
-		$webgroup = $this->freepbx->Config->get('AMPASTERISKWEBGROUP');
-		$webroot = $this->freepbx->Config->get("AMPWEBROOT");
-		$varlibdir = $this->freepbx->Config->get("ASTVARLIBDIR");
-		$astlogdir = $this->freepbx->Config->get("ASTLOGDIR");
-
-		$npmrc = $this->getHomeDir() . "/.npmrc";
-		if(!file_exists($npmrc)) {
-			touch($npmrc);
-			if (posix_getuid() == 0) {
-				chown($npmrc,$webuser);
-			}
-		}
-
-		$cmds = array(
-			'cd '.$cwd,
-			'mkdir -p '.$this->pm2Home
-		);
-
-		$PM2DISABLELOG = $this->freepbx->Config->get("PM2DISABLELOG");
-
-		if(!$PM2DISABLELOG) {
-			$cmds[] = 'mkdir -p '.$cwd.'/logs';
-		}
-
-		$contents = file_get_contents($npmrc);
-		$contents .= "\n";
-		$ini = parse_ini_string($contents, false, INI_SCANNER_RAW);
-		$ini = is_array($ini) ? $ini : array();
-		$ini['prefix'] = '~/.node';
-		if($this->freepbx->Config->get('PM2USEPROXY')) {
-			$ini['proxy'] = $this->freepbx->Config->get('PM2PROXY');
-			$ini['https-proxy'] = $this->freepbx->Config->get('PM2PROXY');
-			$ini['strict-ssl'] = 'false';
-			$cmds[] = 'export NODE_TLS_REJECT_UNAUTHORIZED=0';
-		} else {
-			unset($ini['proxy'],$ini['https-proxy'],$ini['strict-ssl']);
-		}
-
-		$this->write_php_ini($npmrc,$ini);
-
-		foreach($environment as $k => $v) {
-			if(empty($k) || !is_string($v)) {
-				continue;
-			}
-			$cmds[] = 'export '.escapeshellarg($k).'='.escapeshellarg($v);
-		}
-
-		$cmds = array_merge($cmds,array(
-			'export HOME='.escapeshellcmd($this->getHomeDir()),
-			'export PM2_HOME='.escapeshellcmd($this->pm2Home),
-			'export ASTLOGDIR='.escapeshellcmd($astlogdir),
-			'export ASTVARLIBDIR='.escapeshellcmd($varlibdir),
-			'export PATH=$HOME/.node/bin:$PATH',
-			'export NODE_PATH=$HOME/.node/lib/node_modules:$NODE_PATH',
-			'export MANPATH=$HOME/.node/share/man:$MANPATH',
-
-		));
-		$cmds[] = escapeshellcmd($command);
-		$final = implode(" && ", $cmds);
-
-		if (posix_getuid() == 0) {
-			$shell = $this->freepbx->Config->get('PM2SHELL');
-			$shell = !empty($shell) ? $shell : '/bin/bash';
-			$final = "runuser ".escapeshellarg($webuser)." -s ".escapeshellarg($shell)." -c ".escapeshellarg($final);
-		}
-
-		return $final;
-	}
-
 	/**
 	 * Turn all spaces into underscores and remove all utf8
 	 * from filenames
@@ -596,113 +484,39 @@ class Pm2 extends \FreePBX_Helpers implements \BMO {
 	}
 
 	/**
-	 * Get human readable time difference between 2 dates
+	 * getPm2AppsObj
 	 *
-	 * Return difference between 2 dates in year, month, hour, minute or second
-	 * The $precision caps the number of time units used: for instance if
-	 * $time1 - $time2 = 3 days, 4 hours, 12 minutes, 5 seconds
-	 * - with precision = 1 : 3 days
-	 * - with precision = 2 : 3 days, 4 hours
-	 * - with precision = 3 : 3 days, 4 hours, 12 minutes
-	 *
-	 * From: http://www.if-not-true-then-false.com/2010/php-calculate-real-differences-between-two-dates-or-timestamps/
-	 *
-	 * @param mixed $time1 a time (string or timestamp)
-	 * @param mixed $time2 a time (string or timestamp)
-	 * @param integer $precision Optional precision
-	 * @return string time difference
+	 * @param  mixed $freepbx
+	 * @return void
 	 */
-	private function get_date_diff( $time1, $time2, $precision = 2 ) {
-		// If not numeric then convert timestamps
-		if( !is_int( $time1 ) ) {
-			$time1 = strtotime( $time1 );
+	public function pm2Apps() {
+		if (!self::$pm2app) {
+			if (!class_exists('FreePBX\\modules\\pm2\Pm2Apps')) {
+				include_once __DIR__."/admin/modules/pm2/Pm2Apps.php";
+            }
+
+			$config = array(
+				'homedir' => $this->getHomeDir(),
+				'webuser' => $this->freepbx->Config->get('AMPASTERISKWEBUSER'),
+				'varlibdir' => $this->freepbx->Config->get("ASTVARLIBDIR"),
+				'astlogdir' => $this->freepbx->Config->get("ASTLOGDIR"),
+				'disablelogs' => $this->freepbx->Config->get("PM2DISABLELOG"),
+				'useproxy' => $this->freepbx->Config->get('PM2USEPROXY'),
+				'proxy' => $this->freepbx->Config->get('PM2PROXY'),
+				'shell' => $this->freepbx->Config->get('PM2SHELL'),
+			);
+			self::$pm2app = new \FreePBX\modules\pm2\Pm2Apps($config);
 		}
-		if( !is_int( $time2 ) ) {
-			$time2 = strtotime( $time2 );
-		}
-		// If time1 > time2 then swap the 2 values
-		if( $time1 > $time2 ) {
-			list( $time1, $time2 ) = array( $time2, $time1 );
-		}
-		// Set up intervals and diffs arrays
-		$intervals = array( 'year', 'month', 'day', 'hour', 'minute', 'second' );
-		$diffs = array();
-		foreach( $intervals as $interval ) {
-			// Create temp time from time1 and interval
-			$ttime = strtotime( '+1 ' . $interval, $time1 );
-			// Set initial values
-			$add = 1;
-			$looped = 0;
-			// Loop until temp time is smaller than time2
-			while ( $time2 >= $ttime ) {
-				// Create new temp time from time1 and interval
-				$add++;
-				$ttime = strtotime( "+" . $add . " " . $interval, $time1 );
-				$looped++;
-			}
-			$time1 = strtotime( "+" . $looped . " " . $interval, $time1 );
-			$diffs[ $interval ] = $looped;
-		}
-		$count = 0;
-		$times = array();
-		foreach( $diffs as $interval => $value ) {
-			// Break if we have needed precission
-			if( $count >= $precision ) {
-				break;
-			}
-			// Add value and interval if value is bigger than 0
-			if( $value > 0 ) {
-				if( $value != 1 ){
-					$interval .= "s";
-				}
-				// Add value and interval to times array
-				$times[] = $value . " " . $interval;
-				$count++;
-			}
-		}
-		// Return string with times
-		return implode( ", ", $times );
+		return self::$pm2app;
 	}
-
-	private function human_filesize($bytes, $dec = 2) {
-		$size   = array('B', 'kB', 'MB', 'GB', 'TB', 'PB', 'EB', 'ZB', 'YB');
-		$factor = floor((strlen($bytes) - 1) / 3);
-
-		return sprintf("%.{$dec}f", $bytes / pow(1024, $factor)) . @$size[$factor];
-	}
-
-	function write_php_ini($file, $array) {
-		$res = array();
-		foreach($array as $key => $val) {
-			if(is_array($val)) {
-				$res[] = "[$key]";
-				foreach($val as $skey => $sval) {
-					$res[] = "$skey = ".(is_numeric($sval) ? $sval : '"'.$sval.'"');
-				};
-			} else {
-				$res[] = "$key = ".(is_numeric($val) ? $val : '"'.$val.'"');
-			};
-		}
-		$this->safefilerewrite($file, implode("\r\n", $res));
-	}
-
-	function safefilerewrite($fileName, $dataToSave) {
-		if ($fp = fopen($fileName, 'w')) {
-			$startTime = microtime(TRUE);
-			do {
-				$canWrite = flock($fp, LOCK_EX);
-				// If lock not obtained sleep for 0 - 100 milliseconds, to avoid collision and CPU load
-				if(!$canWrite) {
-					usleep(round(rand(0, 100)*1000));
-				};
-			} while ((!$canWrite)and((microtime(TRUE)-$startTime) < 5));
-
-			//file was locked so now we can store information
-			if ($canWrite) {
-				fwrite($fp, $dataToSave);
-				flock($fp, LOCK_UN);
-			}
-			fclose($fp);
-		}
+			
+	/**
+	 * setPm2Obj
+	 *
+	 * @param  mixed $obj
+	 * @return void
+	 */
+	public function setPm2AppsObj($obj){
+		return self::$pm2app = $obj; 
 	}
 }
