@@ -11,6 +11,7 @@ class Sipsettings extends FreePBX_Helpers implements BMO {
 	private $pagename = null;
 	private $pagedata = null;
 	private $tlsCache = null;
+	private static $natObj = false;
 
 	public static $dbDefaults = array(
 		"rtpstart" => "10000", "rtpend" => "20000",
@@ -27,6 +28,9 @@ class Sipsettings extends FreePBX_Helpers implements BMO {
 		"tcpport-0.0.0.0" => "5160", // Defaults, only used if this is an upgrade
 		"udpport-0.0.0.0" => "5061",
 		"tlsport-0.0.0.0" => "5161",
+		"allow_reload" => "no",
+		"debug" => "no",
+		"keep_alive_interval" => 90
 	);
 
 	public function ajaxRequest($req, &$setting) {
@@ -195,17 +199,17 @@ class Sipsettings extends FreePBX_Helpers implements BMO {
 				)
 			);
 
-			if ($driver == "chan_sip" || $driver == "both") {
-				$this->pagedata['sip'] = array(
-					"name" => _("Chan SIP Settings"),
-					"page" => 'chansip.page.php'
+			if ($driver == "chan_pjsip" || $driver == "both") {
+				$this->pagedata['pjsip'] = array(
+					"name" => _("SIP Settings [chan_pjsip]"),
+					"page" => 'chanpj.page.php'
 				);
 			}
 
-			if ($driver == "chan_pjsip" || $driver == "both") {
-				$this->pagedata['pjsip'] = array(
-					"name" => _("Chan PJSIP Settings"),
-					"page" => 'chanpj.page.php'
+			if ($driver == "chan_sip" || $driver == "both") {
+				$this->pagedata['sip'] = array(
+					"name" => _("SIP Legacy Settings [chan_sip]"),
+					"page" => 'chansip.page.php'
 				);
 			}
 
@@ -231,6 +235,8 @@ class Sipsettings extends FreePBX_Helpers implements BMO {
 			return;
 		}
 
+		$ignoreImportedVars = [];
+		$ignoreImportedRegExp = [];
 
 		if (isset($_POST['ice_blacklist_count'])) {
 			$ice_blacklist = array();
@@ -243,6 +249,9 @@ class Sipsettings extends FreePBX_Helpers implements BMO {
 					);
 				}
 			}
+			$ignoreImportedVars[] = 'ice_blacklist_count';
+			$ignoreImportedRegExp[] = 'ice_blacklist_ip_(.+)$';
+			$ignoreImportedRegExp[] = 'ice_blacklist_subnet_(.+)$';
 			$this->setConfig('ice-blacklist',$ice_blacklist);
 		}
 
@@ -257,6 +266,9 @@ class Sipsettings extends FreePBX_Helpers implements BMO {
 					);
 				}
 			}
+			$ignoreImportedVars[] = 'ice_host_candidates_count';
+			$ignoreImportedRegExp[] = 'ice_host_candidates_local_(.+)$';
+			$ignoreImportedRegExp[] = 'ice_host_candidates_advertised_(.+)$';
 			$this->setConfig('ice-host-candidates',$ice_host_candidates);
 		}
 
@@ -285,7 +297,7 @@ class Sipsettings extends FreePBX_Helpers implements BMO {
 			}
 
 			// Finished. Unset it, and continue on.
-			unset($_REQUEST['voicecodecs']);
+			$ignoreImportedVars[] = 'voicecodecs';
 		}
 
 		// Video Codecs
@@ -314,26 +326,12 @@ class Sipsettings extends FreePBX_Helpers implements BMO {
 			}
 
 			// Finished. Unset it, and continue on.
-			unset($_REQUEST['vcodec']);
+			$ignoreImportedVars[] = 'vcodec';
 		}
 
-		// Ignore empty/invalid localnet settings
-		if (isset($_REQUEST['localnets'])) {
-			foreach ($_REQUEST['localnets'] as $i => $arr) {
-				if (empty($arr['net']) || empty($arr['mask'])) {
-					unset($_REQUEST['localnets'][$i]);
-				}
-			}
-			// Renumber the array
-			if (!empty($_REQUEST['localnets'])) {
-				$_REQUEST['localnets'] = array_values($_REQUEST['localnets']);
-			}
-		}
 
-		// Grab the global externalip, if it's there
-		if (isset($_REQUEST['externip'])) {
-			$this->setConfig('externip', $_REQUEST['externip']);
-		}
+		$ignoreImportedVars[] = 'localnets';
+		$ignoreImportedRegExp[] = '(.+)bindip-(.+)$';
 
 		// get and set pjsip_identifers_order
 		if (isset($_REQUEST['pjsip_identifers_order'])) {
@@ -345,28 +343,42 @@ class Sipsettings extends FreePBX_Helpers implements BMO {
 			}
 			$this->setConfig('pjsip_identifers_order', $pjsip_identifers_filtered);
 		}
-		if (isset($_REQUEST['pjsipcertid'])) {
-			$this->setConfig('pjsipcertid', $_REQUEST['pjsipcertid']);
+
+        if (isset($_REQUEST['allow_reload'])) {
+            $this->setConfig('pjsip_allow_reload', $_REQUEST['allow_reload']);
+        }
+
+        if (isset($_REQUEST['verify_client'])) {
+            $this->setConfig('verify_client', $_REQUEST['verify_client']);
+        }
+
+        if (isset($_REQUEST['verify_server'])) {
+            $this->setConfig('verify_server', $_REQUEST['verify_server']);
+        }
+
+		if (isset($_REQUEST['pjsip_debug'])) {
+			$this->setConfig('pjsip_debug', $_REQUEST['pjsip_debug']);
 		}
 
-		if (isset($_REQUEST['method'])) {
-			$this->setConfig('method', $_REQUEST['method']);
+		if (isset($_REQUEST['pjsip_keep_alive_interval'])) {
+			$this->setConfig('pjsip_keep_alive_interval', $_REQUEST['pjsip_keep_alive_interval']);
 		}
 
-		if (isset($_REQUEST['verify_client'])) {
-			$this->setConfig('verify_client', $_REQUEST['verify_client']);
+		$ver_list=array("13.24.0", "16.1.0", "17.0.0", "18.0.0");
+
+		if (isset($_REQUEST['use_callerid_contact']) && version_min($this->FreePBX->Config->get('ASTVERSION'), $ver_list) == true) {
+			$this->setConfig('pjsip_use_callerid_contact', $_REQUEST['use_callerid_contact']);
 		}
 
-		if (isset($_REQUEST['allow_reload'])) {
-			$this->setConfig('pjsip_allow_reload', $_REQUEST['allow_reload']);
-		}
-
-		if (isset($_REQUEST['verify_server'])) {
-			$this->setConfig('verify_server', $_REQUEST['verify_server']);
+		$asteriskVersions =array("13.25.0", "16.2.0", "17.0.0", "18.0.0");
+		if (isset($_REQUEST['taskprocessor_overload_trigger']) && version_min($this->FreePBX->Config->get('ASTVERSION'), $asteriskVersions) == true) {
+			$this->setConfig('taskprocessor_overload_trigger', $_REQUEST['taskprocessor_overload_trigger']);
 		}
+		
+		$ignoreImportedVars = array_merge($ignoreImportedVars,["display", "type", "category", "Submit"]);
 
 		// This is in Request_Helper.class.php
-		$ignored = $this->importRequest(null, "/(.+)bindip-(.+)$/");
+		$ignored = $this->importRequest($ignoreImportedVars, "/(".implode("|",$ignoreImportedRegExp).")/");
 		// There may be binds that matched..
 		$binds = array();
 		foreach ($ignored as $key => $var) {
@@ -381,6 +393,26 @@ class Sipsettings extends FreePBX_Helpers implements BMO {
 			$this->setConfig("binds", $binds);
 		}
 
+		// Ignore empty/invalid localnet settings
+		if (isset($_REQUEST['localnets'])) {
+			foreach ($_REQUEST['localnets'] as $i => $arr) {
+				if (empty($arr['net']) || empty($arr['mask'])) {
+					unset($_REQUEST['localnets'][$i]);
+				}
+			}
+		}
+
+		// Renumber the array
+		if (!empty($_REQUEST['localnets'])) {
+			$localnets = array_values($_REQUEST['localnets']);
+			foreach($localnets as $nets){
+				$timedlocalnets[] = array_map('trim',$nets);
+			}
+			$this->setConfig('localnets',$timedlocalnets);
+		} else {
+			$this->delConfig('localnets');
+		}
+
 		needreload();
 	}
 
@@ -442,7 +474,7 @@ class Sipsettings extends FreePBX_Helpers implements BMO {
 
 	/**
 	 * Retrieve Active Codecs
-	 * @param {string} $type               The Codec Type
+	 * @param {string} $type			   The Codec Type
 	 * @param {bool} $showDefaults=false Whether to show defaults or not
 	 */
 	public function getCodecs($type,$showDefaults=false) {
@@ -519,7 +551,7 @@ class Sipsettings extends FreePBX_Helpers implements BMO {
 
 	/**
 	 * Update or Set Codecs
-	 * @param {string} $type           Codec Type
+	 * @param {string} $type		   Codec Type
 	 * @param {array} $codecs=array() The codecs with order, if blank set defaults
 	 */
 	public function setCodecs($type,$codecs=array()) {
@@ -566,7 +598,7 @@ class Sipsettings extends FreePBX_Helpers implements BMO {
 		foreach ($raw_settings as $var) {
 			switch ($var['type']) {
 			case self::SIP_NORMAL:
-				$sip_settings[$var['keyword']]                 = $var['data'];
+				$sip_settings[$var['keyword']]				 = $var['data'];
 				break;
 			case self::SIP_CUSTOM:
 				// Check if this is 'tcpenable' and return this as part of the array,
@@ -701,7 +733,7 @@ class Sipsettings extends FreePBX_Helpers implements BMO {
 			$cerid = $this->getConfig('pjsipcertid');
 			$cert = $this->FreePBX->Certman->getCertificateDetails($cerid);
 			if(!empty($cert['files']['crt']) && !empty($cert['files']['key'])) {
-				$this->tlsCache['cert_file'] = $cert['files']['pem'];
+				$this->tlsCache['cert_file'] = isset($cert['files']['fullchain'])?$cert['files']['fullchain']:$cert['files']['crt'];
 				$this->tlsCache['priv_key_file'] = $cert['files']['key'];
 			}
 		} else {
@@ -1181,6 +1213,39 @@ class Sipsettings extends FreePBX_Helpers implements BMO {
 		}
 		return $interfaces;
 	}
+	public function dumpDbConfigs(){
+		return $this->Database->query('SELECT * FROM sipsettings')
+			->fetchAll(PDO::FETCH_ASSOC);
+	}
+
+	public function dumpKVStore($ids=false) {
+		if(!is_array($ids)) {
+			$ids = $this->getAllids();
+			$ids[] = 'noid';
+		}
+		$final = [];
+		foreach($ids as $id) {
+			$final[$id] = $this->getAll($id);
+		}
+		return $final;
+	}
+
+	public function loadDbConfigs($configs){
+		$stmt = $this->Database->prepare("REPLACE INTO sipsettings (keyword, data, seq, type) VALUES (:keyword, :data, :seq, :type)");
+		if (!empty($configs)) {
+				foreach ($configs as $conf) {
+					if(count($conf) !== 4){
+						continue;
+					}
+					$stmt->execute([
+						':keyword' => $conf['keyword'],
+						':data' => $conf['data'],
+						':seq' => $conf['seq'],
+						':type' => $conf['type'],
+					]);
+				}
+		}
+	}
 
 	private function checkPjsipTpReload() {
 		$reloadValue = $this->getConfig("pjsip_allow_reload");
@@ -1191,4 +1256,29 @@ class Sipsettings extends FreePBX_Helpers implements BMO {
 			$this->FreePBX->Notifications->delete("sipsettings","PJSIPTPRELOAD");
 		}
 	}
+	
+	/**
+	 * setNatObj
+	 *
+	 * @param  mixed $obj
+	 * @return void
+	 */
+	public function setNatObj($obj){
+		return self::$natObj = $obj; 
+	}
+	
+	/**
+	 * getNatObj
+	 *
+	 * @return void
+	 */
+	public function getNatObj(){
+		if (!self::$natObj) {
+			if (!class_exists('FreePBX\Modules\Sipsettings\NatGet')) {
+				include __DIR__."/Natget.class.php";
+			}
+			self::$natObj = new \FreePBX\Modules\Sipsettings\NatGet();
+		}
+		return self::$natObj;
+	}
 }
