@@ -165,6 +165,7 @@ foreach ($protocols as $p) {
 ?>
 <input type="hidden" name="category" value="pjsip">
 <input type="hidden" name="Submit" value="Submit">
+<div class="alert alert-info"><?php echo _('These settings apply only to SIP [chan_pjsip]')?></div>
 <div class="section-title" data-for="pjsmisc"><h3>
 	<i class="fa fa-minus"></i> <?php echo _("Misc PJSip Settings")?></h3>
 </div>
@@ -192,6 +193,114 @@ foreach ($protocols as $p) {
 		</div>
 	</div>
 	<!--END Allow reload-->
+	<!--Enable Debug-->
+	<div class="element-container">
+		<div class="row">
+			<div class="form-group">
+				<div class="col-md-3">
+					<label class="control-label" for="pjsip_debug"><?php echo _("Enable Debug") ?></label>
+					<i class="fa fa-question-circle fpbx-help-icon" data-for="pjsip_debug"></i>
+				</div>
+				<div class="col-md-9 radioset">
+					<input type="radio" name="pjsip_debug" id="pjsip_debugyes" value="yes" <?php echo ( $this->getConfig("pjsip_debug") == "yes"?"CHECKED":"") ?>>
+					<label for="pjsip_debugyes"><?php echo _("Yes");?></label>
+					<input type="radio" name="pjsip_debug" id="pjsip_debugno" value="no" <?php echo ( $this->getConfig("pjsip_debug") == "yes"?"":"CHECKED") ?>>
+					<label for="pjsip_debugno"><?php echo _("No");?></label>
+				</div>
+			</div>
+		</div>
+		<div class="row">
+			<div class="col-md-12">
+				<span id="pjsip_debug-help" class="help-block fpbx-help-block"><?php echo _("Enable/Disable SIP debug logging.")?></span>
+			</div>
+		</div>
+	</div>
+	<!--END Enable Debug-->
+	<!--Allow reload-->
+	<div class="element-container">
+		<div class="row">
+			<div class="form-group">
+				<div class="col-md-3">
+					<label class="control-label" for="pjsip_keep_alive_interval"><?php echo _("Keep Alive Interval") ?></label>
+					<i class="fa fa-question-circle fpbx-help-icon" data-for="pjsip_keep_alive_interval"></i>
+				</div>
+				<div class="col-md-9 radioset">
+					<input type="number" class="form-control" name="pjsip_keep_alive_interval" id="pjsip_keep_alive_interval" value="<?php echo ( $this->getConfig("pjsip_keep_alive_interval")?$this->getConfig("pjsip_keep_alive_interval"):"90") ?>">
+				</div>
+			</div>
+		</div>
+		<div class="row">
+			<div class="col-md-12">
+				<span id="pjsip_keep_alive_interval-help" class="help-block fpbx-help-block"><?php echo _("The interval (in seconds) to send keepalives to active connection-oriented transports. <br> Default is 90. </br>")?></span>
+			</div>
+		</div>
+	</div>
+	<!--END Allow reload-->
+
+	<?php 
+	$ver_list=array("13.24.0", "16.1.0", "17.0.0", "18.0.0");
+	if (version_min(\FreePBX::Config()->get('ASTVERSION'),$ver_list) == true) { ?>
+	<!--Caller ID into Contact Header-->
+	<div class="element-container">
+		<div class="row">
+			<div class="form-group">
+				<div class="col-md-3">
+					<label class="control-label" for="use_callerid_contact"><?php echo _("Caller ID into Contact Header") ?></label>
+					<i class="fa fa-question-circle fpbx-help-icon" data-for="use_callerid_contact"></i>
+				</div>
+				<div class="col-md-9 radioset">
+					<input type="radio" name="use_callerid_contact" id="use_callerid_contactyes" value="yes" <?php echo ( $this->getConfig("pjsip_use_callerid_contact") == "yes"?"CHECKED":"") ?>>
+					<label for="use_callerid_contactyes"><?php echo _("Yes");?></label>
+					<input type="radio" name="use_callerid_contact" id="use_callerid_contactno" value="no" <?php echo ( $this->getConfig("pjsip_use_callerid_contact") == "yes"?"":"CHECKED") ?>>
+					<label for="use_callerid_contactno"><?php echo _("No");?></label>
+				</div>
+			</div>
+		</div>
+		<div class="row">
+			<div class="col-md-12">
+				<span id="use_callerid_contact-help" class="help-block fpbx-help-block"><?php echo "<i>use_callerid_contact</i> : "._("Place caller-id information into Contact header. Default: no.")?></span>
+			</div>
+		</div>
+	</div>
+	<!--END Caller ID into Contact Header-->
+	<?php 
+	}
+	?>	
+	
+	<!--Taskprocessor Overload Trigger-->
+	<?php 
+	$asteriskVersions=array("13.25.0", "16.2.0", "17.0.0", "18.0.0");
+	if (version_min(\FreePBX::Config()->get('ASTVERSION'),$asteriskVersions) == true) { ?>
+	<div class="element-container">
+		<div class="row">
+			<div class="col-md-12">
+				<div class="row">
+					<div class="form-group">
+						<div class="col-md-3">
+							<label class="control-label" for="taskprocessor_overload_trigger"><?php echo _("Taskprocessor Overload Trigger")?></label>
+							<i class="fa fa-question-circle fpbx-help-icon" data-for="taskprocessor_overload_trigger"></i>
+						</div>
+						<div class="col-md-9 radioset">
+							<input id="taskprocessor_overload_trigger-global" type="radio" name="taskprocessor_overload_trigger" value="global" <?php echo $this->getConfig("taskprocessor_overload_trigger")=="global"?"checked=\"global\"":""?>/>
+							<label for="taskprocessor_overload_trigger-global"><?php echo _("global") ?></label>
+							<input id="taskprocessor_overload_trigger-pjsip_only" type="radio" name="taskprocessor_overload_trigger" value="pjsip_only" <?php echo (empty($this->getConfig("taskprocessor_overload_trigger")) || $this->getConfig("taskprocessor_overload_trigger")=="pjsip_only")?"checked=\"pjsip_only\"":""?>/>
+							<label for="taskprocessor_overload_trigger-pjsip_only"><?php echo _("pjsip_only") ?></label>
+							<input id="taskprocessor_overload_trigger-none" type="radio" name="taskprocessor_overload_trigger" value="none" <?php echo $this->getConfig("taskprocessor_overload_trigger")=="none"?"checked=\"none\"":""?>/>
+							<label for="taskprocessor_overload_trigger-none">none</label>
+						</div>
+					</div>
+				</div>
+			</div>
+		</div>
+		<div class="row">
+			<div class="col-md-12">
+				<span id="taskprocessor_overload_trigger-help" class="help-block fpbx-help-block"><?php echo _("This option specifies the trigger the distributor will use for detecting taskprocessor overloads. When it detects an overload condition, the distrubutor will stop accepting new requests until the overload is cleared.<br /> global - (default) Any taskprocessor overload will trigger.<br /> pjsip_only - Only pjsip taskprocessor overloads will trigger.<br /> none - No overload detection will be performed.")?></span>
+			</div>
+		</div>
+	</div>
+	<?php } ?>
+	<!--Taskprocessor Overload Trigger-->
+
 	<!--Show Advanced Settings-->
 	<div class="element-container">
 		<div class="row">
@@ -395,4 +504,3 @@ foreach ($protocols as $p) {
 </div>
 <?php echo $protohtml?>
 <?php echo $bindhtml?>
-
