@@ -113,6 +113,7 @@ if (!defined('FREEPBX_IS_AUTH')) { die('No direct script access allowed'); }
 	$error_displays = array_merge($error_displays,sipsettings_check_custom_files());
 
 ?>
+<div class="alert alert-info"><?php echo _('These settings apply only to SIP Legacy [chan_sip]')?></div>
 
 	<h2><?php echo _("Edit Settings"); ?></h2>
 
@@ -849,7 +850,7 @@ if (!defined('FREEPBX_IS_AUTH')) { die('No direct script access allowed'); }
 	<!--END Default Context-->
 	<?php
 		$tt = _("Asterisk: bindaddr. The IP address to bind to and listen for calls on the Bind Port. If set to 0.0.0.0 Asterisk will listen on all addresses. It is recommended to leave this blank.");
-		$tt .= ' ' . _("Note that chan_sip does not support IPv6 for UDP protocols. An address of '[::]' will listen on both IPv4 and IPv6, but is not recommended. If you want to use IPv6, it is recommended to use PJSip for those devices or trunks.");
+		$tt .= ' ' . _("Note that chan_sip does not support IPv6 for UDP protocols. An address of '::' will listen on both IPv4 and IPv6, but is not recommended. If you want to use IPv6, it is recommended to use PJSip for those devices or trunks.");
 	?>
 	<!--Bind Address-->
 	<div class="element-container">
@@ -904,13 +905,13 @@ if (!defined('FREEPBX_IS_AUTH')) { die('No direct script access allowed'); }
 					<i class="fa fa-question-circle fpbx-help-icon" data-for="tlsbindaddr"></i>
 				</div>
 				<div class="col-md-9">
-					<input type="text" class="form-control validate-ip" id="tlsbindaddr" name="tlsbindaddr" placeholder='[::]' value="<?php echo $tlsbindaddr ?>">
+					<input type="text" class="form-control validate-ip" id="tlsbindaddr" name="tlsbindaddr" placeholder='::' value="<?php echo $tlsbindaddr ?>">
 				</div>
 			</div>
 		</div>
 		<div class="row">
 			<div class="col-md-12">
-				<span id="tlsbindaddr-help" class="help-block fpbx-help-block"><?php echo _("TCP Port to listen on for TLS (Encrypted) connections. Defaults to '[::]', which listens on all interfaces for both IPv4 and IPv6 connections. Note that PJSip is preferable for all IPv6 communications."); ?></span>
+				<span id="tlsbindaddr-help" class="help-block fpbx-help-block"><?php echo _("TCP Port to listen on for TLS (Encrypted) connections. Defaults to '::', which listens on all interfaces for both IPv4 and IPv6 connections. Note that PJSip is preferable for all IPv6 communications."); ?></span>
 			</div>
 		</div>
 	</div>
@@ -1025,7 +1026,7 @@ if (!defined('FREEPBX_IS_AUTH')) { die('No direct script access allowed'); }
 			<div class="form-group form-inline" id="sc_$idx">
 				<input type="text" id="sip_custom_key_$idx" name="sip_custom_key_$idx" class="sip-custom" value="{$$var_sip_custom_key}" tabindex="$tabindex"> =
 				<input type="text" id="sip_custom_val_$idx" name="sip_custom_val_$idx" value="{$$var_sip_custom_val}" tabindex="$tabindex">
-				<a href="config.php?display=sipsettings&action=delete&key={$$var_sip_custom_key}&val={$$var_sip_custom_val}" onclick="return confirm( _('Are you sure you want to delete this item?') );"><i class="fa fa-trash" ></i></a>
+				<a href="config.php?display=sipsettings&action=delete&key={$$var_sip_custom_key}&val={$$var_sip_custom_val}" onclick="return confirm( _('Are you sure you want to delete this item?') );"><i class="fa fa-trash-o" ></i></a>
 			</div>
 END;
 		}
@@ -1048,7 +1049,7 @@ END;
 							<div class="form-group form-inline" id="sc_0">
 								<input type="text" id="sip_custom_key_0" name="sip_custom_key_0" class="sip-custom" value="<?php echo $sip_custom_key_0 ?>" > =
 								<input type="text" id="sip_custom_val_0" name="sip_custom_val_0" value="<?php echo $sip_custom_val_0 ?>" >
-								<?php if(!empty($sip_custom_val_0) && !empty($sip_custom_key_0)){ ?><a href="config.php?display=sipsettings&action=delete&key=<?php echo $sip_custom_key_0 ?>&val=<?php echo $sip_custom_val_0 ?>" onclick="return confirm(_('Are you sure you want to delete this item?'));"><i class="fa fa-trash" ></i></a><?php }?>
+								<?php if(!empty($sip_custom_val_0) && !empty($sip_custom_key_0)){ ?><a href="config.php?display=sipsettings&action=delete&key=<?php echo $sip_custom_key_0 ?>&val=<?php echo $sip_custom_val_0 ?>" onclick="return confirm(_('Are you sure you want to delete this item?'));"><i class="fa fa-trash-o" ></i></a><?php }?>
 							</div>
 							<?php echo $csotherinputs?>
 							<div id="sip-custom-buttons">
