@@ -57,7 +57,7 @@ if (isset($extens) && is_array($extens)) {
 				$vmail_info["activated_info"][$exten[0]] = $vmbox["context"];
 			} else {
 				$vmail_info["unactivated_info"][$exten[0]] = $vmbox["context"];
-				$unactivated_style = " style='background: #abc9ff;'";
+				$unactivated_style = " style='background: #4FB580;'";
 				$unactivated_txt = " [unactivated]";
 			}
 			$link = "config.php?type=" . $type . "&display=" . $display . "&ext=" . $exten[0] . "&action=bsettings#" . $exten[0];
@@ -307,7 +307,7 @@ $tooltips = array("tz" 	    => array("name" 				=> _("Timezone definition name")
 				     "maxmessage" 			=> _("Max message time length."),
 				     "maxsecs" 				=> _("Max message time length."),
 				     "name"				=> _("Name of account/user"),
-				     "pager"				=> _("Pager/mobile email address that short Voicemail notifications are sent to."),
+				     "pager"				=> _("Pager/mobile email address that short Voicemail notifications are sent to. Separated by |"),
 				     "review"				=> _("Allow sender to review/rerecord their message before saving it [OFF by default]"),
 				     "saycid" 				=> _("Read back caller's telephone number prior to playing the incoming message, and just after announcing the date and time the message was left. If not described, or set to no, it will be in the envelope."),
 				     "sayduration" 			=> _("Turn on/off saying duration information before the message playback. [ON by default]"),
@@ -398,7 +398,8 @@ show_view(dirname(__FILE__).'/views/header.php',array(
 //TODO: Do we really need to say "UPDATE COMPLETED??"
 if ($need_update && $action != 'usage') {
 	$args = array();
-	if (voicemail_update_settings($action, $context, $extension, $_REQUEST)) {
+	$request = freepbxGetSanitizedRequest(FILTER_UNSAFE_RAW);
+	if (voicemail_update_settings($action, $context, $extension, $request)) {
 		$url = "config.php?type=$type&display=$display&action=$action&ext=$extension&updated=true";
 				needreload();
 				redirect($url);
@@ -680,7 +681,8 @@ switch ($action) {
 		*/
 		$scope = voicemail_get_scope($extension);
 		if ($need_update) {
-			voicemail_update_usage($vmail_info, $context, $extension, $_REQUEST);
+			$request = freepbxGetSanitizedRequest(FILTER_UNSAFE_RAW);
+			voicemail_update_usage($vmail_info, $context, $extension, $request);
 			if (!empty($extension)) {
 				$url = "config.php?display=$display&ext=$extension&action=$action&updated=true";
 			} else {
