@@ -37,12 +37,32 @@ var VoicemailC = UCPMC.extend({
 
 		var notify = false;
 		var self = this;
-		async.forEachOf(data.boxes, function (value, extension, callback) {
+
+		/**
+		 * Check all extensions and boxes at once.
+		 */
+		$.ajax({
+			type: "POST",
+			url: UCP.ajaxUrl + "?module=voicemail&command=checkextensions",
+			async: false,
+			data: data.boxes,
+			success: function(vm_data){				
+				window.vm_data = vm_data;
+			},
+			error: function (xhr, ajaxOptions, thrownError) {
+                console.error('Unable to check extensions', thrownError, xhr);
+            },
+		  });
+
+		async.forEachOf(window.vm_data, function (value, extension, callback) {	
 			var el = $(".grid-stack-item[data-rawname='voicemail'][data-widget_type_id='"+extension+"'] .mailbox");
 			self.refreshFolderCount(extension);
-			if(el.length && el.data("inbox") != value || window.update_table == true) {
-				el.data("inbox",value);
-				notify = true;				
+			if(el.length && el.data("inbox").status != value.status || window.update_table == true) {
+				notify = false;
+				if(el.data("inbox") < value){
+					notify = true;
+				}
+				el.data("inbox",value);	
 				if((typeof Cookies.get('vm-refresh-'+extension) === "undefined" && (typeof Cookies.get('vm-refresh-'+extension) === "undefined" || Cookies.get('vm-refresh-'+extension) == 1)) || Cookies.get('vm-refresh-'+extension) == 1) {
 					$(".grid-stack-item[data-rawname='voicemail'][data-widget_type_id='"+extension+"'] .voicemail-grid").bootstrapTable('refresh',{silent: true});
 				}
@@ -120,7 +140,7 @@ var VoicemailC = UCPMC.extend({
 			$("div[data-id='"+widget_id+"'] .voicemail-grid a.listen").click(function() {
 				var id = $(this).data("id"),
 						select = '';
-				$.each(self.staticsettings.mailboxes, function(i,v) {
+				$.each(self.staticsettings.extensions, function(i,v) {
 					select = select + "<option value='"+v+"'>"+v+"</option>";
 				});
 				UCP.showDialog(_("Listen to Voicemail"),
@@ -223,26 +243,36 @@ var VoicemailC = UCPMC.extend({
 			});
 			UCP.showDialog(_("Move Voicemail"),
 				_("To")+':</label><select class="form-control" data-toggle="select" id="VMmove">'+opts+"</select>",
-				'<button class="btn btn-default" id="moveVM">' + _("Move") + "</button>",
+				'<button class="btn btn-default" id="moveVM"><span id="spin"></span>&nbsp;&nbsp;' + _("Move") + "</button>",
 				function() {
 					var total = sel.length;
 					$("#moveVM").click(function() {
 						$("#moveVM").prop("disabled",true);
-						async.forEachOf(sel, function (v, i, callback) {
-							self.moveVoicemail(v.msg_id, $("#VMmove").val(), extension, function(data) {
-								if(data.status) {
-									$("div[data-id='"+widget_id+"'] .voicemail-grid").bootstrapTable('remove', {field: "msg_id", values: [String(v.msg_id)]});
-								}
-								callback();
+						$("#spin").html('<i class="fa fa-spinner fa-spin"></i>')
+						setTimeout(function () {
+							let data = [];
+							Object.keys(sel).forEach(key => {
+								data.push({
+									msg: sel[key].msg_id,
+									folder: $("#VMmove").val(),
+									ext: extension
+								});
 							})
-						}, function(err) {
-							if( err ) {
-								$("#moveVM").prop("disabled",false);
-								UCP.showAlert(err);
-							} else {
-								UCP.closeDialog();
-							}
-						});
+							self.moveVoicemailBulk(data, extension, function (data) {
+								if (data.status) {
+									self.rebuildVM(extension);
+									if (data.moveStatus.includes(false)) {
+										UCP.showAlert('Not able to move some of the voicemails.');
+									}
+									setTimeout(function () {
+										UCP.closeDialog();
+									}, 2000);
+								} else {
+									$("#moveVM").prop("disabled", false);
+									UCP.showAlert(data.error);
+								}
+							})
+						}, 50);
 					});
 					$("#VMmove").keypress(function(event) {
 						if (event.keyCode == 13) {
@@ -260,30 +290,51 @@ var VoicemailC = UCPMC.extend({
 									UCP.showAlert(err);
 								} else {
 									UCP.closeDialog();
+									self.rebuildVM(extension);
 								}
 							});
 						}
 					});
+					$(".delete-selection").prop("disabled",true);
+					$(".forward-selection").prop("disabled",true);
+					$(".move-selection").prop("disabled",true);
 				}
 			);
 		});
-		$("div[data-id='"+widget_id+"'] .delete-selection").click(function() {
+		$("div[data-id='" + widget_id + "'] .delete-selection").click(function () {
+			$('#modal_confirm_button').attr("data-dismiss", '');
 			UCP.showConfirm(_("Are you sure you wish to delete these voicemails?"),'warning',function() {
 				var extension = $("div[data-id='"+widget_id+"']").data("widget_type_id");
 				var sel = $("div[data-id='"+widget_id+"'] .voicemail-grid").bootstrapTable('getAllSelections');
-				async.forEachOf(sel, function(v, i, callback){
-						self.deleteVoicemail(v.msg_id, extension, function(data) {
-							if(data.status) {
-								$("div[data-id='"+widget_id+"'] .voicemail-grid").bootstrapTable('remove', {field: "msg_id", values: [String(v.msg_id)]});
+				var accept = $("#modal_confirm_button").text();
+				$("#modal_confirm_button").html('<i class="fa fa-spinner fa-spin"></i>&nbsp;'+ accept);
+				setTimeout(function () {
+					let data = [];
+					Object.keys(sel).forEach(key => {
+						data.push({
+							msg: sel[key].msg_id,
+							folder: $("#VMmove").val(),
+							ext: extension
+						});
+					});
+					self.deleteVoicemailBulk(data, extension, function (data) {
+						if (data.status) {
+							self.rebuildVM(extension);
+							if (data.deleteStatus.includes(false)) {
+								UCP.showAlert('Not able to delete some of the voicemails.');
 							}
-							callback();
-						})
-				}, function(err) {
-					if( err ) {
-						UCP.showAlert(err);
-					}
-				});
-				$("#delete-selection").prop("disabled",true);
+							setTimeout(function () {
+								$("#modal_confirm_button").html(accept);
+								$("#confirm_modal").modal('toggle');
+							}, 2000);
+						} else {
+							UCP.showAlert(data.error);
+						}
+					});
+				}, 50);
+				$(".delete-selection").prop("disabled",true);
+				$(".forward-selection").prop("disabled",true);
+				$(".move-selection").prop("disabled",true);
 			});
 		});
 		$("div[data-id='"+widget_id+"'] .forward-selection").click(function() {
@@ -467,20 +518,38 @@ var VoicemailC = UCPMC.extend({
 		});
 	},
 	refreshFolderCount: function(extension) {
+		var data = window.vm_data[extension];
+		if(data.status) {
+			window.update_table = false;
+			$.each(data.folders, function(i,v) {		
+				cur_val = $(".grid-stack-item[data-rawname='voicemail'][data-widget_type_id="+extension+"] .mailbox .folder-list .folder[data-name='"+v.name+"'] .badge").text();
+				if(cur_val != v.count){
+					window.update_table = true;
+				}
+				$(".grid-stack-item[data-rawname='voicemail'][data-widget_type_id="+extension+"] .mailbox .folder-list .folder[data-name='"+v.name+"'] .badge").text(v.count);				
+			});
+		}
+
+	},
+	rebuildVM: function(extension){
 		var data = {
 			ext: extension
-		};
-		$.post( UCP.ajaxUrl + "?module=voicemail&command=refreshfoldercount", data, function( data ) {
-			if(data.status) {
-				window.update_table = false;
-				$.each(data.folders, function(i,v) {
-					cur_val = $(".grid-stack-item[data-rawname='voicemail'][data-widget_type_id="+extension+"] .mailbox .folder-list .folder[data-name='"+v.name+"'] .badge").text();
-					if(cur_val != v.count){
-						window.update_table = true;
-					}
-					$(".grid-stack-item[data-rawname='voicemail'][data-widget_type_id="+extension+"] .mailbox .folder-list .folder[data-name='"+v.name+"'] .badge").text(v.count);
-					
-				});
+		},
+		self = this;
+		$.ajax({
+			type: "POST",
+			url: UCP.ajaxUrl + "?module=voicemail&command=rebuildVM",
+			data: data,
+			success: function(data) {
+				self.refreshFolderCount(extension);
+				if(typeof callback === "function") {
+					callback(data);
+				}	
+			},
+			error: function(data) {
+				if(typeof callback === "function") {
+					callback({status: false});
+				}
 			}
 		});
 	},
@@ -491,14 +560,46 @@ var VoicemailC = UCPMC.extend({
 			ext: extension
 		},
 		self = this;
-		$.post( UCP.ajaxUrl + "?module=voicemail&command=moveToFolder", data, function(data) {
-			self.refreshFolderCount(extension);
-			if(typeof callback === "function") {
-				callback(data);
+		$.ajax({
+			type: "POST",
+			url: UCP.ajaxUrl + "?module=voicemail&command=moveToFolder",
+			data: data,
+			async: false,
+			success: function(data) {
+				self.refreshFolderCount(extension);
+				if(typeof callback === "function") {
+					callback(data);
+				}	
+			},
+			error: function(data) {
+				if(typeof callback === "function") {
+					callback({status: false});
+				}
 			}
-		}).fail(function() {
-			if(typeof callback === "function") {
-				callback({status: false});
+		});
+	},
+	moveVoicemailBulk: function (data, extension, callback) {
+		var formData = new FormData();
+		formData.append('data', JSON.stringify(data));
+		self = this;
+		$.ajax({
+			type: "POST",
+			enctype: 'multipart/form-data',
+			url: UCP.ajaxUrl + "?module=voicemail&command=moveToFolderBulk",
+			data: formData,
+			async: false,
+			processData: false,
+			contentType: false,
+			success: function (data) {
+				self.refreshFolderCount(extension);
+				if (typeof callback === "function") {
+					callback(data);
+				}
+			},
+			error: function (data) {
+				if (typeof callback === "function") {
+					callback({ status: false, error: data });
+				}
 			}
 		});
 	},
@@ -524,15 +625,46 @@ var VoicemailC = UCPMC.extend({
 			ext: extension
 		},
 		self = this;
-
-		$.post( UCP.ajaxUrl + "?module=voicemail&command=delete", data, function( data ) {
-			self.refreshFolderCount(extension);
-			if(typeof callback === "function") {
-				callback(data);
+		$.ajax({
+			type: "POST",
+			url: UCP.ajaxUrl + "?module=voicemail&command=delete",
+			data: data,
+			async: false,
+			success: function(data) {
+				self.refreshFolderCount(extension);
+				if(typeof callback === "function") {
+					callback(data);
+				}	
+			},
+			error: function(data) {
+				if(typeof callback === "function") {
+					callback({status: false});
+				}
 			}
-		}).fail(function() {
-			if(typeof callback === "function") {
-				callback({status: false});
+		});
+	},
+	deleteVoicemailBulk: function (data, extension, callback) {
+		var formData = new FormData();
+		formData.append('data', JSON.stringify(data));
+		self = this;
+		$.ajax({
+			type: "POST",
+			enctype: 'multipart/form-data',
+			url: UCP.ajaxUrl + "?module=voicemail&command=deleteBulk",
+			data: formData,
+			async: false,
+			processData: false,
+			contentType: false,
+			success: function (data) {
+				self.refreshFolderCount(extension);
+				if (typeof callback === "function") {
+					callback(data);
+				}
+			},
+			error: function (data) {
+				if (typeof callback === "function") {
+					callback({ status: false });
+				}
 			}
 		});
 	},
