@@ -63,7 +63,16 @@ if($online) {
 					<span class="clear">&nbsp;</span>
 				</div>
 				<table class="table modulelist table-striped" width="100%">
-					<?php foreach($category['data'] as $module) {?>
+					<?php foreach($category['data'] as $module) {
+						if($breaking['breaking'] && isset($breaking['issues'][$module['name']])){
+							if($module['status'] != MODULE_STATUS_NEEDUPGRADE){
+								$module['status'] = MODULE_STATUS_CONFLICT;
+							}
+							if($module['status'] == MODULE_STATUS_NEEDUPGRADE){
+								$module['status'] = MODULE_STATUS_CONFLICT_UPGRADE;
+							}
+						}
+					?>
 					</tr>
 						<td id="fullmodule_<?php echo prep_id($module['name'])?>" class="fullmodule" data-module="<?php echo prep_id($module['name'])?>">
 							<div class="<?php echo $module['mclass']?>" data-module="<?php echo prep_id($module['name'])?>">
@@ -89,6 +98,10 @@ if($online) {
 										case MODULE_STATUS_BROKEN:?>
 											<span class="alert text"><?php echo _('Broken');?></span>
 										<?php break;
+										case MODULE_STATUS_CONFLICT:?>
+											<span class="alert text"><?php echo _('This module has conflicts that may cause system issues');?></span>
+										<?php break;
+
 										case MODULE_STATUS_DISABLED:
 										default:
 											$disabled = ($module['status'] == MODULE_STATUS_DISABLED) ? _('Disabled') . '; ' : '';
@@ -147,7 +160,7 @@ if($online) {
 							<?php if ($module['salert']) { ?>
 								<span class="modulevul">
 									<a class="modulevul_tag" href="#" data-sec='<?php echo json_encode($module['vulnerabilities']['vul'])?>'>
-										<img src="images/notify_security.png" alt="" width="16" height="16" border="0" title="<?php echo sprintf(_("Vulnerable to security issues %s"), implode($module['vulnerabilities']['vul'], ', '))?>" /><?php echo sprintf(_("Vulnerable, Requires: %s"), $module['vulnerabilities']['minver']) ?>
+										<img src="images/notify_security.png" alt="" width="16" height="16" border="0" title="<?php echo sprintf(_("Vulnerable to security issues %s"), implode( ', ',$module['vulnerabilities']['vul']))?>" /><?php echo sprintf(_("Vulnerable, Requires: %s"), $module['vulnerabilities']['minver']) ?>
 									</a>
 								</span>
 							<?php } ?>
@@ -167,6 +180,9 @@ if($online) {
 											<?php if(!empty($module['previous'])) {?>
 												<li role="presentation"><a href="#previous_<?php echo prep_id($module['name'])?>" data-toggle="tab"><?php echo _("Previous")?></a></li>
 											<?php } ?>
+											<?php if($breaking['breaking'] && !empty($breaking['issues'][$module['name']])) {?>
+												<li role="presentation"><a href="#breaking_<?php echo prep_id($module['name'])?>" data-toggle="tab"><?php echo _("Conflicts")?></a></li>
+											<?php } ?>
 											<?php if ($devel) { ?>
 												<li role="presentation"><a href="#devel_<?php echo prep_id($module['name'])?>" data-toggle="tab"><?php echo _("Debug")?></a></li>
 											<?php } ?>
@@ -187,6 +203,12 @@ if($online) {
 														<td><?php echo $module['publisher']?></td>
 													</tr>
 													<?php } ?>
+                                                    <?php if(!empty($module['repo'])) {?>
+													<tr>
+														<td><?php echo _("Repository")?>:</td>
+														<td><?php echo $module['repo']?></td>
+													</tr>
+													<?php } ?>	
 													<?php if(!empty($module['raw']['online']['packaged'])) {?>
 													<tr>
 														<td><?php echo _("Packaged (Released)")?>:</td>
@@ -202,13 +224,13 @@ if($online) {
 													<?php if(!empty($module['signature']['message'])) {?>
 														<tr>
 															<td><?php echo ("Signature Status")?>:</td>
-															<td><?php echo $module['signature']['message']?> <a class="alert-link" href="http://wiki.freepbx.org/display/F2/Module+Signing" target="_blank">(What Does this Mean?)</a></td>
+															<td><?php echo $module['signature']['message']?> <a class="alert-link" href="http://wiki.freepbx.org/display/FPG/Module+Signing" target="_blank"><?php echo _('(What Does this Mean?)') ?></a></td>
 														</tr>
 														<?php } ?>
 													<?php if(!empty($module['salert'])) {?>
 													<tr>
 														<td><?php echo _("Fixes Vulnerabilities")?>:</td>
-														<td><?php echo implode($module['vulnerabilities']['vul'], ', ')?></td>
+														<td><?php echo implode(', ',$module['vulnerabilities']['vul'])?></td>
 													</tr>
 													<?php } ?>
 													<tr>
@@ -224,15 +246,22 @@ if($online) {
 														<td>
 															<?php if(!empty($module['info'])) {?>
 																<a href="<?php echo $module['info'] ?>" target="_new"><?php echo $module['info'] ?></a>
-															<?php } else { ?>
-																<a href="<?php echo $freepbx_help_url?>&amp;freepbx_module=<?php echo urlencode($module['name'])?>" target="help"><?php echo sprintf(_("Get help for %s"),$module['pretty_name'])?></a>
-															<?php } ?>
+															<?php } else { 
+                                                                if (!empty($module['more-info'])){
+                                                                    ?> 
+                                                                    <a href="<?php echo $module['more-info']?>" target="help"><?php echo sprintf(_("Get help for %s"),$module['pretty_name'])?></a> <?php
+                                                                } else {
+                                                                    ?>
+                                                                    <a href="<?php echo $freepbx_help_url?>&amp;freepbx_module=<?php echo urlencode($module['name'])?>" target="help"><?php echo sprintf(_("Get help for %s"),$module['pretty_name'])?></a> <?php
+                                                                }
+                                                            }
+                                                            ?>
 														</td>
 													</tr>
 													<?php if($module['commercial']['status']) {?>
 														<?php if($module['commercial']['sysadmin'] || $module['name'] == 'sysadmin' && $module['status'] == MODULE_STATUS_ENABLED) {?>
 															<tr>
-																<td><a href="#" class="info"><?php echo _('Commercial Status')?>:<span><?php echo _('Commercial Status of this module. Commercial Modules are maintained and supported through Schmoozecom, INC')?></span></a></td>
+																<td><a href="#" class="info"><?php echo _('Commercial Status')?>:<span><?php echo _('Commercial Status of this module. Commercial Modules are maintained and supported through  Sangoma Technologies Inc.')?></span></a></td>
 																<td>
 																	<?php if(!$module['commercial']['licensed'] && isset($module['commercial']['type'])) { ?>
 																		<?php switch($module['commercial']['type']) {
@@ -245,7 +274,14 @@ if($online) {
 																			<?php case 'paid':?>
 																			<?php default:?>
 																				<a href="<?php echo $module['commercial']['purchaselink']?>" class="btn btn-primary" target="_new"><?php echo _('Learn More')?></a>
-																				<a href="<?php echo $module['commercial']['purchaselink']?>" class="btn fpbx-buy" data-rawname="<?php echo $module['name']?>" target="_new"><?php echo _('Buy')?></a>
+
+<?php if (\FreePBX::Modules()->checkStatus('sysadmin') && function_exists('sysadmin_get_buyurl')) {
+        $buyurl = sysadmin_get_buyurl($module['commercial']['purchaselink']);
+} else {
+        $buyurl = $module['commercial']['purchaselink'];
+}
+?>
+																				<a href="<?php echo $buyurl; ?>" class="btn fpbx-buy" data-rawname="<?php echo $module['name']?>" target="_new"><?php echo _('Buy')?></a>
 																			<?php break;?>
 																		<?php } ?>
 																	<?php } else { ?>
@@ -280,6 +316,7 @@ if($online) {
 																	continue;
 																}
 															?>
+																<input id="id="version_<?php echo prep_id($module['name'])?>"" type="hidden" name="version[<?php echo prep_id($module['name'])?>]" value="<?php echo $module['version']?>"/>
 																<input id="track_<?php echo $track?>_<?php echo prep_id($module['name'])?>" type="radio" name="trackaction[<?php echo prep_id($module['name'])?>]" value="<?php echo $track?>" <?php echo ($checked) ? 'checked' : ''?>/>
 																<label for="track_<?php echo $track?>_<?php echo prep_id($module['name'])?>"><?php echo ucfirst($track)?></label>
 															<?php } ?>
@@ -290,6 +327,7 @@ if($online) {
 													<tr>
 														<td colspan="2">
 															<input id="track_stable_<?php echo prep_id($module['name'])?>" type="hidden" name="trackaction[<?php echo prep_id($module['name'])?>]" value="stable"/>
+															<input id="track_version_<?php echo prep_id($module['name'])?>" type="hidden" name="version[<?php echo prep_id($module['name'])?>]" value="<?php echo $module['version']?>"/>
 														</td>
 													</tr>
 												<?php } ?>
@@ -362,6 +400,24 @@ if($online) {
 																				<label for="uninstall_<?php echo prep_id($module['name'])?>"><?php echo _('Uninstall')?></label>
 																			<?php }
 																		break;
+																		case MODULE_STATUS_CONFLICT:
+																			if (!EXTERNAL_PACKAGE_MANAGEMENT) {
+																				if (!$module['blocked']['status']) { ?>
+																					<input type="radio" id="forceinstall_<?php echo prep_id($module['name'])?>" name="moduleaction[<?php echo prep_id($module['name'])?>]" value="installignoreconflicts" />
+																					<label for="forceinstall_<?php echo prep_id($module['name'])?>"><?php echo _('Force Install')?></label>
+																				<?php } ?>
+																			<?php }
+																		break;
+																		case MODULE_STATUS_CONFLICT_UPGRADE:
+																			if (!EXTERNAL_PACKAGE_MANAGEMENT) {
+																				if (!$module['blocked']['status']) { ?>
+																					<input type="radio" id="forceupgrade_<?php echo prep_id($module['name'])?>" name="moduleaction[<?php echo prep_id($module['name'])?>]" value="upgradeignoreconflicts" />
+																					<label for="forceupgrade_<?php echo prep_id($module['name'])?>"><?php echo _('Force Update')?></label>
+																					<input type="radio" id="uninstall_<?php echo prep_id($module['name']) ?>" name="moduleaction[<?php echo prep_id($module['name']) ?>]" value="uninstall" />
+																					<label for="uninstall_<?php echo prep_id($module['name']) ?>"><?php echo _('Uninstall') ?></label>
+																				<?php } ?>
+																			<?php }
+																		break;
 																		default:
 																			// check for online upgrade
 																			$track = $module['track'];
@@ -433,6 +489,13 @@ if($online) {
 													</table>
 												</div>
 											<?php } ?>
+											<?php if ($breaking['breaking'] && !empty($breaking['issues'][$module['name']])) { ?>
+												<div id="breaking_<?php echo prep_id($module['name']) ?>" class="limitheight tab-pane">
+													<h5><?php echo _("Conflicts") ?>: <?php echo $module['version'] ?></h5>
+													<span><?php echo implode(PHP_EOL, $breaking['issues'][$module['name']]) ?></span>
+												</div>
+											<?php
+												} ?>
 											<?php if ($devel) { ?>
 												<div id="devel_<?php echo prep_id($module['name'])?> limitheight" class="tab-pane">
 													<h5><?php echo $module['name']?></h5>
