@@ -9,6 +9,8 @@ use Symfony\Component\Console\Output\OutputInterface;
 use Symfony\Component\Console\Question\ChoiceQuestion;
 use Symfony\Component\Console\Question\Question;
 use Symfony\Component\Console\Helper\ProgressBar;
+use Symfony\Component\Process\Process;
+use Symfony\Component\Process\Exception\ProcessFailedException;
 
 class FreePBXInstallCommand extends Command {
 	private $rootPath = null;
@@ -21,6 +23,14 @@ class FreePBXInstallCommand extends Command {
 			'default' => 'asterisk',
 	 		'description' => 'Database name'
 		),
+		'dbhost' => array(
+			'default' => 'localhost',
+			'description' => 'Database server address'
+		),
+		'dbport' => array(
+			'default' => '3306',
+			'description' => 'Database server port'
+		),
 		'cdrdbname' => array(
 			'default' => 'asteriskcdrdb',
 			'description' => 'CDR Database name'
@@ -96,6 +106,9 @@ class FreePBXInstallCommand extends Command {
 			'description' => 'Directory for FreePBX html5 playback files'
 		),
 	);
+	private $isTtySupported;
+	private $output;
+	private $input;
 
 	protected function configure() {
 		$this
@@ -134,13 +147,8 @@ class FreePBXInstallCommand extends Command {
 
 	protected function execute(InputInterface $input, OutputInterface $output) {
 		global $amp_conf; /* This makes pandas sad. :( */
-
-		//still needed for module_admin and retrieve_conf
-		$filePath = stream_resolve_include_path('Console/Getopt.php');
-		if ($filePath === false) {
-			$output->writeln("<error>PEAR must be installed (requires Console/Getopt.php)</error>");
-			return false;
-		}
+		$this->output = $output;
+		$this->input = $input;
 
 		$this->rootPath = dirname(__DIR__);
 		date_default_timezone_set('America/Los_Angeles');
@@ -235,7 +243,7 @@ class FreePBXInstallCommand extends Command {
 			if(isset($aconf['directories']['astmoddir'])) {
 				$aconf['directories']['astmoddir'] = !empty($answers['astmoddir']) ? $answers['astmoddir'] : (file_exists('/usr/lib64/asterisk/modules') ? '/usr/lib64/asterisk/modules' : '/usr/lib/asterisk/modules');
 			}
-			if(isset($$aconf['directories']['astvarlibdir'])) {
+			if(isset($aconf['directories']['astvarlibdir'])) {
 				$aconf['directories']['astvarlibdir'] = !empty($answers['astvarlibdir']) ? $answers['astvarlibdir'] : "/var/lib/asterisk";
 			}
 			if(isset($aconf['directories']['astagidir'])) {
@@ -286,10 +294,10 @@ class FreePBXInstallCommand extends Command {
 			// Parse Asterisk version.
 			if (preg_match('/^Asterisk (?:SVN-|GIT-)?(?:branch-)?(\d+(\.\d+)*)(-?(.*)) built/', $astver, $matches)) {
 				$determined = true;
-				if (version_compare($matches[1], "11", "lt") || version_compare($matches[1], "18", "ge")) {
+				if (version_compare($matches[1], "13", "lt") || version_compare($matches[1], "19", "ge")) {
 					$output->writeln("<error>Error!</error>");
 					$output->writeln("<error>Unsupported Version of ". $matches[1]."</error>");
-					$output->writeln("<error>Supported Asterisk versions: 11, 12, 13, 14, 15, 16, 17</error>");
+					$output->writeln("<error>Supported Asterisk versions: 13, 14, 15, 16, 17, 18</error>");
 					exit(1);
 				}
 				$output->writeln("Yes. Determined Asterisk version to be: ".$matches[1]);
@@ -309,6 +317,18 @@ class FreePBXInstallCommand extends Command {
 			exit(1);
 		}
 
+		$output->write("Checking if NodeJS is installed and we can get a version from it...");
+		$nodejsout = exec("node --version"); //v0.10.29
+		$nodejsout = str_replace("v","",trim($nodejsout));
+		if(empty($nodejsout)) {
+			$output->writeln("<error>NodeJS 8 or higher is not installed. This is now a requirement</error>");
+			return false;
+		}
+		if(version_compare($nodejsout,'8.0.0',"<")) {
+			$output->writeln(sprintf("NodeJS version is: %s requirement is %s or higher",$nodejsout,'8.0.0'));
+			return false;
+		}
+
 		if((file_exists($freepbx_conf_path) && !file_exists(AMP_CONF)) || (!file_exists($freepbx_conf_path) && file_exists(AMP_CONF))) {
 			if(file_exists($freepbx_conf_path)) {
 				$output->writeln("<error>Error!</error>");
@@ -319,6 +339,7 @@ class FreePBXInstallCommand extends Command {
 			}
 			exit(1);
 		}
+		$output->writeln("Yes. Determined NodeJS version to be: ".$nodejsout);
 
 		$output->writeln("Preliminary checks done. Starting FreePBX Installation");
 
@@ -415,7 +436,8 @@ class FreePBXInstallCommand extends Command {
 
 			$amp_conf['AMPDBUSER'] = $answers['dbuser'];
 			$amp_conf['AMPDBPASS'] = $answers['dbpass'];
-			$amp_conf['AMPDBHOST'] = 'localhost';
+			$amp_conf['AMPDBHOST'] = $answers['dbhost'];
+			$amp_conf['AMPDBPORT'] = $answers['dbport'];
 
 			if($dbroot) {
 				$output->write("Database Root installation checking credentials and permissions..");
@@ -503,7 +525,7 @@ class FreePBXInstallCommand extends Command {
 					$pdodb->query("DROP DATABASE IF EXISTS ".$amp_conf['AMPDBNAME']);
 				}
 				$pdodb->query("CREATE DATABASE IF NOT EXISTS ".$amp_conf['AMPDBNAME']." DEFAULT CHARACTER SET ".$dbencoding." DEFAULT COLLATE ".$dbencoding."_unicode_ci");
-				$sql = "GRANT ALL PRIVILEGES ON ".$amp_conf['AMPDBNAME'].".* TO '" . $amp_conf['AMPDBUSER'] . "'@'localhost' IDENTIFIED BY '" . $amp_conf['AMPDBPASS'] . "'";
+				$sql = "GRANT ALL PRIVILEGES ON ".$amp_conf['AMPDBNAME'].".* TO '" . $amp_conf['AMPDBUSER'] . "'@'".$amp_conf['AMPDBHOST']."' IDENTIFIED BY '" . $amp_conf['AMPDBPASS'] . "'";
 				$pdodb->query($sql);
 			} else {
 				//check collate
@@ -543,7 +565,7 @@ class FreePBXInstallCommand extends Command {
 					$db->query("DROP DATABASE IF EXISTS ".$amp_conf['CDRDBNAME']);
 				}
 				$db->query("CREATE DATABASE IF NOT EXISTS ".$amp_conf['CDRDBNAME']." DEFAULT CHARACTER SET ".$dbencoding." DEFAULT COLLATE ".$dbencoding."_unicode_ci");
-				$sql = "GRANT ALL PRIVILEGES ON ".$amp_conf['CDRDBNAME'].".* TO '" . $amp_conf['AMPDBUSER'] . "'@'localhost' IDENTIFIED BY '" . $amp_conf['AMPDBPASS'] . "'";
+				$sql = "GRANT ALL PRIVILEGES ON ".$amp_conf['CDRDBNAME'].".* TO '" . $amp_conf['AMPDBUSER'] . "'@'".$amp_conf['AMPDBHOST']."' IDENTIFIED BY '" . $amp_conf['AMPDBPASS'] . "'";
 				$db->query($sql);
 			} else {
 				//check collate
@@ -699,6 +721,7 @@ class FreePBXInstallCommand extends Command {
 			"module.sig",
 			"install.php",
 			"LICENSE",
+			"phpunit.xml",
 			"README.md"
 		);
 		foreach($copyFrameworkFiles as $file) {
@@ -789,6 +812,7 @@ class FreePBXInstallCommand extends Command {
 \$amp_conf['AMPDBUSER'] = '{$amp_conf['AMPDBUSER']}';
 \$amp_conf['AMPDBPASS'] = '{$amp_conf['AMPDBPASS']}';
 \$amp_conf['AMPDBHOST'] = '{$amp_conf['AMPDBHOST']}';
+\$amp_conf['AMPDBPORT'] = '{$amp_conf['AMPDBPORT']}';
 \$amp_conf['AMPDBNAME'] = '{$amp_conf['AMPDBNAME']}';
 \$amp_conf['AMPDBENGINE'] = '{$amp_conf['AMPDBENGINE']}';
 \$amp_conf['datasource'] = ''; //for sqlite3
@@ -815,11 +839,13 @@ require_once('{$amp_conf['AMPWEBROOT']}/admin/bootstrap.php');
 		}
 
 		//run this here so that we make sure everything is square for module installs
-		system($amp_conf['AMPSBIN'] . "/fwconsole chown > /dev/tty");
+		$output->writeln("Chowning directories...");
+		system($amp_conf['AMPSBIN']."/fwconsole chown");
+		$output->writeln("Done");
 
 		// module_admin install framework
 		$output->writeln("Installing framework...");
-		system($amp_conf['AMPSBIN']."/fwconsole ma install framework");
+		$this->executeSystemCommand($amp_conf['AMPSBIN']."/fwconsole ma install framework");
 		$output->writeln("Done");
 
 		if(method_exists(\FreePBX::create()->View,'getScripts')) {
@@ -845,26 +871,27 @@ require_once('{$amp_conf['AMPWEBROOT']}/admin/bootstrap.php');
 
 		if($answers['dev-links']) {
 			$output->writeln("Enabling Developer mode");
-			system($amp_conf['AMPSBIN'] . "/fwconsole setting DEVEL 1 > /dev/null");
+			$this->executeSystemCommand($amp_conf['AMPSBIN'] . "/fwconsole setting DEVEL 1 > /dev/null");
 		}
 
 		/* read modules list from MODULE_DIR */
 		if(file_exists(MODULE_DIR) && $newinstall) {
 			$output->write("Installing base modules...");
-			system($amp_conf['AMPSBIN']."/fwconsole ma install core dashboard sipsettings voicemail");
+			$this->executeSystemCommand($amp_conf['AMPSBIN']."/fwconsole ma install core dashboard sipsettings voicemail certman");
+			$output->writeln("Done installing base modules");
 			if(!$answers['skip-install']) {
 				$output->write("Installing all modules...");
 				system($amp_conf['AMPSBIN']."/fwconsole ma installlocal");
+				$output->writeln("Done installing all modules");
 			}
-			$output->writeln("Done installing modules");
+
 		}
 
 		//run this here so that we make sure everything is square for asterisk
-		//Note this is redirected to /dev/tty so the progress bar will work properly.
-		system($amp_conf['AMPSBIN'] . "/fwconsole chown > /dev/tty");
+		$this->executeSystemCommand($amp_conf['AMPSBIN'] . "/fwconsole chown");
 
 		if($answers['dev-links'] && $newinstall) {
-			system($amp_conf['AMPSBIN']."/fwconsole setting AMPMGRPASS ".$amp_conf['AMPMGRPASS']);
+			$this->executeSystemCommand($amp_conf['AMPSBIN']."/fwconsole setting AMPMGRPASS ".$amp_conf['AMPMGRPASS']." > /dev/null");
 		}
 
 		// generate_configs();
@@ -992,4 +1019,39 @@ require_once('{$amp_conf['AMPWEBROOT']}/admin/bootstrap.php');
 			}
 		}
 	}
+
+	/**
+	 * Execute system command, check for tty first.
+	 *
+	 * @param string $command
+	 * @return void
+	 */
+	private function executeSystemCommand($command,$timeout =180) {
+		$process = new Process($command);
+		$process->setTimeout($timeout);
+		if($this->isTtySupported()) {
+			$process->setTty(true);
+			$process->mustRun();
+			if(!posix_isatty(STDIN)) {
+				$this->output->write($process->getOutput());
+			} else {
+				$process->mustRun();
+			}
+		} else {
+			$process->mustRun();
+		}
+	}
+
+	/**
+	 * https://github.com/symfony/symfony/blob/4.2/src/Symfony/Component/Process/Process.php#L1249
+	 *
+	 * @return boolean
+	 */
+	private function isTtySupported() {
+		if(isset($this->isTtySupported)) {
+			return $this->isTtySupported;
+		}
+		$this->isTtySupported = (bool) @proc_open('echo 1 >/dev/null', array(array('file', '/dev/tty', 'r'), array('file', '/dev/tty', 'w'), array('file', '/dev/tty', 'w')), $pipes);
+		return $this->isTtySupported;
+	}
 }
