@@ -244,6 +244,30 @@ class Installer {
 	'type' => CONF_TYPE_BOOL,
 	);
 
+	$category = 'Cron JOBS';
+	$settings[$category]['FWJOBS_LOGS'] = array(
+		'description' => _('Enable / disable framework cron jobs logs. If enabled logs will save to ASTLOGDIR/fwjobs.log file'),
+		'name' => _('Enable fwconsloe cron jobs execution logs'),
+		'value' => false,
+		'readonly' => 0,
+		'hidden' => 0,
+		'level' => 1,
+		'emptyok' => 0,
+		'sortorder' => 30,
+		'type' => CONF_TYPE_BOOL
+	);
+
+	$settings[$category]['JOBSRANDOMSLEEP'] = array(
+		'defaultval' =>0,
+		'options' => array(0,30),
+		'readonly' => 0,
+		'hidden' => 0,
+		'level' => 0,
+		'name' => _('FreePBX JOBs Maximum Random sleep'),
+		'description' => _('0 for disabling the Random Sleep And Max sleep you can set is 30'),
+		'type' => CONF_TYPE_INT
+	);
+
 
 	$category = 'System Setup';
 
@@ -265,6 +289,24 @@ class Installer {
 	'type' => CONF_TYPE_BOOL,
 	);
 
+	$settings[$category]['AMPSHOWUPDATESONSSH'] = array(
+	'value' => false,
+	'options' => '',
+	'name' => 'Show Updates on SSH login',
+	'description' => 'Show system and module updates information on SSH login.',
+	'sortorder' => -170,
+	'type' => CONF_TYPE_BOOL,
+	);
+
+	$settings[$category]['AUTOMODULEUPDATESANDRELOAD'] = array(
+	'value' => false,
+	'options' => '',
+	'name' => 'Reload Config After Automatic Module Updates',
+	'description' => 'Do fwconsole reload After automatic module update',
+	'sortorder' => -170,
+	'type' => CONF_TYPE_BOOL,
+	);
+
 	$settings[$category]['LOG_OUT_MESSAGES'] = array(
 	'value' => true,
 	'options' => '',
@@ -499,7 +541,7 @@ class Installer {
 	'value' => true,
 	'options' => '',
 	'name' => 'Browser Stats',
-	'description' => 'Setting this to true will allow the development team to use google analytics to anonymously analyze browser information to help make better development decisions.',
+	'description' => 'Setting this to true will allow the development team to use google analytics to anonymously analyze browser information to help make better development decisions. Setting to false will stop the PBX from using google analytics directly, but this option will not control web contents displayed in iframes.',
 	'type' => CONF_TYPE_BOOL,
 	);
 
@@ -516,7 +558,9 @@ class Installer {
 	'options' => '',
 	'name' => 'Enable Module Signature Checking',
 	'description' => 'Checks to make sure modules and their files are validly signed. Will display a notice on any module page that is not correctly verified.',
+	'readonly' => 1,
 	'type' => CONF_TYPE_BOOL,
+	'level' => 4,
 	);
 
 	$settings[$category]['AMPTRACKENABLE'] = array(
@@ -531,6 +575,7 @@ class Installer {
 	$settings[$category]['REMOTEUNLOCK'] = array(
 	'value' => false,
 	'options' => '',
+	'hidden' => 1,
 	'name' => 'Enable Remote Unlocking',
 	'description' => 'Enabling this option will allow a remote user to automatically authenticate as an admin via use of a one-time key generated by "amportal a genunlockkey"',
 	'level' => 2,
@@ -557,6 +602,17 @@ class Installer {
 	'type' => CONF_TYPE_TEXT,
 	);
 
+	$settings[$category]['FREEPBX_SYSTEM_IDENT_REM_DASHBOARD_HELP'] = array(
+	'value' => 'no', 
+	'options' => '',
+	'readonly' => 0,
+	'hidden' => 1,
+	'name' => 'Remove dashboard help text to change System Identity',
+	'description' => 'This will be used to decide if we need to show help text to change system identity in dashboard or not ',
+	'type' => CONF_TYPE_TEXT,
+	);
+
+
 	$category = 'Dialplan and Operational';
 
 	$settings[$category]['RFC7462'] = array(
@@ -747,7 +803,7 @@ class Installer {
 	'value' => 'app_confbridge',
 	'options' => array('app_meetme', 'app_confbridge'),
 	'name' => 'Conference Room App',
-	'description' => 'The asterisk application to use for conferencing. If only one is compiled into asterisk, FreePBX will auto detect and change this value if set wrong. The app_meetme application is considered "depreciated" and should no longer be used',
+	'description' => _("The asterisk application to use for conferencing. As the app_meetme Asterisk application is deprecated and scheduled for removal in Asterisk 21, it's recommended that it be changed to app_confbridge."),
 	'type' => CONF_TYPE_SELECT,
 	);
 
@@ -782,7 +838,7 @@ class Installer {
 	);
 
 	$settings[$category]['ASTSIPDRIVER'] = array(
-	'value' => 'both',
+	'value' => 'chan_pjsip',
 	'options' => array('both', 'chan_sip', 'chan_pjsip'),
 	'level' => 2,
 	'name' => 'SIP Channel Driver',
@@ -1228,6 +1284,17 @@ class Installer {
 	'type' => CONF_TYPE_BOOL,
 	);
 
+	$settings[$category]['ENABLE_MIRROR_LOG'] = array(
+	'value' => false,
+	'options' => '',
+	'name' => 'Enable Mirror Debug Log',
+	'description' => 'Log the requests and responses from the Mirror server',
+	'readonly' => 1,
+	'hidden' => 1,
+	'level' => 2,
+	'type' => CONF_TYPE_BOOL,
+	);
+
 	$settings[$category]['DISPLAY_MONITOR_TRUNK_FAILURES_FIELD'] = array(
 	'value' => false,
 	'options' => '',
@@ -1833,7 +1900,7 @@ class Installer {
 	'value' => 'no',
 	'options' => array('no', 'yes', 'nonat', 'update'),
 	'name' => 'SIP canrenivite (directmedia)',
-	'description' => 'Default setting for SIP canreinvite (same as directmedia). See Asterisk documentation for details.',
+	'description' => 'Default setting for (new Extension) SIP canreinvite (same as directmedia). See Asterisk documentation for details.',
 	'type' => CONF_TYPE_SELECT,
 	'sortorder' => 20,
 	);
@@ -1851,7 +1918,7 @@ class Installer {
 	'value' => 'yes',
 	'options' => array('no', 'yes'),
 	'name' => 'SIP trustrpid',
-	'description' => 'Default setting for SIP trustrpid. See Asterisk documentation for details.',
+	'description' => 'Default setting for (new Extension) SIP trustrpid. See Asterisk documentation for details.',
 	'type' => CONF_TYPE_SELECT,
 	'sortorder' => 30,
 	);
@@ -1860,16 +1927,16 @@ class Installer {
 	'value' => 'pai',
 	'options' => array('no', 'yes', 'pai'),
 	'name' => 'SIP sendrpid',
-	'description' => "Default setting for SIP sendrpid. A value of 'yes' is equivalent to 'rpid' and will send the 'Remote-Party-ID' header. A value of 'pai' will send the 'P-Asserted-Identity' header. See Asterisk documentation for details.",
+	'description' => "Default setting for (new Extension) SIP sendrpid. A value of 'yes' is equivalent to 'rpid' and will send the 'Remote-Party-ID' header. A value of 'pai' will send the 'P-Asserted-Identity' header. See Asterisk documentation for details.",
 	'type' => CONF_TYPE_SELECT,
 	'sortorder' => 40,
 	);
 
 	$settings[$category]['DEVICE_SIP_NAT'] = array(
-	'value' => 'no',
+	'value' => 'yes',
 	'options' => array('no', 'yes', 'never', 'route'),
 	'name' => 'SIP nat',
-	'description' => "Default setting for SIP nat. A 'yes' will attempt to handle nat, also works for local (uses the network ports and address instead of the reported ports), 'no' follows the protocol, 'never' tries to block it, no RFC3581, 'route' ignores the rport information. See Asterisk documentation for details.",
+	'description' => "Default setting for (new Extension) SIP nat. A 'yes' will attempt to handle nat, also works for local (uses the network ports and address instead of the reported ports), 'no' follows the protocol, 'never' tries to block it, no RFC3581, 'route' ignores the rport information. See Asterisk documentation for details.",
 	'type' => CONF_TYPE_SELECT,
 	'sortorder' => 50,
 	);
@@ -1878,7 +1945,7 @@ class Installer {
 	'value' => 'no',
 	'options' => array('no', 'yes'),
 	'name' => 'SIP encryption',
-	'description' => "Default setting for SIP encryption. Whether to offer SRTP encrypted media (and only SRTP encrypted media) on outgoing calls to a peer. Calls will fail with HANGUPCAUSE=58 if the peer does not support SRTP. See Asterisk documentation for details.",
+	'description' => "Default setting for (new Extension)  SIP encryption. Whether to offer SRTP encrypted media (and only SRTP encrypted media) on outgoing calls to a peer. Calls will fail with HANGUPCAUSE=58 if the peer does not support SRTP. See Asterisk documentation for details.",
 	'type' => CONF_TYPE_SELECT,
 	'sortorder' => 60,
 	);
@@ -1887,7 +1954,7 @@ class Installer {
 	'value' => 60,
 	'options' => array(15, 86400),
 	'name' => 'SIP qualifyfreq',
-	'description' => "Default setting for SIP qualifyfreq. Only valid for Asterisk 1.6 and above. Frequency that 'qualify' OPTIONS messages will be sent to the device. Can help to keep NAT holes open but not dependable for remote client firewalls. See Asterisk documentation for details.",
+	'description' => "Default setting for (new Extension) SIP qualifyfreq. Only valid for Asterisk 1.6 and above. Frequency that 'qualify' OPTIONS messages will be sent to the device. Can help to keep NAT holes open but not dependable for remote client firewalls. See Asterisk documentation for details.",
 	'type' => CONF_TYPE_INT,
 	'sortorder' => 70,
 	);
@@ -1896,7 +1963,7 @@ class Installer {
 	'value' => 'yes',
 	'options' => '',
 	'name' => 'SIP and IAX qualify',
-	'description' => "Default setting for SIP and IAX qualify. Whether to send periodic OPTIONS messages (for SIP) or otherwise monitor the channel, and at what point to consider the channel unavailable. A value of 'yes' is equivalent to 2000, time in msec. Can help to keep NAT holes open with SIP but not dependable for remote client firewalls. See Asterisk documentation for details.",
+	'description' => "Default setting for (new Extension) SIP and IAX qualify. Whether to send periodic OPTIONS messages (for SIP) or otherwise monitor the channel, and at what point to consider the channel unavailable. A value of 'yes' is equivalent to 2000, time in msec. Can help to keep NAT holes open with SIP but not dependable for remote client firewalls. See Asterisk documentation for details.",
 	'type' => CONF_TYPE_TEXT,
 	'sortorder' => 80,
 	);
@@ -1905,7 +1972,7 @@ class Installer {
 	'value' => '',
 	'options' => '',
 	'name' => 'SIP and IAX disallow',
-	'description' => "Default setting for SIP and IAX disallow (for codecs). Codecs to disallow, can help to reset from the general settings by setting a value of 'all' and then specifically including allowed codecs with the 'allow' directive. Values van be separated with '&' e.g. 'g729&g722'. See Asterisk documentation for details.",
+	'description' => "Default setting for (new Extension) SIP and IAX disallow (for codecs). Codecs to disallow, can help to reset from the general settings by setting a value of 'all' and then specifically including allowed codecs with the 'allow' directive. Values van be separated with '&' e.g. 'g729&g722'. See Asterisk documentation for details.",
 	'type' => CONF_TYPE_TEXT,
 	'emptyok' => 1,
 	'sortorder' => 90,
@@ -1915,7 +1982,7 @@ class Installer {
 	'value' => '',
 	'options' => '',
 	'name' => 'SIP and IAX allow',
-	'description' => "Default setting for SIP and IAX allow (for codecs). Codecs to allow in addition to those set in general settings unless explicitly 'disallowed' for the device. Values van be separated with '&' e.g. 'ulaw&g729&g729' where the preference order is preserved. See Asterisk documentation for details.",
+	'description' => "Default setting for (new Extension) SIP and IAX allow (for codecs). Codecs to allow in addition to those set in general settings unless explicitly 'disallowed' for the device. Values van be separated with '&' e.g. 'ulaw&g729&g729' where the preference order is preserved. See Asterisk documentation for details.",
 	'type' => CONF_TYPE_TEXT,
 	'emptyok' => 1,
 	'sortorder' => 90,
@@ -1925,7 +1992,7 @@ class Installer {
 	'value' => '',
 	'options' => '',
 	'name' => 'SIP and DAHDi callgroup',
-	'description' => "Default setting for SIP, DAHDi (and Zap) callgroup. Callgroup(s) that the device is part of, can be one or more callgroups, e.g. '1,3-5' would be in groups 1,3,4,5. See Asterisk documentation for details.",
+	'description' => "Default setting for (new Extension) SIP, DAHDi (and Zap) callgroup. Callgroup(s) that the device is part of, can be one or more callgroups, e.g. '1,3-5' would be in groups 1,3,4,5. See Asterisk documentation for details.",
 	'type' => CONF_TYPE_TEXT,
 	'emptyok' => 1,
 	'sortorder' => 100,
@@ -1935,7 +2002,7 @@ class Installer {
 	'value' => '',
 	'options' => '',
 	'name' => 'SIP and DAHDi pickupgroup',
-	'description' => "Default setting for SIP, DAHDi (and Zap) pickupgroup. Pickupgroups(s) that the device can pickup calls from, can be one or more groups, e.g. '1,3-5' would be in groups 1,3,4,5. Device does not have to be in a group to be able to pickup calls from that group. See Asterisk documentation for details.",
+	'description' => "Default setting for (new Extension) SIP, DAHDi (and Zap) pickupgroup. Pickupgroups(s) that the device can pickup calls from, can be one or more groups, e.g. '1,3-5' would be in groups 1,3,4,5. Device does not have to be in a group to be able to pickup calls from that group. See Asterisk documentation for details.",
 	'type' => CONF_TYPE_TEXT,
 	'emptyok' => 1,
 	'sortorder' => 110,
@@ -2095,6 +2162,9 @@ class Installer {
 		'type' => CONF_TYPE_TEXT
 	);
 
+
+	
+
 	// The following settings are used in various modules prior to 2.9. If they are found in amportal.conf then we
 	// retain their values until the individual modules are updated and their install scripts run where a full
 	// configuration (descriptions, defaults, etc.) will be provided and maintained. This provides just enough to
